package com.fmi.weprom.impl.mysql;

import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.PERMISSIONS_TABLE;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.Permission;
import com.fmi.weprom.util.WepromRequestCycleUtil;

public class PermissionImpl extends Permission {
  
  private static final Log LOG = LogFactory.getLog(PermissionImpl.class);
  private static final String PERMISSION_CACHE = "PermissionCache";
  private static final String PERMISSION_ID_CACHE = "PermissionIDCache";
  
  
  public PermissionImpl(int id, String name) {
    super(id, name);
  }
  
  private static PermissionImpl getCachedPermission(String name) {
    //LOG.debug("getCachedPermission("+name+")");
    Map permissionCache = Util.currentSession().getCache(PERMISSION_CACHE);
    return (PermissionImpl) permissionCache.get(name);
  }

  private static PermissionImpl getCachedPermission(int id) {
    //LOG.debug("getCachedPermission("+id+")");
    Map permissionIDCache = Util.currentSession().getCache(PERMISSION_ID_CACHE);
    return (PermissionImpl) permissionIDCache.get(new Integer(id));
  }
  
  private static void cache(Permission p) {
    LOG.debug("cache : "+p);
    Map permissionCache = Util.currentSession().getCache(PERMISSION_CACHE);
    permissionCache.put(p.getName(), p);
    
    Map permissionIDCache = Util.currentSession().getCache(PERMISSION_ID_CACHE);
    permissionIDCache.put(new Integer(p.getID()), p);
  }
  
  static PermissionImpl getInstance(String name) {
    PermissionImpl p = getCachedPermission(name);
    if (p==null) {
      p = loadPermission(name, 0);
    } else {
      LOG.debug("getInstance(...) loaded from cache : " + p);
    }
    return p;
  }
  
  static PermissionImpl getInstance(int id) {
    PermissionImpl p = getCachedPermission(id);
    if (p==null) {
      p = loadPermission(null, id);
    } else {
      LOG.debug("getInstance(...) loaded from cache : " + p);
    }
    return p;
  }
  
  private static PermissionImpl loadPermission(String name, int id) {
    boolean selectByName = name!=null;
    PermissionImpl p = null;
    /*
    if (selectByName) {
      p = getCachedPermission(name);
    } else {
      p = getCachedPermission(id);
    }
    if (p!=null) {
      return p;
    }
    */
    String _method = null;
    if (selectByName) {
      _method = "loadPermission("+name+") ";
    } else {
      _method = "loadPermission("+id+")";
    }
    LOG.debug(_method);
    Connection conn = null;
    Statement stmt = null;
    ResultSet rs = null;
    try {
      conn = Util.currentSession().getConnection();
      stmt = conn.createStatement();
      
      StringBuilder sb = new StringBuilder();
      if (selectByName) {
        sb.append("SELECT id FROM ").append(PERMISSIONS_TABLE).append(" WHERE name='").append(name).append('\'');
      } else {
        sb.append("SELECT name FROM ").append(PERMISSIONS_TABLE).append(" WHERE id='").append(id).append('\'');
      }
      
      String sql = sb.toString();
      //LOG.debug("loadPermission(...) sql="+sql);
      
      rs = stmt.executeQuery(sql);
      if (rs.next()) {
        if (selectByName) {
          int permission_id = rs.getInt(1);
          LOG.debug(_method+" : " + permission_id);
          p = new PermissionImpl(permission_id, name);          
        } else {
          String _name = rs.getString(1);
          LOG.debug(_method+" : " + _name);
          p = new PermissionImpl(id, _name);
        }
        cache(p);
        return p;
      }
    } catch (SQLException e) {
      LOG.error(_method+" ... FAILED", e);
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }       
      if (stmt!=null) {
        try {
          stmt.close();
        } catch (SQLException ignore) {}
      }
    }  
    return null;
    
  }

  public boolean implies(Permission permission) {
    return id==permission.getID();
  }
}
