package com.fmi.weprom.impl.mysql;

import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.ROLES_TABLE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.ROLE_PERMISSIONS_TABLE;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.Role;
import com.fmi.weprom.util.WepromRequestCycleUtil;

public class RoleImpl extends Role {
  
  private static final Log LOG = LogFactory.getLog(RoleImpl.class);
  private static final String ROLE_CACHE = "RoleCache";
  private static final String ROLE_ID_CACHE = "RoleIDCache";
  private List permissions;
  
  public RoleImpl(int id, String name) {
    super(id, name);
  }
  
  public List getPermissions() {
    LOG.debug("getPermissions() of "+toString());
    if (permissions==null) {
      permissions = loadPermissions(); 
    }
    return permissions;
  }
  
  private List loadPermissions() {
    LOG.debug("loadPermissions() for "+toString());
    List result = null;
    Connection conn = null;
    Statement stmt = null;
    ResultSet rs = null;
    try {
      conn = Util.currentSession().getConnection();
      stmt = conn.createStatement();
      
      StringBuilder sb = new StringBuilder();
      sb.append("SELECT permission_id FROM ").append(ROLE_PERMISSIONS_TABLE).append(" WHERE ");
      sb.append("role_id='").append(id).append('\'');
      
      String sql = sb.toString();
      //LOG.debug("loadPermissions() sql="+sql);
      
      rs = stmt.executeQuery(sql);
      result = new ArrayList();
      while (rs.next()) {
        int permission_id = rs.getInt(1);
        PermissionImpl p = PermissionImpl.getInstance(permission_id);
        result.add(p);
      }
    } catch (SQLException e) {
      LOG.error("loadPermissions() FAILED", e);
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }      
      if (stmt!=null) {
        try {
          stmt.close();
        } catch (SQLException ignore) {}
      }
    }      
    return result;
  }

  private static void cache(Role r) {
    Map roleCache = Util.currentSession().getCache(ROLE_CACHE);
    roleCache.put(r.getName(), r);
    
    Map roleIDCache = Util.currentSession().getCache(ROLE_ID_CACHE);
    roleIDCache.put(new Integer(r.getID()), r);
  }
  
  private static Role getCachedRole(String name) {
    Map roleCache = Util.currentSession().getCache(ROLE_CACHE);
    return (Role) roleCache.get(name);
  }
  
  private static Role getCachedRole(int id) {
    Map roleIDCache = Util.currentSession().getCache(ROLE_ID_CACHE);
    return (Role) roleIDCache.get(new Integer(id));
  } 
  
  static Role getInstance(int id) {
    Role role = getCachedRole(id);
    if (role!=null) {
      LOG.debug("getInstance("+id+") loaded from cache : " + role);
      return role;
    }    
    return loadRole(null, id);
  }
  
  static Role getInstance(String name) {
    Role role = getCachedRole(name);
    if (role!=null) {
      LOG.debug("getInstance("+name+") loaded from cache : " + role);
      return role;
    }
    return loadRole(name, 0);
  }  

  private static Role loadRole(String name, int roleID) {
    boolean selectByName = name!=null;
    String _method = null;
    if (selectByName) {
      _method = "loadRole("+name+") ";
    } else {
      _method = "loadRole("+roleID+") ";
    }
    LOG.debug(_method);
    
    ResultSet rs = null;
    Connection conn = null;
    Statement stmt = null;
    
    StringBuffer sb = new StringBuffer();
     
    if (selectByName) {
      sb.append("SELECT id from ").append(ROLES_TABLE).append(" WHERE name='").append(name).append('\'');
    } else {
      sb.append("SELECT name from ").append(ROLES_TABLE).append(" WHERE id='").append(roleID).append('\'');
    }
    String sql = sb.toString();
    
    //LOG.debug("loadRole sql="+sql);
    try {
      conn = Util.currentSession().getConnection();
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sql);
      Role role = null;
      if (rs.next()) {
        if (selectByName) {
          int _roleID = rs.getInt(1);
          LOG.debug(_method+"id="+_roleID);
          role = new RoleImpl(_roleID, name);
        } else {
          String _name = rs.getString(1);
          LOG.debug(_method+"name="+_name);
          role = new RoleImpl(roleID, _name);
        }
        cache(role);
        return role;
      }
    } catch (SQLException e) {
      LOG.error(_method+"FAILED", e);
    } finally {
      if (stmt!=null) {
        try {
          stmt.close();
        } catch (SQLException ignore) {}
      }
    }  
    return null;
  }
    
  private void writeObject(ObjectOutputStream oos) throws IOException {
    oos.defaultWriteObject();    
    LOG.debug("SERIALIZE: " + toString());    
  }
  
  private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
    ois.defaultReadObject();    
    LOG.debug("DESERIALIZE: " + toString());    
  }  
}
