package com.fmi.weprom.impl.mysql;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.util.CacheManager;
import com.fmi.weprom.util.WepromRequestCycle;
import com.fmi.weprom.util.WepromRequestCycleUtil;

/**
 * Manages the current thread SQL Connection (may be this code should be moved to a ConnectionFactory). 
 * Could be used also to cache the persistant objects for the current session. 
 *
 */
public class Session implements WepromRequestCycle {
  
  private static final Log LOG = LogFactory.getLog(Session.class);
  private Connection conn = null;
  private CacheManager cacheManager;
  private CacheManager applicationSessionScopeCacheManager;
  
  static {
    // Load the JDBC driver
    try {      
      Class.forName("com.mysql.jdbc.Driver");
      LOG.info("MySQL Driver loaded successfuly.");
    } catch (ClassNotFoundException e) {
      LOG.error("MySQL Driver loading FAILED:", e);
    }    
  }
  
  public Connection getConnection() throws SQLException {
    LOG.debug("getConnection()");
    if (conn==null) {
      LOG.debug("getConnection() create");
      String url = "jdbc:mysql://localhost:3306/weprom";
      String user = "weprom";
      String password = "weprom123";
      conn = DriverManager.getConnection(url, user, password);
      
      // uncomment this to improve the overall db access perfromance (together with the commented below label2 
      // conn.setAutoCommit(false);
    }
    return conn;
  }

  private CacheManager getCacheManager() {
    if (cacheManager==null) {
      cacheManager = new CacheManager();
    }
    return cacheManager;
  }
  
  Map getCache(String cacheName) {
    return getCacheManager().getCache(cacheName);
  }
  
  public void close() {
    LOG.debug("close()");
    
    // close SQL connection
    if (conn!=null) {
      try {
        // conn.commit();
        conn.close();
      } catch (SQLException ignore) {}
      conn = null;
    }
    
    if (cacheManager!=null) {
      cacheManager.clearCaches();
      cacheManager = null;
    }
  }

  // we should not have sql transactions during the rendering so this is the place to commit  
  public void beginRender() {
    LOG.debug("beginRender()");
    
    // label2:

    /*
    if (conn!=null) {
      try {
        conn.commit();
      } catch (SQLException e) {
        LOG.error("beginRender() commit failed", e);
      }
    }
    */
  }
  
  private CacheManager getAppSessionCacheManager() {
    if (applicationSessionScopeCacheManager==null) {
      Object ssData = WepromRequestCycleUtil.currentSharedSessionData();
      if (ssData!=null) {
        LOG.debug("getAppSessionCacheManager() exists");
        applicationSessionScopeCacheManager = (CacheManager) ssData;
      } else {
        LOG.debug("getAppSessionCacheManager() create");
        applicationSessionScopeCacheManager = new CacheManager();
        WepromRequestCycleUtil.setSharedSessionData(applicationSessionScopeCacheManager);
      }
    }
    return applicationSessionScopeCacheManager;
  }
}
