package com.fmi.weprom.impl.mysql;

import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.ACCESS_TYPE_PRIVATE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.INVALID_ID;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TABLE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TYPE_PROJECT;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TYPE_USERWORKSPACE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.UNSAVED_ID;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.Calendar;
import com.fmi.weprom.api.Contact;
import com.fmi.weprom.api.File;
import com.fmi.weprom.api.Message;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.UserWorkspace;
import com.fmi.weprom.api.WEPROMException;
import com.fmi.weprom.util.WepromRequestCycleUtil;

public class UserWorkspaceImpl extends WorkspaceImpl implements UserWorkspace {
  
  private static final Log LOG = LogFactory.getLog(UserWorkspaceImpl.class);  

  private static final String CACHE_NAME = "UserWorkspaceCache";
  

  //protected WEPROMSystemImpl weprom;
  protected transient UserImpl owner;
  protected int owner_id = INVALID_ID;
  protected CalendarImpl calendar;
  
  
  public UserWorkspaceImpl(UserImpl owner) {
    super();
    this.owner_id = owner.getID();
    this.id = UNSAVED_ID;
  }
  
  public List getInboxMessages() throws WEPROMException {
    return MessageImpl.loadInboxMessages(this);
  }

  public List getOutboxMessages() throws WEPROMException {
    return MessageImpl.loadOutboxMessages(this);
  }

  public Message createMessage() {
    return new MessageImpl(this);
  }

  public Message getMessage(int id) {
    return MessageImpl.getInstance(this, id);
  }

  public List getContacts() {
    return ContactImpl.loadContacts(getID());
  }

  public Contact getContact(int id) {
    return ContactImpl.getInstance(getID(), id);
  }

  public Contact createContact() {
    return new ContactImpl(getID());
  }

  public List getTasks() {
    return TaskImpl.loadUserTasks(owner_id);
  }

  public Calendar getCalendar() {
    if (calendar==null) {
      calendar = new CalendarImpl(this);
    }
    return calendar;
  }

  public List getUsers() {
    List list = new ArrayList();
    list.add(getOwner());
    return list;
  }

  void save() throws SQLException {
    Statement stmt = Util.currentSession().getConnection().createStatement();
    LOG.debug("save(...) id="+id + " user.id="+owner_id);
    if (id!=UNSAVED_ID) {
      throw new IllegalStateException("UserWorkspace already exists");
    }
    ResultSet rs = null;
    StringBuffer sb = new StringBuffer();
    sb.append("INSERT INTO").append(' ').append(WEPROMSystemImpl.MASTER_TABLE).append(' ');
    sb.append('(');
    sb.append("type").append(',');
    sb.append("owner_id").append(',');
    sb.append("access_type").append(',');
    sb.append("creation_date");
    sb.append(')');
    sb.append(' ').append("VALUES").append('(');
    sb.append('\'').append(MASTER_TYPE_USERWORKSPACE).append('\'').append(',');
    sb.append('\'').append(owner_id).append('\'').append(',');
    sb.append('\'').append(ACCESS_TYPE_PRIVATE).append('\'').append(',');
    sb.append("NOW()");
    sb.append(')');
    String sql = sb.toString();
    LOG.debug("save() "+sql);
      
    stmt.executeUpdate(sql, Statement.RETURN_GENERATED_KEYS);
      
    // get autogenerated user id
    rs = stmt.getGeneratedKeys();
    if (rs.next()) {
      int userWorkspaceID = rs.getInt(1);
      this.id = userWorkspaceID; 
      LOG.debug("save() generated id="+id);
      cache();
    } else {
      LOG.debug("save() FAILED for "+toString());
    }   
  }
  
  void delete() {
    // ...
    
    uncache();
  }

  boolean load() {
    LOG.debug("load()");
    Connection conn = null;
    Statement stmt = null;
    ResultSet rs = null;
    try {
      conn = Util.currentSession().getConnection();
      stmt = conn.createStatement(); 
      String sql = "SELECT id FROM "+MASTER_TABLE+" WHERE owner_id='"+owner_id+"\'";
      //LOG.debug("load() sql="+sql);
      rs = stmt.executeQuery(sql);
      if (rs.next()) {
        id = rs.getInt(1);
        LOG.debug("load() id="+id);
        return true;
      } else {
        LOG.error("load() FAILED: no UserWorkspace found for user with nick="+getOwner().getNick());
      }
    } catch (SQLException e) {
      LOG.error("load() FAILED to load UserWorkspace for user with nick="+getOwner().getNick(), e);
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (stmt!=null) {
        try {
          stmt.close();
        } catch (SQLException ignore) {}
      }      
    }
    return false;
  }
  
  public User getOwner() {
    if (owner==null && owner_id!=INVALID_ID) {
      owner = UserImpl.getInstance(owner_id); 
    }    
    return owner;
  }

  private void uncache() {
    String nick = getOwner().getNick();
    LOG.debug("uncache: " + toString());
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    cache.remove(nick);
  }

  private void cache() {
    String nick = getOwner().getNick();
    LOG.debug("cache: " + toString());
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    cache.put(nick, this);
  }
  
  private static UserWorkspaceImpl getCached(String nick) {
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    UserWorkspaceImpl uw = (UserWorkspaceImpl) cache.get(nick);
    return uw;
  }
  
  static UserWorkspaceImpl getInstance(UserImpl user) {
    String nick = user.getNick();
    LOG.debug("getInstance("+nick+")");
    UserWorkspaceImpl uw = getCached(nick);
    if (uw==null) {      
      uw = new UserWorkspaceImpl(user); 
      if (user.getID()!=UNSAVED_ID) {
        boolean ok = uw.load();
        if (ok) {
          uw.cache();
          return uw;
        }
      }
    } else {
      LOG.debug("obtain cached UserWorkspace object for: " + nick);
      return uw;
    }
    return null;
  }
  
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("UserWorkspace[").append(id).append(',').append(getOwner().getNick()).append(']');
    return sb.toString();
  }
  
  private void writeObject(ObjectOutputStream oos) throws IOException {
    oos.defaultWriteObject();    
    LOG.debug("SERIALIZE: " + toString());    
  }
  
  private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
    ois.defaultReadObject();    
    LOG.debug("DESERIALIZE: " + toString());    
  }

  public int getUnreadMessageCount() throws WEPROMException {
    return MessageImpl.getUnreadMessageCount(this);
  }
  
  int getType() {
    return MASTER_TYPE_USERWORKSPACE;
  }

  int getDefaultStatus() {
    return 0;
  }

}
