package com.fmi.weprom.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.User;
import com.fmi.weprom.api.WEPROMSystem;
//import com.fmi.weprom.impl.mysql.MysqlSession;

public class WepromRequestCycleUtil {
  
  private static final Log LOG = LogFactory.getLog(WepromRequestCycleUtil.class);
  private static String SESSION_FACTORY = "com.fmi.weprom.impl.mysql.SessionFactory";
  private static WepromRequestCycleFactory wrcFactory = null;
  

  //////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////

  private static final ThreadLocal threadLocalUser = new ThreadLocal();

  public static void setUser(User user) {    
    LOG.debug("Caller set to : "+user);
    if (threadLocalUser.get()!=null) {
      throw new IllegalStateException("Caller already set!");
    }
    threadLocalUser.set(user);
  }
  
  public static User currentUser() {
    User user = (User) threadLocalUser.get();
    //LOG.debug("ThreadLocal current user: " + user);
    return user;
  }
  
  private static void clearUser() {
    threadLocalUser.set(null);
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////

  private static final ThreadLocal threadLocalWepromRequestCycle = new ThreadLocal();
    
  public static WepromRequestCycle currentWepromRequestCycle() {    
    //LOG.debug("currentSession()");
    WepromRequestCycle session = (WepromRequestCycle) threadLocalWepromRequestCycle.get();
    
    // Open a new Session, if this Thread has none yet
    if (session == null) {
      session = createWepromRequestCycle(); //this should be thread-safe method
      threadLocalWepromRequestCycle.set(session);
    }
    return session;
  }

  private static void closeWepromRequestCycle() {
    //LOG.debug("closeSession()");
    WepromRequestCycle wrc = (WepromRequestCycle) threadLocalWepromRequestCycle.get();
    threadLocalWepromRequestCycle.set(null);
    if (wrc != null) {
      wrc.close();
    }
  }  
  
  //////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////
  

  private static final ThreadLocal threadLocalSharedSessionData = new ThreadLocal();

  public static void setSharedSessionData(Object ssData) {    
    LOG.debug("SharedSessionData set to : "+ssData);
    threadLocalSharedSessionData.set(ssData);
  }
    
  public static Object currentSharedSessionData() {
    Object ssData = threadLocalSharedSessionData.get();
    //LOG.debug("Current SharedSessionData: " + ssData);
    return ssData;
  } 
  
  private static void clearSharedSessionData() {
    threadLocalSharedSessionData.set(null);
  }

  //////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////
  
  private static WepromRequestCycleFactory getWepromRequestCycleFactory() {
    if (wrcFactory==null) {
      try {
        LOG.info("getWepromRequestCycleFactory() "+"instantiate factory: " + SESSION_FACTORY);
        Class factoryClass = Class.forName(SESSION_FACTORY);
        wrcFactory = (WepromRequestCycleFactory) factoryClass.newInstance();
      } catch (Exception e) {
        LOG.error("getWepromRequestCycleFactory() "+"FAILED to instantiate factory: " + SESSION_FACTORY, e);
      }
    }
    return wrcFactory;
  }
  
  private static WepromRequestCycle createWepromRequestCycle() {
    LOG.debug("createWepromRequestCycle()");
    WepromRequestCycleFactory mlsf = getWepromRequestCycleFactory();
    if (mlsf==null) {
      throw new Error("no WepromRequestCycleFactory available");
    }
    return mlsf.createWepromRequestCycle();
    //return new MysqlSession(); // later may need to create a SessionFactory ...
  }
  
  public static void close() {
    LOG.debug("close()");
    closeWepromRequestCycle();
    clear();
  }
  
  public static void open() {
    LOG.debug("open()");    
    // clear to prevent dirty obejcts usage
    clear();
    
    // currentWepromRequestCycle().open();
  }
  
  public static void beginRender() {
    LOG.debug("beginRender()");    
    WepromRequestCycle wrc = (WepromRequestCycle) threadLocalWepromRequestCycle.get();
    if (wrc!=null) {
      wrc.beginRender();
    }
  }
  
  private static void clear() {
    clearUser();
    clearSharedSessionData();
  }

}

/*

Vseki Tapestry page za koito trqbva da ima logged in user
 trqbva da implementira org.apache.tapestry.event.PageValidateListener
... v tazi vrazka moje da napraim bazov klas primerno:

WEPROMPage

koito shte set-va tekushto lognatiq user kato thread local variable. 

Taka model layer-a shte znae koi e User caller i shte moje lesno da check-va pravata
bez da mu se poda vav vseki method argumenta - User caller
( poneje shte moje da si go zeme lesno prez static methoda Util.currentUser() )

V dopalnenie RequestFilter shte izchistva tozi thread local user osven che go slojihme
za da zatvarq sessiona na model layer, t.e. sql connection-a :

RequestFilter.doFilter(...) {
    ...
    SessionUtil.closeSession();
    Util.setUser(null);
    ...
}


*/