package workspace;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Date;

import com.fmi.weprom.api.Message;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.UserWorkspace;
import com.fmi.weprom.api.WEPROMException;

public class MessageUtil implements Serializable {
  
  public static String getRplyFrwrdNewStatsDisp(Message message, User user) {    
    return getStatus(message.isReplied()) + getStatus(message.isForwarded())
          + getStatus(message.isNew());
  }

  public static String getStatus(boolean status) {
    if (status)
      return "1";
    else
      return "0";
  }
  
  public static Message createMessage(User user, int msgType) {
    Message message = user.getUserWorkspace().createMessage();
    message.setSubject("");
    message.setType(msgType);
    message.setBody("");
    return message;
  }
  
  public static Message forwardMessage(User user, Message message) {
    Message messageForward = user.getUserWorkspace().createMessage();
    messageForward.setSubject("Fwd: " + message.getSubject());
    messageForward.setType(Message.MESSAGE_TYPE_TEXT);

    StringBuffer bodyForwarded = new StringBuffer();
    bodyForwarded.append("\n\n---------- Forwarded message ----------");
    bodyForwarded.append("\n > From: ");
    bodyForwarded.append(user.getLastName() + " "
        + user.getName() + " [ " + user.getNick() + " ]");
    bodyForwarded.append("\n > Date: ");
    bodyForwarded.append(message.getDate());
    bodyForwarded.append("\n > Subject: ");
    bodyForwarded.append(message.getSubject());
    bodyForwarded.append("\n > To: ");
    bodyForwarded.append(user.getNick().toString());
    bodyForwarded.append("\n\n > ");
    String body = message.getBody();
    if(body == null)
      bodyForwarded.append("");
    else
      bodyForwarded.append(body);
    bodyForwarded.append("\n\n---------------------------------------");

    messageForward.setBody(bodyForwarded.toString());
    return messageForward;
  }

  public static Message replyMessage(User user, Message message) {
    Message messageReply = user.getUserWorkspace().createMessage();
    messageReply.setReceiver(message.getSender());
    messageReply.setSubject("Re: " + message.getSubject());
    messageReply.setType(Message.MESSAGE_TYPE_TEXT);

    StringBuffer bodyReplied = new StringBuffer();
    bodyReplied.append("\n\n---------------------------------------");
    bodyReplied.append("\n\n> On ");
    bodyReplied.append(message.getDate());
    bodyReplied.append(", ");
    bodyReplied.append(message.getSender().getNick());
    bodyReplied.append(" wrote: ");
    bodyReplied.append("\n\n> ");
    String body = message.getBody();
    if(body == null)
      bodyReplied.append("");
    else
      bodyReplied.append(body);
    bodyReplied.append("\n\n---------------------------------------");

    messageReply.setBody(bodyReplied.toString());
    return messageReply;
  }
  
  public static Message inviteMessage(User receiver, Message message, String projectName) {
    message.setReceiver(receiver);
    message.setSubject("Invitation to project " + projectName);
    
    StringBuffer body = new StringBuffer();
    body.append(" On ");
    body.append(new Date());
    body.append(", ");
    body.append(message.getSender().getNick());
    body.append(" sends you an invitation to project: " + projectName);
    body.append("\n ");
    body.append(" You can accept or decline the invitation!");
    body.append("\n ");

    message.setBody(body.toString());
    return message;
  }

  public static boolean getInbox(User user, Message message) {
    System.out.println("getInbox curr user " + user );
    System.out.println("getInbox mess receiver " + message.getReceiver());
    if(message.getReceiver() != null)
      if(user.getID()==message.getReceiver().getID())
        return true;
    return false;
  }
  
  public static void addReceivers(List receivers, List receiversVal) {
    for (int i = 0; i < receiversVal.size(); i++) {
      User receiverNew = (User) receiversVal.get(i);
      if (isNewReceiver(receivers, receiverNew))
        receivers.add(receiversVal.get(i));
    }
  }
  
  public static boolean isNewReceiver(List receivers, User receiverNew) {
    for (int i = 0; i < receivers.size(); i++) {
      User receiverOld = (User) receivers.get(i);
      if (receiverNew.getNick() == receiverOld.getNick())
        return false;
    }
    return true;
  }

  public static List sendMessage(UserWorkspace userWorkspace, Message message, 
      List receivers){
    
    User firstReceiver = (User)receivers.get(0);
    message.setReceiver(firstReceiver);
    List msgToSend = new ArrayList();
    msgToSend.add(message);
    
    if(receivers.size() > 1)
      for(int i=1; i<receivers.size(); i++){
        Message messageCopy = userWorkspace.createMessage();
        messageCopy.setSender(userWorkspace.getOwner());
        messageCopy.setReceiver((User)receivers.get(i));
        messageCopy.setType(Message.MESSAGE_TYPE_TEXT);
        messageCopy.setSubject(message.getSubject());
        messageCopy.setBody(message.getBody());
        msgToSend.add(messageCopy);
      }
    List error = new ArrayList();
    for(int i=0; i<msgToSend.size(); i++)
      try{        
        ((Message) msgToSend.get(i)).send();
      }catch(WEPROMException e){
        error.add((Message) msgToSend.get(i));
        System.out.println("msg could not be sent " + (Message) msgToSend.get(i) );
        System.out.println("error " + e );
      }
    return error;
  }
}
