package workspace;

import com.fmi.weprom.api.AccessRights;

import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Reminder;
import com.fmi.weprom.api.Task;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.UserWorkspace;
import com.fmi.weprom.api.WEPROMFactory;
import com.fmi.weprom.api.Workspace;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.Role;
import com.fmi.weprom.api.WEPROMSystem;
import com.fmi.weprom.api.WEPROMException;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Date;

import workspace.web.CalendarDay;
import workspace.web.CalendarMonth;
import workspace.web.DateComparator;
import workspace.web.DateComparatorEEEddMMyyyy;
import workspace.web.PercentageComparator;

public class Visit implements Serializable {

  private String userId;
  private int userWorkspaceID;
  private int projectID;
  private int communityID;
  private WEPROMSystem weprom;
  private CalendarMonth calendarMonth = null;
  public static DateComparator dtComparator = new DateComparator();
  public static DateComparatorEEEddMMyyyy dtComparatorEEEddMMyyyy = new DateComparatorEEEddMMyyyy();
  public static PercentageComparator prcntgComparator = new PercentageComparator();
  Object sharedSessionData = null;

  public Visit() {
  }

  public String getUserId() {
    return userId;
  }

  public int getCommunityID() {
    return communityID;
  }

  public int getProjectID() {
    return projectID;
  }

  public int getUserWorkspaceID() {
    return userWorkspaceID;
  }

  public User getUser() {    
    if (userId == null)
      return null;   
    return getWeprom().getUser(userId);
  }

  public void setUser(User userEpromVal) {
    userId = null;
    if (userEpromVal == null)
      return;
    userId = userEpromVal.getNick();
  }

  public UserWorkspace getUserWorkspace() {
    if (getUser() != null)
      return getUser().getUserWorkspace();
    return null;
  }

  public void setUserWorkspace(UserWorkspace userWorkspace) {
    userWorkspaceID = 0;
    this.calendarMonth = null;
    if (userWorkspace == null)
      return;
    userWorkspaceID = userWorkspace.getID();
  }

  public Project getProject() {
    if (projectID == 0)
      return null;    
    return getWeprom().getProject(projectID);
  }

  public void setProject(Project project) {
    projectID = 0;
    this.calendarMonth = null;
    if (project == null)
      return;
    projectID = project.getID();
  }

  public Community getCommunity() {
    if (communityID == 0)
      return null;
    return getWeprom().getCommunity(communityID);
  }

  public void setCommunity(Community community) {
    communityID = 0;
    this.calendarMonth = null;
    if (community == null)
      return;
    communityID = community.getID();
  }
  
  public Object getSharedSessionData() {
    return sharedSessionData;
  }

  public void setSharedSessionData(Object sharedSessionData) {
    this.sharedSessionData = sharedSessionData;
  }

  public boolean isUserLoggedIn() {
    return userId != null;
  }

  public boolean login(String inputUsername, String inputPass) {
    User loginUser = getWeprom().getUser(inputUsername);
    if (loginUser!=null) 
      if(loginUser.checkPassword(inputPass)){
        clearSessionCache();
        setUser(loginUser);
        setUserWorkspace(loginUser.getUserWorkspace());
        return true;
      }
    return false;
  }

  public boolean logout() {
    if (userId != null || getUser() != null) {
      clearSessionCache();
      return true;
    }
    return false;
  }

  public boolean register(String username, String password, String lastName,
      String firstName, String email, String phone, String information) {

    if (getWeprom().getUser(username)!=null)
      return false;
    User newUser = getWeprom().createUser(username);
    newUser.setPassword(password);
    newUser.setName(firstName);
    newUser.setLastName(lastName);
    newUser.setEmail(email);
    newUser.setInformation(information);
    try {
      newUser.save();
    }catch(WEPROMException wE){
      return false;
    }
    return true;
  }

  public void clearProjectCache() {
    setProject(null);
  }

  public void clearCommunityCache() {
    setCommunity(null);
  }

  public void clearSessionCache() {
    setUser(null);
    setUserWorkspace(null);
    setProject(null);
    setCommunity(null);
  }

  // Used by ContactEditor, Message Editor
  public List getUserList() {
    List userList = getWeprom().getUsers();
    userList.remove(getUser());
    return userList;
  }
  
  public List getUserNotInProjectList(){
    if(getProject() != null)      
      return getAccessRights().getUsersWithoutRoles(getProject());
    return null;
  }
  
  public List getUserProjectList() {
    if (userId != null) {
      return getUser().getProjects();
    }
    return null;
  }
  
  public List getProjectUserList() {
    if (getProject() != null)
      return getProject().getUsers();
    else
      return null;
  }
  public List getWorkspaceUserList(Workspace workspace) {
    if (workspace != null)      
      return workspace.getUsers();
    return null;
  }

  public String getUserRoleInProject() {
    return getUserRoleInProjectStr(getUser());
  }

  public String getUserRoleInProjectStr(User user) {
    if (getProject() != null) {
      List roles = getAccessRights().getUserRoles(user, getProject());
      StringBuffer buffer = new StringBuffer();
      int count = roles.size();
      int x = 0;
      Iterator i = roles.iterator();
      while (i.hasNext()) {
        Role role = (Role) i.next();
        if(!role.getName().equals(Role.USER)){
          if (++x > 1)
            if (x > count)
              buffer.append("");
            else
              buffer.append(", ");               
          buffer.append(role.getName());
        }
      }
      return buffer.toString();
    } else
      return "";
  }
  
  public List getUserRoleInProject(User userVal) {
    if (getProject() != null)
      return getAccessRights().getUserRoles(userVal, getProject());
    else
      return null;
  }

  public List getRoles() {
    List roles = new ArrayList();
    if(getProject()!=null){
      roles.add(getAccessRights().getRole(Role.PROJECT_COORDINATOR));
      roles.add(getAccessRights().getRole(Role.PROJECT_PARTICIPANT));
    }
    else if(getCommunity()!=null){
      roles.add(getAccessRights().getRole(Role.COMMUNITY_COORDINATOR));
      roles.add(getAccessRights().getRole(Role.COMMUNITY_PARTICIPANT));
    }
    return roles;
  }

  public boolean projectSave(Project project) {
    if (checkUserPermission(Permission.CREATE_PROJECT)) 
      try{  
        project.save();
        return true;
      }catch(WEPROMException wE){
        return false;
      }
    return false;
  }

  public boolean projectUpdate(Project project) {
    if (checkUserPermission(Permission.UPDATE_PROJECT)& project != null) {
      try{  
        project.update();
        return true;
      }catch(WEPROMException wE){
        return false;
      }
    }
    return false;
  }

  public boolean projectDelete(Project project) {
    if (checkUserPermission(Permission.DELETE_PROJECT)) 
      try{  
        project.delete();
        return true;
      }catch(WEPROMException wE){
        return false;
      }    
    return false;
  }

  public void removeUserFromProject(User userVal, Project project) {
    if (checkUserPermission(Permission.PROJECT_USER_EDIT_ROLES))
       getAccessRights().deleteUserRoles(userVal, project);
  }

  public boolean assignUserRolesInProject(User userToAssign, Project project,
      List roles) {
    if (checkUserPermission(Permission.PROJECT_USER_EDIT_ROLES)) {
        try{
          getAccessRights().grantUserRoles(userToAssign, roles, project);
        }catch(WEPROMException wE){
          return false;  
        }
      return true;
    } else
      return false;
  }

  public boolean deleteUserRolesInProject(User userToAssign, Project project,
      List roles) {
    if (checkUserPermission(Permission.PROJECT_USER_EDIT_ROLES)) {
          getAccessRights().deleteUserRoles(userToAssign, roles, project);
      return true;
    } else
      return false;
  }

  public boolean checkUserPermission(String permissionName) {
    Permission permission = getAccessRights().getPermission(permissionName);
    if(getProject() != null)
    return getAccessRights()
        .checkPermission(getUser(), getProject(), permission);
    else if(getCommunity() != null)
      return getAccessRights()
        .checkPermission(getUser(), getCommunity(), permission);
    else
      return getAccessRights()
        .checkPermission(getUser(), getUserWorkspace(), permission);
  }

  // Community
  public List getUserNotInCommunityList(){
    if (getCommunity() != null)      
      return getAccessRights().getUsersWithoutRoles(getCommunity());
    return null;
  }
  
  public List getUserCommunityList() {
    if (userId != null) 
      return getUser().getCommunities();
    return null;
  }

  public List getCommunityUserList() {
    if (getCommunity() != null)
      return getCommunity().getUsers();
    else
      return null;
  }
  
  public String getUserRoleInCommunity() {
    return getUserRoleInCommunityStr(getUser());
  }

  public String getUserRoleInCommunityStr(User user) {
    if (getCommunity() != null) {
      List roles = getAccessRights().getUserRoles(user, getCommunity());
      StringBuffer buffer = new StringBuffer();
      int count = roles.size();
      int x = 0;
      Iterator i = roles.iterator();
      while (i.hasNext()) {
        Role role = (Role) i.next();
        if(!role.getName().equals(Role.USER)){
         if (++x > 1)
            if (x > count)
              buffer.append("");
            else
              buffer.append(", ");
          buffer.append(role.getName());
        }
      }
      return buffer.toString();
    } else
      return "";
  }

  public List getUserRoleInCommunity(User userVal) {
    if (getCommunity() != null)
      return getAccessRights().getUserRoles(userVal, getCommunity());
    else
      return null;
  }

  public boolean communitySave(Community community) {
    if (checkUserPermission(Permission.CREATE_COMMUNITY)) 
      try{  
        community.save();
        return true;
      }catch(WEPROMException wE){
        return false;
      }    
    return false;
  }

  public boolean communityUpdate(Community community) {
    if (checkUserPermission(Permission.UPDATE_COMMUNITY) & community != null) 
      try{  
        community.update();
        return true;
      }catch(WEPROMException wE){
        return false;
      }    
    return false;
  }

  public boolean communityDelete(Community community) {
    if (checkUserPermission(Permission.DELETE_COMMUNITY)) 
      try{  
        community.delete();
        return true;
      }catch(WEPROMException wE){
        return false;
      }    
    return false;
  }

  public void removeUserFromCommunity(User userVal, Community community) {
    if (checkUserPermission(Permission.COMMUNITY_USER_EDIT_ROLES))
      getAccessRights().deleteUserRoles(userVal, community);
  }

  
  public boolean assignUserRolesInCommunity(User userToAssign, Community community,
      List roles) {
    if (checkUserPermission(Permission.COMMUNITY_USER_EDIT_ROLES)) {
        try{
          getAccessRights().grantUserRoles(userToAssign, roles, community);
        }catch(WEPROMException wE){
          return false;  
        }
      return true;
    } else
      return false;
  }

  public boolean deleteUserRolesInCommunity(User userToAssign, Community community,
      List roles) {
    if (checkUserPermission(Permission.COMMUNITY_USER_EDIT_ROLES)) {
          getAccessRights().deleteUserRoles(userToAssign, roles, community);
      return true;
    } else
      return false;
  }
  
  // Workspace Tasks
  public List getTaskList() {
    if (getProject() == null & getUserWorkspace() != null)
      return getUserWorkspace().getTasks();
    else if (getProject() != null)
      return getProject().getTasks();
    else
      return null;
  }

  public boolean taskSave(Task task) {
    if (checkUserPermission(Permission.CREATE_PROJECT_TASK)) 
      if (getProject() != null)
        try{
          task.save();
          return true;
        }catch(WEPROMException wE){
           return false;
        }
    return false;
  }

  public boolean taskDelete(Task task) {
    if (checkUserPermission(Permission.DELETE_PROJECT_TASK)) 
      if (getProject() != null)
        try{
          task.delete();
          return true;
        }catch(WEPROMException wE){
           return false;
        }
    return false;
  }

  public boolean taskUpdate(Task task) {
    Permission permission = getAccessRights().getPermission(Permission.UPDATE_PROJECT_TASK);
    boolean hasPermission = getAccessRights()
            .checkPermission(getUser(), task.getProject(), permission);
    if (hasPermission&
        getUser().equals(task.getOwner()) || 
        getUser().equals(task.getProcessor())){
      Project project = (getProject() != null)?getProject():task.getProject();
      if (project != null)
        try{
          task.update();
          return true;
        }catch(WEPROMException wE){
           return false;
        }
    }
    return false;
  }

  // Workspace Reminders and Calendar
  public com.fmi.weprom.api.Calendar getCalendar() {
    if (getProject() == null & getCommunity() == null & getUserWorkspace() != null)
      return getUserWorkspace().getCalendar();
    else if (getProject() != null & getCommunity() == null)
      return getProject().getCalendar();
    else if (getCommunity() != null)
      return getCommunity().getCalendar();
    else
      return null;
  }
  
  public List getReminders() {
    com.fmi.weprom.api.Calendar calendar = getCalendar();
    if(calendar != null)
      return calendar.getReminders();
    else
      return null;
  }

  public boolean reminderSave(Reminder reminder, String time) {
    com.fmi.weprom.api.Calendar calendar = getCalendar();
    Date dateVal= DateComparator.setTimeToDate(time, reminder.getDate());
    System.out.println("reminderSave()"+dateVal);
    reminder.setDate(dateVal);
    if(calendar != null)
      try{
        reminder.save();
        return true;
      }catch(WEPROMException wE){
         return false;
      }
    return false;
  }

  public boolean reminderUpdate(Reminder reminder, String time) {
    com.fmi.weprom.api.Calendar calendar = getCalendar();
    Date dateVal= DateComparator.setTimeToDate(time, reminder.getDate());
    reminder.setDate(dateVal);
    if(calendar != null)
      try{
        reminder.update();
        return true;
      }catch(WEPROMException wE){
         return false;
      }
    return false;
  }

  public boolean reminderDelete(Reminder reminder) {
    com.fmi.weprom.api.Calendar calendar = getCalendar();
    if(calendar != null)
      try{
        reminder.delete();
        return true;
      }catch(WEPROMException wE){
         return false;
      }
    return false;
  }

  public CalendarMonth getCalendarMonth(){
    if (getProject() == null & getCommunity() == null & getUserWorkspace() != null)
      return getCalendarMonth(getUserWorkspace());
    else if (getProject() != null & getCommunity() == null)
      return getCalendarMonth(getProject());
    else if (getCommunity() != null)
      return getCalendarMonth(getCommunity());
    else
      return null;    
  }
  
  public CalendarMonth getCalendarMonth(Object workspace) {
    if (calendarMonth == null) {
      Calendar c = GregorianCalendar.getInstance();
      int year = c.get(Calendar.YEAR);
      int month = c.get(Calendar.MONTH);
      int day = c.get(Calendar.DAY_OF_MONTH);
      calendarMonth = new CalendarMonth(workspace, year, month, day);
    }
    return calendarMonth;
  }


  public CalendarDay getWebCalendarDay(Date date) {
    CalendarMonth calendarMonth = getCalendarMonth();
    if(calendarMonth != null)
      return calendarMonth.getDay(date);
    else 
      return null;
  }

  public CalendarDay updateWebCalendarDay(CalendarDay calendarDay) {
    if (getProject() == null & getCommunity() == null & getUserWorkspace() != null)
      return getCalendarMonth(getUserWorkspace()).updateDay(getUserWorkspace(), calendarDay);
    else if (getProject() != null & getCommunity() == null)
      return getCalendarMonth(getProject()).updateDay(getProject(), calendarDay);
    else if (getCommunity() != null)
      return getCalendarMonth(getCommunity()).updateDay(getCommunity(), calendarDay);
    else
      return null; 
  }

  public CalendarDay updateWebCalendarDay(Date date) {
    if (getProject() == null & getCommunity() == null & getUserWorkspace() != null)
      return getCalendarMonth( getUserWorkspace()).updateDay(getUserWorkspace(), date);
    else if (getProject() != null & getCommunity() == null)
      return getCalendarMonth(getProject()).updateDay(getProject(), date);
    else if (getCommunity() != null)
      return getCalendarMonth(getCommunity()).updateDay(getCommunity(), date);
    else
      return null;
  }

  public CalendarDay[][] getWebCalendarMonth() {
    CalendarMonth calendarMonth = getCalendarMonth();
    if(calendarMonth != null)
      return calendarMonth.getCalendarDayList();
    else 
      return null;
  }

  public CalendarDay[][] getWebCalendarMonth(int year, int month, int date) {
    if (getProject() == null & getCommunity() == null & getUserWorkspace() != null)
      return getCalendarMonth(getUserWorkspace()).getCalendarDayList(getUserWorkspace(), year, month, date);
    else if (getProject() != null & getCommunity() == null)
      return getCalendarMonth(getProject()).getCalendarDayList(getProject(), year, month, date);
    else if (getCommunity() != null)
      return getCalendarMonth(getCommunity()).getCalendarDayList(getCommunity(), year, month, date);
    else
      return null;
  }

  public int getYear() {
    CalendarMonth calendarMonth = getCalendarMonth();
    if(calendarMonth != null)
      return calendarMonth.getYear();
    else 
      return 0;
  }

  public int getMonth() {
    CalendarMonth calendarMonth = getCalendarMonth();
    if(calendarMonth != null)
      return calendarMonth.getMonth();
    else 
      return 0;
  }
  
  public WEPROMSystem getWeprom() {
    if (weprom == null)
      weprom = WEPROMFactory.getWEPROMSystem();
    if (weprom==null) {
      System.out.println("Could not load WEPROMSystem.");    
      return null;
    }
    else
      return weprom;
  }

  public AccessRights getAccessRights() {
    return getWeprom().getAccessRights();
  }

}
