package workspace.web.components;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.util.ComponentAddress;

import workspace.Visit;
import workspace.web.DateComparatorEEEddMMyyyy;
import workspace.web.PercentageComparator;
import workspace.web.StatusSelectionModel;
import workspace.web.pages.ProjectWorkspacePage;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Task;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.Workspace;
import com.fmi.weprom.api.Permission;

public abstract class AllProjectList extends BaseComponent implements
    PageRenderListener {

  // Mode constants
  public static final int MODE_VIEW = 3;

  // Fields
  private IPrimaryKeyConvertor m_projectConvertor;
  private StatusSelectionModel statusModel;
  public SimpleDateFormat dateFormat = null;

  // Pairs of getter-setter methods for the ContactList component's properties
  public abstract List getProjectList();
  public abstract void setProjectList(List projectList);

  // Managing the Project Convertor
  public AllProjectList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_projectConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
        Project project = (Project) objValue;
        System.out.println("IPrimaryKeyConvertor: " + project.getName() + "->"
            + project.getID());
        return project.getID() + "";
      }

      public Object getValue(Object objPrimaryKey) {
        String idStr = (String) objPrimaryKey + "";
        List list = getProjectList();
        // Find the item
        int count = list.size();
        for (int i = 0; i < count; i++) {
          Project project = (Project) list.get(i);
          if (idStr.equals(project.getID() + "")) {
            System.out.println("IPrimaryKeyConvertor: " + idStr + "->"
                + project.getName());
            return project;
          }
        }
        return null; 
      }
    };
  }

  public IPrimaryKeyConvertor getProjectConvertor() {
    return m_projectConvertor;
  }

  // Managing the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }

  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnStatusDisp = new SimpleTableColumn("statusDisp",
          "", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getStatDisp(((Project) objRow));
            }
          }, false);
      // The following overrides the default SimpleTableColumn.getColumnValue to
      // render the Block component, defined on the HTML template
      columnStatusDisp.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("statusDispColumnValue"))));

      SimpleTableColumn columnName = new SimpleTableColumn("name", "Name",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getProjectName(((Project) objRow));
            }
          }, true);

      SimpleTableColumn columnStatus = new SimpleTableColumn("projStatus",
          "Status", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getStatusDisplay((Project) objRow);
            }
          }, true);

      SimpleTableColumn columnProgress = new SimpleTableColumn("progress",
          "Progress", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((Project) objRow).getCompleted() + " %";
            }
          }, true);
      columnProgress.setColumnComparator(Visit.prcntgComparator);

      columns.put(columnStatusDisp.getColumnName(), columnStatusDisp);
      columns.put(columnName.getColumnName(), columnName);
      columns.put(getStartDateColumn().getColumnName(), getStartDateColumn());
      columns.put(getEndDateColumn().getColumnName(), getEndDateColumn());
      columns.put(columnStatus.getColumnName(), columnStatus);
      columns.put(columnProgress.getColumnName(), columnProgress);
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }
  }

  public ITableColumn getStartDateColumn() {
    SimpleTableColumn columnStartDate = new SimpleTableColumn("startDate",
        "Start date", new ITableColumnEvaluator() {
          public Object getColumnValue(ITableColumn objColumn, Object objRow) {
            return getProjectDate(((Project) objRow).getStartDate());
          }
        }, true);
    columnStartDate.setColumnComparator(Visit.dtComparatorEEEddMMyyyy);
    return columnStartDate;
  }

  public ITableColumn getEndDateColumn() {
    SimpleTableColumn columnEndDate = new SimpleTableColumn("endDate",
        "End date", new ITableColumnEvaluator() {
          public Object getColumnValue(ITableColumn objColumn, Object objRow) {
            return getProjectDate(((Project) objRow).getEndDate());
          }
        }, true);
    columnEndDate.setColumnComparator(Visit.dtComparatorEEEddMMyyyy);
    return columnEndDate;
  }

  // Formatting the task list display
  public String getStatDisp(Project project) {
    int status = project.getStatus();
    String res = "3";
    switch(status){
    case 0: 
      res = "0";
      break;
    case 1:
      res = "1";
      break;
    case 2:
      res = "3";
      break;
    case 3:
      res = "2";
      break;
    }
    return res;
  }

  public String getStatDisp(int projectID) {
    Project project = getVisit().getWeprom().getProject(projectID);
    return getStatDisp(project);
  }

  public String getStatusDisplay(Project project){
    return getStatusModel().getStatDisplay(project.getStatus());
  }
  
  public String getProjectDate(Date projDate) {
    if (projDate != null) {
      return getDateFormat().format(projDate);
    }
    return "";
  }

  public String getProjectName(Project project) {
    String name = project.getName();
    if (name.length() > 38) {
      return name.substring(0, 36) + "...";
    } else
      return name;
  }

  public DateFormat getDateFormat() {
    if (dateFormat == null) {
      dateFormat = new SimpleDateFormat("EEE dd.MM.yy", Locale.US);
    }
    return dateFormat;
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    List projectList = getVisit().getWeprom().getProjects();
    setProjectList(projectList);
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public StatusSelectionModel getStatusModel() {
    if(statusModel == null)
      statusModel = new StatusSelectionModel(StatusSelectionModel.PROJECT);    
    return statusModel;
  }

}
