package workspace.web.components;

import java.text.SimpleDateFormat;
//import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.ArrayList;
import java.util.List;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;

//import database.MySQLConnector;
import workspace.Visit;
import workspace.web.CalendarDay;
import workspace.web.pages.CalendarDayListPage;
import workspace.web.pages.ReminderCreatorPage;

import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Reminder;
import com.fmi.weprom.api.User;

public abstract class Calendar extends BaseComponent implements
    PageRenderListener {

  // Month constants
  public static final String JANUARY = "January";
  public static final String FEBRUARY = "February";
  public static final String MARCH = "March";
  public static final String APRIL = "April";
  public static final String MAY = "May";
  public static final String JUNE = "June";
  public static final String JULY = "July";
  public static final String AUGUST = "August";
  public static final String SEPTEMBER = "September";
  public static final String OCTOBER = "October";
  public static final String NOVEMBER = "November";
  public static final String DECEMBER = "December";

  public static final String[] MONTH_MODEL_ARR = { JANUARY, FEBRUARY, MARCH,
      APRIL, MAY, JUNE, JULY, AUGUST, SEPTEMBER, OCTOBER, NOVEMBER, DECEMBER };

  public static final IPropertySelectionModel MONTH_MODEL = new StringPropertySelectionModel(
      MONTH_MODEL_ARR);

  // Year constants
  public static final String Y_2000 = "2000";
  public static final String Y_2001 = "2001";
  public static final String Y_2002 = "2002";
  public static final String Y_2003 = "2003";
  public static final String Y_2004 = "2004";
  public static final String Y_2005 = "2005";
  public static final String Y_2006 = "2006";
  public static final String Y_2007 = "2007";
  public static final String Y_2008 = "2008";
  public static final String Y_2009 = "2009";
  public static final String Y_2010 = "2010";

  public static final String YEAR_MODEL_ARR[] = { Y_2000, Y_2001, Y_2002,
      Y_2003, Y_2004, Y_2005, Y_2006, Y_2007, Y_2008, Y_2009, Y_2010 };

  public static final IPropertySelectionModel YEAR_MODEL = new StringPropertySelectionModel(
      YEAR_MODEL_ARR);

  // Mode constants
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  
  // Calendar component's properties
  public abstract CalendarDay[][] getReminderList();
  public abstract void setReminderList(CalendarDay[][] reminderList);

  public abstract String getMonth();
  public abstract void setMonth(String month);

  public abstract String getYear();
  public abstract void setYear(String year);

  public abstract ArrayList getNmbrWeeks();
  public abstract void setNmbrWeeks(ArrayList weeks);

  public abstract ArrayList getNmbrDays7();
  public abstract void setNmbrDays7(ArrayList days);

  public abstract int getIndex_i();
  public abstract int getIndex_j();

  public abstract int getDeltaMonth();
  public abstract void setDeltaMonth(int delta);

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    if (!event.getRequestCycle().isRewinding()) {
      System.out.println("Reminder List: pageBeginRender: >--->");
      System.out.println("initializing: ---");
      CalendarDay[][] reminderList = getVisit().getWebCalendarMonth();
      setYear("" + getVisit().getYear());
      setMonth("" + MONTH_MODEL_ARR[getVisit().getMonth()]);
      setReminderList(reminderList);
      setNmbrDays7(buildNmbrDays7());
      setNmbrWeeks(buildNmbrWeeks(reminderList[0].length));
      System.out.println("new weeks' number: " + reminderList[0].length);
      System.out.println("Reminder List: pageBeginRender: <---<");
      setDeltaMonth(0);
    }
  }

  // Listeners
  public void viewCalendarDay(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters != null && parameters.length > 0) {
      Integer index_i = (Integer) parameters[0];
      Integer index_j = (Integer) parameters[1];
      if (index_i != null & index_j != null) {
        int i = index_i.intValue();
        int j = index_j.intValue();
        CalendarDay[][] reminderList = (CalendarDay[][]) getReminderList();
        CalendarDay day = reminderList[i][j];
        if (day.hasReminders()) {
          CalendarDayListPage calendarDayListPage = (CalendarDayListPage) cycle
              .getPage("CalendarDayListPage");
          System.out
              .println("Calendar: activate day " + day.getDisplayId());
          calendarDayListPage.setDay(day);
          calendarDayListPage.setIsAnotherDay(true);
          cycle.activate(calendarDayListPage);
        } else {
          Reminder reminder = getVisit().getCalendar().createReminder();
          reminder.setDate(day.getDate());
          ReminderCreatorPage reminderCreatorPage = (ReminderCreatorPage) cycle
              .getPage("ReminderCreatorPage");
          reminderCreatorPage.setReminder(reminder);
          reminderCreatorPage.setMode(MODE_CREATE);
          reminderCreatorPage.setTime(getTime(reminder));
          reminderCreatorPage.setCalendarDay(day);
          cycle.activate(reminderCreatorPage);
        }
      } else
        System.out.println("CalendarDayListPage()cannot be activated: week= "
            + index_i + " " + index_j);
    } else
      System.out
          .println("CalendarDayListPage()cannot be activated: parameters= "
              + parameters);
  }

  public void formSubmit(IRequestCycle cycle) {
    if (getDeltaMonth() != 0) {
      // Form is submitted by the arrows
      // Create a new calendar
      java.util.Calendar c = GregorianCalendar.getInstance();
      // Get the currently displayed month and year
      int monthCurr = translateMonth(getMonth());
      int yearCurr = translateYear(getYear());
      // Set the calendar to the 1st day of the current month of the current year
      c.set(yearCurr, monthCurr, 1);
      // Add delta month to calendar
      c.add(java.util.Calendar.MONTH, getDeltaMonth());
      int yearNew = c.get(java.util.Calendar.YEAR);
      int monthNew = c.get(java.util.Calendar.MONTH);
      setMonth("" + MONTH_MODEL_ARR[monthNew]);
      setYear("" + yearNew);
      setDeltaMonth(0);
      CalendarDay[][] reminderList;
      reminderList = getVisit().getWebCalendarMonth(yearNew, monthNew, 1);
      setReminderList(reminderList);
      setNmbrWeeks(buildNmbrWeeks(reminderList[0].length));
    } else {
      // Form is submitted by the comboboxes
      int month = translateMonth(getMonth());
      int year = translateYear(getYear());
      CalendarDay[][] reminderList;
      reminderList = getVisit().getWebCalendarMonth(year, month, 1);
      setReminderList(reminderList);
      setNmbrWeeks(buildNmbrWeeks(reminderList[0].length));
    }
  }

  // Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  // Helper methods for visualizing
  public int translateMonth(String value) {
    int j = 0;
    for (int i = 0; i < MONTH_MODEL_ARR.length; i++)
      if (MONTH_MODEL_ARR[i] == value)
        j = i;
    return j;
  }

  public int translateYear(String value) {
    int j = 0;
    for (int i = 0; i < YEAR_MODEL_ARR.length; i++)
      if (YEAR_MODEL_ARR[i] == value)
        j = Integer.parseInt(value);
    return j;
  }

  public ArrayList buildNmbrWeeks(int nmbrWeeks) {
    ArrayList nmbrWeeksArr = new ArrayList();
    for (int i = 0; i < nmbrWeeks; i++)
      nmbrWeeksArr.add(i, new Integer(i));
    return nmbrWeeksArr;
  }

  public ArrayList buildNmbrDays7() {
    ArrayList nmbrDays7 = new ArrayList();
    for (int i = 0; i < 7; i++)
      nmbrDays7.add(i, new Integer(i));
    return nmbrDays7;
  }

  public String getDayIJDisp() {
    return getReminderList()[getIndex_i()][getIndex_j()].getDisplayId();
  }

  public String getDayIJRemNmbr() {
    return getReminderList()[getIndex_i()][getIndex_j()].getRemNmbr();
  }

  public boolean getDayIJSpam() {
    return getReminderList()[getIndex_i()][getIndex_j()].getSpam();
  }

  public boolean getDayIJRem() {
    return getReminderList()[getIndex_i()][getIndex_j()].hasReminders();
  }

  public boolean isDayIJRem() {
    return getReminderList()[getIndex_i()][getIndex_j()].hasReminders();
  }

  public String getTime(Reminder reminder) {
    SimpleDateFormat formatter = new SimpleDateFormat("HH:mm");
    return formatter.format(reminder.getDate());
  }

  public boolean getHasPermissionCreateReminder(){
    if(getVisit().getProject() != null)
      return getVisit().checkUserPermission(Permission.CREATE_PROJECT_REMINDER);  
    else if(getVisit().getCommunity() != null)
      return getVisit().checkUserPermission(Permission.CREATE_COMMUNITY_REMINDER); 
    else 
      return true;
  }

}
