package workspace.web.components;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Locale;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.util.ComponentAddress;

import workspace.Visit;
import workspace.web.CalendarDay;
import workspace.web.DateComparator;
import workspace.web.pages.ReminderEditorPage;
import com.fmi.weprom.api.Reminder;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.Permission;

public abstract class CalendarDayList extends BaseComponent implements
    PageRenderListener {
  // Mode constants
  public static final int MODE_VIEW = 3;

	// Fields
	private IPrimaryKeyConvertor m_calendarDayConvertor;

	public SimpleDateFormat dateFormat = null;
	
  // Pairs of getter-setter methods for: 
	// CalendarDayList component's parameters
	public abstract CalendarDay getDay();

	public abstract List getCalendarDayList();

	public abstract void setCalendarDayList(List calendarDayList);
	
  // Managing the CalendarDay Convertor
  public CalendarDayList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_calendarDayConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
          Reminder reminder = (Reminder) objValue;
          return reminder.getID() + "";
      }

      public Object getValue(Object objPrimaryKey) {
          String prKey = (String) objPrimaryKey;
          List list = getCalendarDayList();
          // Find the item
          int count = list.size();
          for (int i = 0; i < count; i++) {
          	Reminder reminder = (Reminder) list.get(i);
              if (prKey.equals(reminder.getID()+"")) {                        
                   return reminder;
              }
          }
          Reminder reminder = getVisit().getCalendar().createReminder();
          reminder.setDate(new Date());
          reminder.setSubject("New Reminder");
          //getVisit().addReminder(reminder, "00:00");
          return reminder; //Handle data item not found
      }
    };
  }
  
  public IPrimaryKeyConvertor getCalendarDayConvertor() {
      return m_calendarDayConvertor;
  }
  
  // Manage the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }
  
  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnStatus = new SimpleTableColumn("status", "",
              new ITableColumnEvaluator() {       			
                  public Object getColumnValue(ITableColumn objColumn, Object objRow) {
                      return getStatusDisp(((Reminder) objRow).getID());
                  }
              }, false);
      // The following overrides the default SimpleTableColumn.getColumnValue to
      // render the Block component, defined on the HTML template
      columnStatus.setValueRendererSource(new BlockTableRendererSource(
      				new ComponentAddress(getComponent("statusColumnValue"))));
      
      SimpleTableColumn columnSubject = new SimpleTableColumn("remSubject",
          "Subject", new ITableColumnEvaluator() {
                  public Object getColumnValue(ITableColumn objColumn, Object objRow) {
                      return getReminderSubject((Reminder) objRow);
                  }
              }, true);

      // The following overrides ...
      columnSubject.setValueRendererSource(new BlockTableRendererSource(
      				new ComponentAddress(getComponent("subjectColumnValue"))));
      
      SimpleTableColumn columnComment = new SimpleTableColumn("comment",
          "Description", new ITableColumnEvaluator() {
                  public Object getColumnValue(ITableColumn objColumn, Object objRow) {
                  	return getCommentDisplayId(((Reminder) objRow).getDescription());
                  }
              }, true);

      columns.put(columnStatus.getColumnName(), columnStatus);
      columns.put(getTimeDateColumn().getColumnName(), getTimeDateColumn());
      columns.put(columnSubject.getColumnName(), columnSubject);
      columns.put(columnComment.getColumnName(), columnComment);
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }
  } 
  
  public ITableColumn getTimeDateColumn() {
    SimpleTableColumn columnDate = new SimpleTableColumn("time", "Time",
            new ITableColumnEvaluator() {            			
                public Object getColumnValue(ITableColumn objColumn, Object objRow) {
                	return getTimeDate(((Reminder) objRow));
                }
            }, true);
  	if(getDay()==null){        	
  		columnDate.setColumnComparator(Visit.dtComparator);
  	}
    return columnDate;
  }
 
  //Formatting the reminder list display
  
  public String getStatusDisp(int remId){
    Reminder reminder = getVisit().getUserWorkspace().getCalendar()
        .getReminder(remId);
    System.out.println("reminder status : " + reminder.getStatus());
    return reminder.getStatus()==Reminder.STATUS_ON?"On":"Off";
  }
  
  public String getTimeDate(Reminder reminder) {
    Date remDate = reminder.getDate();
    if (remDate!=null) {
      return getDateFormat().format(remDate);
    }
    return "";
  }  
  
  public DateFormat getDateFormat() {
  	if(getDay()!=null)        	
  		dateFormat = new SimpleDateFormat("HH:mm", Locale.US); 
      else
          dateFormat = new SimpleDateFormat("dd.MM.yyyy kk:mm", Locale.US);   	
      return dateFormat;
  }
  
  public String getCommentDisplayId(String comment){
  	if(comment != null){
      if(comment.length()>40)
    		return comment.substring(0,36) + "...";
    	else
    		return comment;
    } else
      return "";
  }
    
  public String getReminderSubject(Reminder reminder){
  	String subject = reminder.getSubject();
  	if(subject.length()>26){
  		return subject.substring(0,23) + "...";
  	} else
  		return subject;
  }
    
	// Manage rendering
  public void pageBeginRender(PageEvent event){
  	System.out.println("CalendarDay List: pageBeginRender: >--->" );
 
  	if(getDay()!=null)
  		setCalendarDayList(getDay().getReminders()); 
  	else    	
  		setCalendarDayList(getVisit().getReminders());
 		
  	System.out.println("CalendarDay List: pageBeginRender: <---<" );
	}	
	
    // Listeners
  public void viewReminder(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters!=null & parameters.length>0 & checkPermission()) {
      int remId = ((Integer) parameters[0]).intValue();
      Reminder reminder = getVisit().getUserWorkspace().getCalendar()
          .getReminder(remId);
      //Reminder reminder = (Reminder) parameters[0];
      if(reminder!=null){            	
        ReminderEditorPage reminderEditorPage = (ReminderEditorPage) cycle
            .getPage("ReminderEditorPage");
      	reminderEditorPage.setReminder(reminder);
        CalendarDay calDay = getDay();
        if(calDay == null)
          calDay = getVisit().getWebCalendarDay(reminder.getDate());
        
        reminderEditorPage.setCalendarDay(calDay);             
        reminderEditorPage.setMode(MODE_VIEW);
      	reminderEditorPage.setTime(getTime(reminder));
      	cycle.activate(reminderEditorPage);
      }else
        System.out
            .println("ReminderEditorPage()cannot be activated: calendarDay= "
  				+ reminder);
    }else
      System.out
          .println("ReminderEditorPage()cannot be activated: parameters= "
				+ parameters);
  }  
    
  public void formSubmit(IRequestCycle cycle) {
  }

  public boolean checkPermission(){
    Community c = getCommunity();
    Project p = getProject();
    User u = getUser();
    if(p == null & c == null & u != null)
      return true;
    else if(c == null & p != null)
      return getVisit().checkUserPermission(Permission.VIEW_PROJECT_REMINDER);
    else if(c != null)
      return getVisit().checkUserPermission(Permission.VIEW_COMMUNITY_REMINDER);
    else 
      return false;
  }

    //Api methods
	public Visit getVisit() {
	    return (Visit) getPage().getVisit();
	}
    
	public User getUser() {
	    return (User) getVisit().getUser();
	}
	
  public Project getProject() {
    return (Project) getVisit().getProject();
  }

  public Community getCommunity() {
    return (Community) getVisit().getCommunity();
  }

	// Helper methods for visualizing
  public String getTitle() {
  	CalendarDay day = getDay();
  	String title=null;
  	if(day!=null){
  		String[] longDisp = day.getDisplayIdLong();
      title = "On " + longDisp[0] + " you have " + longDisp[1] + " reminder(s)";
      	return title;
  	}else{
      	title = "You have " + getCalendarDayList().size()+ " reminder(s)";
      	return title;
  	}
  }
  
	public String getTime(Reminder reminder){
        SimpleDateFormat formatter = new SimpleDateFormat("HH:mm");        
        return formatter.format(reminder.getDate());
	}

  public boolean getHasPermissionUpdateReminder() {
    if (getVisit().getProject() != null)
      return getVisit().checkUserPermission(Permission.UPDATE_PROJECT_REMINDER);
    else if (getVisit().getCommunity() != null)
      return getVisit().checkUserPermission(
          Permission.UPDATE_COMMUNITY_REMINDER);
    else
      return true;
  }

}
