package workspace.web.components;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;

import workspace.Visit;
import workspace.web.pages.ContactListPage;
import com.fmi.weprom.api.Contact;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.UserWorkspace;
import com.fmi.weprom.api.WEPROMException;


public abstract class ContactEditor extends BaseComponent {
  // Mode constants
  public static final int MODE_VIEW_ALL = 0;
  public static final int MODE_UPDATE = 1;
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  public static final int MODE_DELETE = 4;

  public static String TITLE[] = { "Result", "Update Contact",
      "Create Contact", "View Contact", "Delete Contact" };

  public static String SUCCESS[] = { "", "Your contact has been updated!",
      "Your contact has been created!", "Your contact has been read!",
      "Your contact has been deleted!" };

  public static String FAILURE1[] = { "",
      "Your contact has not been updated! Please try again.",
      "Your contact has not been created! Probably it was empty.",
      "Your contact has not been read! Please try again.",
      "Your contact has not been deleted! Please try again." };

  // Pairs of getters-setters for:
  // Component's parameters
  public abstract Contact getContact();
  public abstract void setContact(Contact contact);

  public abstract int getMode();
  public abstract void setMode(int mode);

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  // Component's non-persistent properties
/*  public abstract User getUserEprom();
  public abstract Set getSelectedUsers();
  public abstract void setSelectedUsers(Set selectedUsers);
*/
  // Component's persistent properties
  public abstract String getResultString();
  public abstract void setResultString(String result);

  // Listeners
  public void updateContact(IRequestCycle cycle) {
    Contact contact = getContact();
    System.out.println("updateContact: --->" + contact.getID());
    setParamProp(MODE_UPDATE, contact, false, "");
  }

  public void deleteContact(IRequestCycle cycle) {
    setMode(MODE_DELETE);
  }

  public void manageContact(IRequestCycle cycle) {
    System.out.println("Contact List: updateContact: --->");
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if (delegate.getHasErrors()) {
      System.out.println("formUpdateContact - has errors");
    } else {
      Contact contact = getContact();
      System.out.println("formUpdateContact - no errors");
      switch (getMode()) {
      case MODE_CREATE:
        try{
          contact.save();
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        }catch(WEPROMException e){
          setResultString(FAILURE1[getMode()]);
        }
        break;
      case MODE_UPDATE:
        try{
          contact.update();
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        }catch(WEPROMException e){
          setResultString(FAILURE1[getMode()]);
        }
        break;
      case MODE_DELETE:
        try{
          contact.delete();
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        }catch(WEPROMException e){
          setResultString(FAILURE1[getMode()]);
        }
        break;
      case MODE_VIEW:
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        ContactListPage contactListPage = (ContactListPage) cycle
            .getPage("ContactListPage");
        cycle.activate(contactListPage);
        break;
      }
    }
    System.out.println("Contact List: updateContact: --->");
  }

  // selectContacts- invoked, when EPROM users are added to the receivers' list
/*  public void selectEpromUser(IRequestCycle cycle) {
    System.out.println("Contact Editor: selectEpromUser: >--->");
    Contact contact = getContact();
    if (getSelectedUsers().isEmpty()) {
      System.out.println("No selected EpromUsers!");
      System.out.println("Contact Editor: selectEpromUser: <---<");
      return;
    }
    Iterator i = getSelectedUsers().iterator();
    User userEprom = (User) i.next();
    String userEpromId = userEprom.getUsrId();
    contact.setUserEprom(userEpromId);
    System.out.println("Set usrId:" + ((Contact) getContact()).getUserEprom());

    setContact((Contact) getContact());
    System.out.println("Set usrId:" + ((Contact) getContact()).getUserEprom());
    System.out.println("Contact Editor: selectEpromUser: <---<");
  }
*/
  public void formSubmit(IRequestCycle cycle) {
  }

  private boolean isNotEmpty(String field) {
    if (field == null)
      return false;
    return field.length() > 0;
  }

/*  private boolean validEpromUser(IValidationDelegate delegate) {
    System.out.println("Contact Editor: validEpromUser: >--->");
    String userEprom = getContact().getUserEprom();
    if (isNotEmpty(userEprom)) {
      System.out.println(userEprom);
      for (int i = 0; i < getUserList().size(); i++) {
        String validUserId = ((User) getUserList().get(i)).getUsrId();
        if (validUserId.equals(userEprom))
          return true;
      }
      IFormComponent inputEpromUser = (IFormComponent) getComponent("inputEpromUser");
      delegate.setFormComponent(inputEpromUser);
      delegate.record("You should specify an existing W-EPROM User!",
          ValidationConstraint.CONSISTENCY);
      System.out.println("Contact Editor: validEpromUser: <---<");
      return false;
    }
    return true;
  }
*/
  // Parameters and properties order: mode, contact, result, resultString
  private void setParamProp(int mode, Contact contact, boolean result,
      String resultString) {
    setResultString(resultString);
    setContact(contact);
    setResult(result);
    setMode(mode);
  }

  // Api Methods
  public List getUserList() {
    if (getInCreateMode())
      return getVisit().getUserList();
    else
      return null;
  }

  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public UserWorkspace getUserWorkspace() {
    return (UserWorkspace) getVisit().getUserWorkspace();
  }

  // Methods for synthetic properties
/*  public void setUserSelected(boolean userSelected) {
    System.out.println("Message Editor: setContactSelected: >--->");
    Set selectedUsers = getSelectedUsers();
    if (selectedUsers == null)
      setSelectedUsers(new HashSet());
    if (userSelected) {
      getSelectedUsers().add(getUserEprom());
    } else {
      getSelectedUsers().remove(getUserEprom());
    }
    System.out.println("Message Editor: setContactSelected: <---<");
  }

  public boolean isUserSelected() {
    if (getInCreateMode()) {
      if (getSelectedUsers() == null) {
        setSelectedUsers(new HashSet());
      }
      return getSelectedUsers().contains(getUserEprom());
    } else
      return false;
  }
*/
  // Helper Methods for visualizing
  public boolean isResult() {
    return getMode() == MODE_VIEW_ALL;
  }

  public String getButtonName() {
    switch (getMode()) {
    case MODE_UPDATE:
      return "update";
    case MODE_CREATE:
      return "create";
    case MODE_VIEW:
      return "ok";
    case MODE_DELETE:
      return "delete";
    }
    return null;
  }

  public String getTitle() {
    int mode = getMode();
    String title = null;
    if (mode >= 0 && mode < 5) {
      title = TITLE[mode];
    } else
      title = "";
    return title;
  }

  public boolean getInputDisabled() {
    return (getMode() == MODE_VIEW || getMode() == MODE_DELETE);
  }

  public boolean getContactToolbarShow() {
    int mode = getMode();
    return (mode == MODE_UPDATE || mode == MODE_VIEW || mode == MODE_DELETE);
  }

  public boolean getContactToolbarButtonDsbl() {
    /*boolean hasPermission;
    hasPermission = getVisit().checkUserPermission(Permission.UPDATE_CONTACT);*/
    return getMode() != MODE_VIEW ;//|| !hasPermission;
  }

  public boolean getInCreateMode() {
    int mode = getMode();
    return (mode == MODE_CREATE);
  }

}
