package workspace.web.components;

import java.util.List;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;

import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.WEPROMException;

import workspace.Visit;
import workspace.web.pages.CalendarDayListPage;
import workspace.web.pages.ProjectEditorPage;
import workspace.web.pages.CommunityEditorPage;
import workspace.web.pages.MessageCreatorPage;
import workspace.web.pages.ReminderCreatorPage;

public abstract class LeftMenu extends BaseComponent {
	private String _pageSubType;
  public abstract void setBProjectClass(String bProjectClass);
  public abstract void setBCommunityClass(String bCommunityClass);
	public abstract void setBUserClass(String bUserClass);
	public abstract void setBTaskClass(String bTaskClass);
	public abstract void setBMessageClass(String bMessageClass);
	public abstract void setBContactClass(String bContactClass);
  public abstract void setBCalendarClass(String bCalendarClass);
  public abstract void setBFolderClass(String bFolderClass);
	
	public void finishLoad() {
		IPage page = getPage();
		String pageName = page.getPageName();
		_pageSubType = page.getSpecification().getProperty("page-sub-type");
		if (_pageSubType == null)
			_pageSubType = pageName;
    setBProjectClass(selectMenuClass("Project", "here"));
    setBCommunityClass(selectMenuClass("Community", "here"));
		setBUserClass(selectMenuClass("User", "here"));
		setBTaskClass(selectMenuClass("Task", "here"));
		setBMessageClass(selectMenuClass("Message", "here"));
		setBContactClass(selectMenuClass("Contact", "here"));
    setBCalendarClass(selectMenuClass("Calendar", "here"));
    setBFolderClass(selectMenuClass("Folder", "here"));
	}
	
	private String selectMenuClass(String subType, String activeTab) {
		return _pageSubType.equals(subType)? activeTab : "";
	}
  public boolean getIsPageSubTypeProject(){
    return _pageSubType.equals("Project")? true : false;     
  }
  public boolean getIsPageSubTypeCommunity(){
    return _pageSubType.equals("Community")? true : false;     
  }
	public boolean getIsPageSubTypeUser(){
		return _pageSubType.equals("User")? true : false;		 
	}
	public boolean getIsPageSubTypeTask(){
		return _pageSubType.equals("Task")? true : false;		 
	}
	public boolean getIsPageSubTypeMessage(){
		return _pageSubType.equals("Message")? true : false;		 
	}
	public boolean getIsPageSubTypeContact(){
		return _pageSubType.equals("Contact")? true : false;		 
	}
  public boolean getIsPageSubTypeCalendar(){
    return _pageSubType.equals("Calendar")? true : false;    
  }
  public boolean getIsPageSubTypeFolder(){
    return _pageSubType.equals("Folder")? true : false;    
  }
  public boolean getIsProjectWorkspace(){
    List roles = ((Visit) getPage().getVisit()).
            getUserRoleInProject(((Visit) getPage().getVisit()).getUser());
    if(((Visit) getPage().getVisit()).getProject()!=null & roles!=null)
      return true;
    else
      return false;
  }  
  public boolean getIsCommunityWorkspace(){
    List roles = ((Visit) getPage().getVisit()).
            getUserRoleInCommunity(((Visit) getPage().getVisit()).getUser());
    if(((Visit) getPage().getVisit()).getCommunity()!=null & roles!=null)
      return true;
    else
      return false;
  }  
  public boolean getHasPermissionUpdateProject(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.UPDATE_PROJECT);    
  }
  public boolean getHasPermissionUpdateCommunity(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.UPDATE_COMMUNITY);    
  }
  public boolean getHasPermissionInviteUserToProj(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.PROJECT_USER_EDIT_ROLES);    
  }
  public boolean getHasPermissionInviteUserToComm(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.COMMUNITY_USER_EDIT_ROLES);    
  }
  public boolean getHasPermissionCreateProjectTask(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.CREATE_PROJECT_TASK);    
  }
  public boolean getHasPermissionAssignProjectTask(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.ASSIGN_TASKS);    
  }
  public boolean getHasPermissionCreateProjectReminder(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.CREATE_PROJECT_REMINDER);    
  }
  public boolean getHasPermissionViewProjectCalendar(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.VIEW_PROJECT_CALENDAR);    
  }
  public boolean getHasPermissionCreateCommunityReminder(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.CREATE_COMMUNITY_REMINDER);    
  }
  public boolean getHasPermissionViewCommunityCalendar(){
    return ((Visit) getPage().getVisit()).checkUserPermission(Permission.VIEW_COMMUNITY_CALENDAR);    
  }
  
  public String getProjectName(){
    Visit visit = (Visit) getPage().getVisit();
    if(visit.isUserLoggedIn()){
        if(visit.getProject()!=null)
          return visit.getProject().getName();
        else 
          return "";
    }
      else return "";
  }

  public String getCommunityName(){
    Visit visit = (Visit) getPage().getVisit();
    if(visit.isUserLoggedIn()){
        if(visit.getCommunity()!=null)
          return visit.getCommunity().getName();
        else 
          return "";
    }
      else return "";
  }

  public boolean getNewMessages(){
    Visit visit = (Visit) getPage().getVisit();
    try{
      System.out.println("msg count: " +
          visit.getUserWorkspace().getUnreadMessageCount());
      return visit.getUserWorkspace().getUnreadMessageCount()>0;
    }catch(WEPROMException e){
      System.out.println("msg count threw exception");
      return false;
    }
  }
  
	public String getUnreadMessageCount(){
    Visit visit = (Visit) getPage().getVisit();
    System.out.println("LeftMenu: getUnreadMessageCount");
    try{
    return visit.getUserWorkspace().getUnreadMessageCount() + "";
    }catch(WEPROMException e){
      System.out.println("msg count threw exception");
      return "0";
    }
  }
  
  public void viewReminders(IRequestCycle cycle){
    CalendarDayListPage calendarDayListPage = (CalendarDayListPage) cycle.getPage("CalendarDayListPage");
    System.out.println("LeftMenu: get My reminders ");
    calendarDayListPage.setDay(null);
    cycle.activate(calendarDayListPage);
  }
  
  public void createMessage(IRequestCycle cycle){
    MessageCreatorPage messageCreatorPage = (MessageCreatorPage) cycle.getPage("MessageCreatorPage");
    System.out.println("LeftMenu: new Message ");
    messageCreatorPage.setMode(MessageCreatorPage.MODE_CREATE);
    cycle.activate(messageCreatorPage);
  }
  
  public void createReminder(IRequestCycle cycle){
    ReminderCreatorPage reminderCreatorPage = (ReminderCreatorPage) cycle.getPage("ReminderCreatorPage");
    System.out.println("LeftMenu: new Reminder ");
    reminderCreatorPage.setMode(ReminderCreatorPage.MODE_CREATE);
    cycle.activate(reminderCreatorPage);
  }
  
  public void updateProject(IRequestCycle cycle){
    ProjectEditorPage projectEditorPage = (ProjectEditorPage) cycle.getPage("ProjectEditorPage");
    System.out.println("LeftMenu: Update Project ");
    projectEditorPage.setProject(((Visit) getPage().getVisit()).getProject());
    projectEditorPage.setMode(ProjectEditor.MODE_VIEW);
    cycle.activate(projectEditorPage);
  }
  
  public void updateCommunity(IRequestCycle cycle){
    CommunityEditorPage communityEditorPage = (CommunityEditorPage) cycle.getPage("CommunityEditorPage");
    System.out.println("LeftMenu: Update Community ");
    communityEditorPage.setCommunity(((Visit) getPage().getVisit()).getCommunity());
    communityEditorPage.setMode(CommunityEditor.MODE_VIEW);
    cycle.activate(communityEditorPage);
  }
  
  public boolean getIsLogIn() {
    Visit visit = (Visit) getPage().getVisit();
    return visit.isUserLoggedIn();
  }


}
