package workspace.web.components;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.lang.reflect.*;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;

import workspace.Visit;
import workspace.MessageUtil;
import workspace.web.WorkspaceSelectionModel;
import workspace.web.pages.MessageInboxPage;
import workspace.web.pages.MessageOutboxPage;

import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Role;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.UserWorkspace;
import com.fmi.weprom.api.Contact;
import com.fmi.weprom.api.Message;
import com.fmi.weprom.api.Workspace;
import com.fmi.weprom.api.WEPROMException;

public abstract class MessageEditor extends BaseComponent implements PageRenderListener {
  // Mode constants
  public static final int MODE_VIEW_ALL = 0;
  public static final int MODE_REPLY = 1;
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  public static final int MODE_DELETE = 4;
  public static final int MODE_FORWARD = 5;

  public static String TITLE[] = { "Success!", "Reply Message", "Send Message",
      "View Message", "Delete Message", "Forward Message" };

  public static String SUCCESS[] = { "", "Your message has been sent!",
      "Your message has been sent!", "Your message has been read!",
      "Your message has been deleted!", "Your message has been forwarded!" ,
      "You have been added to the project!",
      "You have been added to the community!",
      "The task has been successfuly assigned to you!"};

  public static String FAILURE1[] = { "",
      "Your message has not been sent! Please try again.",
      "Your message has not been sent! Please try again.",
      "Your message has not been read! Please try again.",
      "Your message has not been deleted! Please try again.",
      "Your message has not been forwarded! Please try again." ,
      "You have not been added to the project!",
      "You have not been added to the community!",
      "The task has not been assigned to you!"};

  public static String FAILURE2[] = { "",
      "Your message has not been SET replied! Please try again.", "", "", "",
      "Your message has not been SET forwarded! Please try again." };

  // User group constants
  public static final String ALL_USERS = "All Users";
  public static final String FILTER_BY_PROJECTS = "Filter by Projects";
  public static final String FILTER_BY_COMMUNITIES = "Filter by Communities";

  public static final IPropertySelectionModel WORKSPACE_TYPE_MODEL = new StringPropertySelectionModel(
      new String[] { ALL_USERS, FILTER_BY_PROJECTS, FILTER_BY_COMMUNITIES });

  // Box constants
  public static final int OUTBOX = 0;
  public static final int INBOX = 1;

  // Filter receivers combobox contants
  public static final String BY_COMMUNITY = "By Community";
  public static final String BY_PROJECT = "By Project";

  public static final IPropertySelectionModel FILTER_RECEIVERS = new StringPropertySelectionModel(
      new String[] { BY_COMMUNITY, BY_PROJECT });

  // Pairs of getters-setters for:
  // Component's parameters
  public abstract Message getMessage();
  public abstract void setMessage(Message message);

  public abstract int getMode();
  public abstract void setMode(int mode);

  public abstract String getReceiverString();
  public abstract void setReceiverString(String receiverString);

  public abstract List getReceiverList();
  public abstract void setReceiverList(List receiverList);

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  // Component's non-persistent properties
  public abstract User getUser();
  public abstract Set getSelectedUsers();
  public abstract void setSelectedUsers(Set selectedUsers);

  // Component's persistent properties
  public abstract String getResultString();
  public abstract void setResultString(String result);

  public abstract Message getMessageOld();
  public abstract void setMessageOld(Message message);

  public abstract String getWorkspaceTypeFilter();
  public abstract void setWorkspaceTypeFilter(String workspaceTypeFilter);
  
  public abstract Workspace getWorkspaceFilter();
  public abstract void setWorkspaceFilter(Workspace workspaceFilter);
   
  public abstract List getUserList();
  public abstract void setUserList(List userList);

  // Fields
  private WorkspaceSelectionModel workspaceModel;
  
  // Listeners
  // deleteMessage- invoked, when the delete icon on the MessageToolbar is clicked
  public void deleteMessage(IRequestCycle cycle) {
    setMode(MODE_DELETE);
  }

  // replyMessage- invoked, when the reply icon on the MessageToolbar is clicked
  public void replyMessage(IRequestCycle cycle) {
    Message message = getMessage();
    Message messageReply = MessageUtil.replyMessage(getCurrUser(), message);
    String receivers = "";
    List receiverList = new ArrayList();
    if(messageReply.getReceiver() != null){
      receivers = messageReply.getReceiver().getLastName() + " " +
        messageReply.getReceiver().getName()+ 
        " [ "+ messageReply.getReceiver().getNick() + " ]";
      receiverList.add(messageReply.getReceiver());
    }
    setParamProp(MODE_REPLY, messageReply, message, false, "", 
        receivers, receiverList);
  }

  // forwardMessage- invoked, when the forward icon on the MessageToolbar is clicked
  public void forwardMessage(IRequestCycle cycle) {
    Message message = getMessage();
    Message messageForward = MessageUtil.forwardMessage(getCurrUser(), message);
    setParamProp(MODE_FORWARD, messageForward, message, false, "", "", new ArrayList());
  }

  // manageMessage- invoked, when the message is sent/replied/deleted/forwarded/viewed
  public void manageMessage(IRequestCycle cycle) {
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if (!delegate.getHasErrors()) {
      Message message = getMessage();
      Message messageOld = getMessageOld();
      List error;
      int mode = getMode();
      switch (mode) {
      case MODE_CREATE:
        if (!validReceivers(delegate))
          return;
        error = MessageUtil.sendMessage(getUserWorkspace(), message, getReceiverList());
        if(error.size() == 0)          
          setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()], null, null);
        else
          setResultString(FAILURE1[getMode()]);        
        break;
      case MODE_DELETE:
        try{
          message.delete();
          setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()], null, null);
        }catch(WEPROMException e){
          setResultString(FAILURE1[getMode()]);
        }
        break;
      case MODE_REPLY:
        if (!validReceivers(delegate))
          return;
        error = MessageUtil.sendMessage(getUserWorkspace(), message, getReceiverList());
        if(error.size() == 0){          
          setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()], null, null);
          try{
          messageOld.setReplied(true);
          messageOld.update();
          }catch(WEPROMException e){
          }
        } else
          setResultString(FAILURE1[getMode()]);        
        break;
      case MODE_FORWARD:
        if (!validReceivers(delegate))
          return;
        error = MessageUtil.sendMessage(getUserWorkspace(), message, getReceiverList());
        if(error.size() == 0){
          setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()], null, null);
          try{
            messageOld.setForwarded(true);
            messageOld.update();
          }catch(WEPROMException e){
          }
        }else
          setResultString(FAILURE1[getMode()]);        
        break;
      case MODE_VIEW:
        try{
          message.setNew(false);
          message.update();
        }catch(WEPROMException e){          
        }
        setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()], null, null);
        if (MessageUtil.getInbox(getCurrUser(), message)) {
          MessageInboxPage messageBoxPage = (MessageInboxPage) cycle
              .getPage("MessageInboxPage");
          cycle.activate(messageBoxPage);
        } else {
          MessageOutboxPage messageBoxPage = (MessageOutboxPage) cycle
              .getPage("MessageOutboxPage");
          cycle.activate(messageBoxPage);
        }
        break;
      }
    }
  }

  public void accept(IRequestCycle cycle) {
    Message message = getMessage();
    try {
      message.accept();
      String resString = "";
      if(message.getType()==Message.MESSAGE_TYPE_PROJECT_INVITATION)
        resString = SUCCESS[6];
      if(message.getType()==Message.MESSAGE_TYPE_COMMUNITY_INVITATION)
        resString = SUCCESS[7];
      if(message.getType()==Message.MESSAGE_TYPE_TASK_ASSIGNMENT)
        resString = SUCCESS[8];
      setParamProp(MODE_VIEW_ALL, null, null, true, resString, null, null);
    } catch (WEPROMException e) {
      setResultString("Unsuccessful operation!");        
      e.printStackTrace();
    }
  }
  
  public void decline(IRequestCycle cycle) {
    Message message = getMessage();
    try {
      message.decline();
      String resString = "";
      if(message.getType()==Message.MESSAGE_TYPE_PROJECT_INVITATION)
        resString = FAILURE1[6];
      if(message.getType()==Message.MESSAGE_TYPE_COMMUNITY_INVITATION)
        resString = FAILURE1[7];
      if(message.getType()==Message.MESSAGE_TYPE_TASK_ASSIGNMENT)
        resString = FAILURE1[8];
      setParamProp(MODE_VIEW_ALL, null, null, true, resString, null, null);
    } catch (WEPROMException e) {
      setResultString("Unsuccessful operation!");        
      e.printStackTrace();
    }
  }
  
  // selectUsers- invoked, when EPROM users are added to the receivers' list
  public void selectUsers(IRequestCycle cycle) {
    String receivers = getReceiversList();
    List receiversList = new ArrayList();
    if (getSelectedUsers().isEmpty()) {
    } else {
      Iterator i = getSelectedUsers().iterator();
      while (i.hasNext()) {
        User user = (User) i.next();
        receiversList.add(user);
      }
    }
    MessageUtil.addReceivers(getReceiverList(), receiversList);
    String receiversString = getReceiverString();
    if (receiversString == null || receiversString == "")
      setReceiverString(receivers);
    else
      setReceiverString(getReceiverString() + ", " + receivers);
    setMessage((Message) getMessage());
  }

  public void formSubmit(IRequestCycle cycle) {
    String workspaceTypeFilter = getWorkspaceTypeFilter();
    if(workspaceTypeFilter == ALL_USERS){
      setWorkspaceFilter(null);
      setUserList(getVisit().getUserList());  
      setSelectedUsers(null);
    } else if(workspaceTypeFilter == FILTER_BY_PROJECTS ||
        workspaceTypeFilter == FILTER_BY_COMMUNITIES){
      Workspace workspaceFilter = getWorkspaceFilter();
      if(workspaceFilter!= null & workspaceFilter instanceof Workspace)
        setUserList(getVisit().getWorkspaceUserList(workspaceFilter));
      else
        setUserList(null);
      setSelectedUsers(null);
    }
  }

  // Parameters and properties order: mode, message, messageOld, result,
  // resultString, receiverString
  private void setParamProp(int mode, Message message, Message messageOld,
      boolean result, String resultString, String receiverString, List receiverList) {
    setResultString(resultString);
    setReceiverString(receiverString);
    setReceiverList(receiverList);
    setMessageOld(messageOld);
    setMessage(message);
    setResult(result);
    setMode(mode);
  }

  // Validator methods
  private boolean validReceivers(IValidationDelegate delegate) {
    String receiversString = getReceiverString();
    if (receiversString == null || receiversString == "") {
      IFormComponent inputReceiver = (IFormComponent) getComponent("inputReceiver");
      delegate.setFormComponent(inputReceiver);
      delegate.record("You should specify at least one receiver!",
          ValidationConstraint.CONSISTENCY);
      return false;
    } else
      return true;
  }

  // Formatting methods
  private String getReceiversList() {
    if (getSelectedUsers().isEmpty()) {
      return "";
    }
    StringBuffer buffer = new StringBuffer();
    int count = getSelectedUsers().size();
    int x = 0;
    Iterator i = getSelectedUsers().iterator();
    while (i.hasNext()) {
      if (++x > 1) {
        if (x > count)
          buffer.append("");
        else
          buffer.append(", ");
      }
      User user = (User) i.next();
      String displayId = user.getLastName() + " " + user.getName ()+ " [ "+ user.getNick() + " ]"; 
      buffer.append(displayId);
    }
    return buffer.toString();
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event){
    if(!event.getRequestCycle().isRewinding()){
      String userGroupFilter = getWorkspaceTypeFilter();
      if(userGroupFilter==null){
        setWorkspaceTypeFilter(ALL_USERS);
        setWorkspaceFilter(null);
        setUserList(getVisit().getUserList());  
      }
    }
} 

  // Api Methods
  public List getUserFilteredList() {
    if(getUserList()!=null)
      for(int i=0; i<getUserList().size(); i++)
        System.out.println("Filetered users: " + ((User)getUserList().get(i)).getNick());
    return getUserList();  
    }

  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public User getCurrUser() {
    return (User) getVisit().getUser();
  }

  public UserWorkspace getUserWorkspace() {
    return (UserWorkspace) getVisit().getUserWorkspace();
  }

  // Methods for synthetic properties
  public void setUserSelected(boolean userSelected) {
    Set selectedUsers = getSelectedUsers();
    if (selectedUsers == null)
      setSelectedUsers(new HashSet());
    if (userSelected) 
      getSelectedUsers().add(getUser());
    else 
      getSelectedUsers().remove(getUser());
  }

  public boolean isUserSelected() {
    if (getInCreateMode()) {
      if (getSelectedUsers() == null) {
        setSelectedUsers(new HashSet());
      }
      return getSelectedUsers().contains(getUser());
    } else
      return false;
  }

  // Helper Methods for visualizing
  public WorkspaceSelectionModel getWorkspaceModel() {
    String workspaceTypeFilter = getWorkspaceTypeFilter();
    List workspaces = new ArrayList();  
    
    if(workspaceTypeFilter==FILTER_BY_PROJECTS){
        workspaces = getVisit().getWeprom().getProjects();   
        workspaceModel = 
          new WorkspaceSelectionModel(workspaces, WorkspaceSelectionModel.PROJECT);        
    } else if(workspaceTypeFilter==FILTER_BY_COMMUNITIES){
        workspaces = getVisit().getWeprom().getCommunities(); 
        workspaceModel = 
          new WorkspaceSelectionModel(workspaces, WorkspaceSelectionModel.COMMUNITY);
    }
    if(workspaceModel==null)
      workspaceModel = new WorkspaceSelectionModel(workspaces);
    else
      workspaceModel.setWorkspaceList(workspaces);
    
    return workspaceModel;
  }

  public boolean isResult() {
    return getMode() == MODE_VIEW_ALL;
  }

  public String getTitle() {
    int mode = getMode();
    String title = null;
    if (mode >= 0 && mode < 6) {
      title = TITLE[mode];
    } else
      title = "";
    return title;
  }

  public String getButtonName() {
    switch (getMode()) {
    case MODE_FORWARD:
      return "send";
    case MODE_REPLY:
      return "send";
    case MODE_CREATE:
      return "send";
    case MODE_VIEW:
      return "ok";
    case MODE_DELETE:
      return "delete";
    }
    return null;
  }

  public boolean getInputDisabled() {
    return (getMode() == MODE_VIEW || getMode() == MODE_DELETE || 
        getMessage().getType()!=Message.MESSAGE_TYPE_TEXT);
  }
  
  public boolean getDisabledWrkspcFltr() {
    return (getInputDisabled() || getWorkspaceTypeFilter()==ALL_USERS);
  }
  
  public boolean getInCreateMode() {
    int mode = getMode();
    return (mode == MODE_FORWARD || mode == MODE_REPLY || mode == MODE_CREATE);
  }

  public boolean getChooseReceiversVisible() {
    int msgType = getMessage().getType();
    return (msgType == Message.MESSAGE_TYPE_TEXT);
  }

  public boolean getMessageToolbarShow() {
    int mode = getMode();
    return (mode == MODE_FORWARD || mode == MODE_REPLY || 
        mode == MODE_VIEW || mode == MODE_DELETE)&
        getMessage().getType() == Message.MESSAGE_TYPE_TEXT;
  }

  public boolean getMessageToolbarReplyEnbl() {
    return MessageUtil.getInbox(getCurrUser(),getMessage()) & getMode() == MODE_VIEW ;
  }

  public boolean getMessageToolbarForwardDeleteEnbl() {
    return getMode() == MODE_VIEW ;
  }
  public boolean getAcceptDecline(){
    boolean msgTypeNotText = (getMessage().getType() != Message.MESSAGE_TYPE_TEXT)? true : false;
    User receiver = getMessage().getReceiver();
    if(receiver!=null){
      boolean inbox = (receiver.equals(getCurrUser()))? true : false;
      return (msgTypeNotText & inbox & !getMessage().isReplied()) ? true : false;
    } else
      return false;
  }
}
