package workspace.web.components;

import java.util.ArrayList;
import java.util.List;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.form.IFormComponent;

import workspace.Visit;
import workspace.web.StatusSelectionModel;
import workspace.web.pages.ProjectListPage;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.User;

public abstract class ProjectEditor extends BaseComponent {

  // Fields
  private StatusSelectionModel statusModel;
  
  // ProjectEditor mode constans
  public static final int MODE_VIEW_ALL = 0;
  public static final int MODE_UPDATE = 1;
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  public static final int MODE_DELETE = 4;

  // ProjectEditor title constants
  public static String TITLE[] = { "Result", "Update Project",
      "Create Project", "View Project", "Delete Project" };

  // Result strings
  public static String SUCCESS[] = { "", "Your project has been updated!",
      "Your project has been created!", "Your project has been viewed!",
      "Your project has been deleted!" };

  public static String FAILURE1[] = { "",
      "Your project has not been updated! Please try again.",
      "Your project has not been created! Probably the name is already taken.",
      "Your project has not been viewed! Please try again.",
      "Your project has not been deleted! Please try again." };

  // Pairs of getters-setters for:
  // Component's parameters
  public abstract Project getProject();
  public abstract void setProject(Project project);
  public abstract int getMode();
  public abstract void setMode(int mode);

  // Component's persistent properties
  public abstract String getResultString();
  public abstract void setResultString(String result);

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  // Listeners
  public void updateProject(IRequestCycle cycle) {
    Project project = getProject();
    System.out.println("updateProject: --->" + project.getID());
    setParamProp(MODE_UPDATE, project, false, "");
  }

  public void deleteProject(IRequestCycle cycle) {
    setMode(MODE_DELETE);
  }

  public void manageProject(IRequestCycle cycle) {
    System.out.println("Project List: updateProject: --->");
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if (delegate.getHasErrors()) {
      System.out.println("formUpdateProject - has errors");
    } else {
      Project project = getProject();
      System.out.println("formUpdateProject - no errors");
      switch (getMode()) {
      case MODE_CREATE:
        if (!validDates(project, delegate))
          return;
        if(getVisit().projectSave(project))
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        else
          error(delegate, "inputName", "The name "+ project.getName()+
              " is already taken. Please, choose another one.",
              ValidationConstraint.CONSISTENCY);        
        break;
      case MODE_UPDATE:
        if (!validDates(project, delegate))
          return;
        if(getVisit().projectUpdate(project))
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        else
          error(delegate, "inputName", FAILURE1[getMode()],
              ValidationConstraint.CONSISTENCY); 
        break;
      case MODE_DELETE:
        getVisit().projectDelete(project);
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        break;
      case MODE_VIEW:
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        ProjectListPage projectListPage = (ProjectListPage) cycle
            .getPage("ProjectListPage");
        cycle.activate(projectListPage);
        break;
      }
    }
    System.out.println("Project List: updateProject: --->");
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Validate start and end dates
  private boolean validDates(Project project, IValidationDelegate delegate) {
    if (project.getStartDate() == null) {
      error(delegate, "inputStartDate", "Start Date is required.",
          ValidationConstraint.REQUIRED);
      return false;
    }
    System.out.println("End date: " + project.getEndDate());
    if (project.getEndDate() != null)
      if (project.getStartDate().after(project.getEndDate())) {
        error(delegate, "inputEndDate", "End Date must be after Start Date.",
            ValidationConstraint.CONSISTENCY);
        return false;
      }
    return true;
  }

  private void error(IValidationDelegate delegate, String componentId,
      String message, ValidationConstraint constraint) {
    IFormComponent component = (IFormComponent) getComponent(componentId);
    delegate.setFormComponent(component);
    delegate.record(message, constraint);
  }

  // Parameters and properties order: mode, project, result, resultString
  private void setParamProp(int mode, Project project, boolean result,
      String resultString) {
    setResultString(resultString);
    setProject(project);
    setResult(result);
    setMode(mode);
  }

  // Api Methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public User getUser() {
    return (User) getVisit().getUser();
  }

  public Project getCurrProject() {
    return (Project) getVisit().getProject();
  }

  // Helper Methods for visualizing
  public StatusSelectionModel getStatusModel() {
    statusModel = new StatusSelectionModel(StatusSelectionModel.PROJECT);    
    return statusModel;
  }

  public boolean isResult() {
    return getMode() == MODE_VIEW_ALL;
  }

  public String getButtonName() {
    switch (getMode()) {
    case MODE_UPDATE:
      return "update";
    case MODE_CREATE:
      return "create";
    case MODE_VIEW:
      return "ok";
    case MODE_DELETE:
      return "delete";
    }
    return null;
  }

  public String getTitle() {
    int mode = getMode();
    String title = null;
    if (mode >= 0 && mode < 5) {
      title = TITLE[mode];
    } else
      title = "";
    return title;
  }

  public boolean getInputDisabled() {
    return (getMode() == MODE_VIEW || getMode() == MODE_DELETE);
  }

  public boolean getProjectToolbarShow() {
    int mode = getMode();
    return (mode == MODE_UPDATE || mode == MODE_VIEW || mode == MODE_DELETE);
  }

  public boolean getProjectToolbarButtonUpdateDsbl() {
    boolean hasPermission;
    if (getProject() != null)
      hasPermission = getVisit().checkUserPermission(Permission.UPDATE_PROJECT);
    else
      hasPermission = false;
    return getMode() != MODE_VIEW || !hasPermission;
  }

  public boolean getProjectToolbarButtonDeleteDsbl() {
    boolean hasPermission;
    if (getProject() != null)
      hasPermission = getVisit().checkUserPermission(Permission.DELETE_PROJECT);
    else
      hasPermission = false;
    return getMode() != MODE_VIEW || !hasPermission;
  }

  public boolean getInCreateMode() {
    int mode = getMode();
    return (mode == MODE_CREATE);
  }

}
