package workspace.web.components;

import java.util.Date;
import java.lang.NumberFormatException;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;

import workspace.Visit;
import workspace.web.CalendarDay;
import workspace.web.DateComparator;
import workspace.web.pages.CalendarDayListPage;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Reminder;
import com.fmi.weprom.api.Task;
import com.fmi.weprom.api.User;

public abstract class ReminderEditor extends BaseComponent {
  public static final IPropertySelectionModel REMINDER_STATUS_MODEL = new StringPropertySelectionModel(
      new String[] { "On", "Off" });

  // ReminderEditor mode constans
  public static final int MODE_VIEW_ALL = 0;
  public static final int MODE_UPDATE = 1;
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  public static final int MODE_DELETE = 4;

  public static String TITLE[] = { "Result", "Update Reminder",
      "Create Reminder", "View Reminder", "Delete Reminder" };

  public static String SUCCESS[] = { "", "Your reminder has been updated!",
      "Your reminder has been created!", "Your reminder has been read!",
      "Your reminder has been deleted!" };

  public static String FAILURE1[] = { "",
      "Your reminder has not been updated! Please try again.",
      "Your reminder has not been created! Please try again.",
      "Your reminder has not been read! Please try again.",
      "Your reminder has not been deleted! Please try again." };

  // Pairs of getters-setters for:
  // Component's parameters
  public abstract Reminder getReminder();
  public abstract void setReminder(Reminder reminder);

  public abstract int getMode();
  public abstract void setMode(int mode);

  public abstract CalendarDay getCalendarDay();

  public abstract String getTime();
  public abstract void setTime(String time);

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  // Component's persistent properties
  public abstract String getResultString();
  public abstract void setResultString(String result);

  // Listeners
  // deleteReminder- invoked, when the delete icon on the ReminderToolbar is clicked
  public void deleteReminder(IRequestCycle cycle) {
    setMode(MODE_DELETE);
  }

  // updateReminder- invoked, when the update icon on the ReminderToolbar is clicked
  public void updateReminder(IRequestCycle cycle) {
    setParamProp(MODE_UPDATE, getReminder(), getTime(), false, "");
  }

  // manageReminder- invoked, when the reminder is sent/replied/deleted/forwarded/viewed
  public void manageReminder(IRequestCycle cycle) {
    System.out.println("Reminder Editor: manageReminder: >--->");
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if (!delegate.getHasErrors()) {
      Reminder reminder = getReminder();
      int mode = getMode();
      switch (mode) {
      case MODE_CREATE:
        if (!validDates(delegate) || !validTime(delegate))
          return;
        //getVisit().setReminderTime(reminder, getTime());
        if(getVisit().reminderSave(reminder, getTime())){
          updateSetCalendarDay(cycle, getCalendarDay(), reminder.getDate());
          setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()]);
        } else
          setResultString(FAILURE1[getMode()]);
        break;
      case MODE_DELETE:
        if(getVisit().reminderDelete(reminder)){        
          updateSetCalendarDay(cycle, getCalendarDay(), reminder.getDate());
          setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()]);
        } else
          setResultString(FAILURE1[getMode()]);          
        break;
      case MODE_UPDATE:
        if (!validDates(delegate) || !validTime(delegate))
          return;
        //getVisit().setReminderTime(reminder, getTime());
        if(getVisit().reminderUpdate(reminder, getTime())){        
          updateSetCalendarDay(cycle, getCalendarDay(), reminder.getDate());
          setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()]);
        } else
          setResultString(FAILURE1[getMode()]);          
        break;
      case MODE_VIEW:
        setParamProp(MODE_VIEW_ALL, null, null, true, SUCCESS[getMode()]);
        CalendarDayListPage calendarDayListPage = (CalendarDayListPage) cycle
            .getPage("CalendarDayListPage");
        cycle.activate(calendarDayListPage);
        break;
      }
    }
    System.out.println("Reminder Editor: manageReminder: <---<");
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Validate reminder's time
  // Validate date
  private boolean validDates(IValidationDelegate delegate) {
    if (getReminder().getDate() == null) {
      error(delegate, "inputDate", "Date is required.",
          ValidationConstraint.REQUIRED);
      return false;
    }
    return true;
  }
  
  private boolean validTime(IValidationDelegate delegate) {
    String time = getTime();
    if (time == null)
      return true;
    int hour;
    int min;
    String[] hourMin = time.split(":");
    if (hourMin.length != 2 || hourMin[0] == null || hourMin[1] == null
        || hourMin[0].length() != 2 || hourMin[1].length() != 2) {
      error(delegate, "inputTime", 
          "You should specify the time of the reminder in format hh:mm of 24-hour day!",
          ValidationConstraint.CONSISTENCY);
      return false;
    }

    try {
      hour = Integer.parseInt(hourMin[0]);
      min = Integer.parseInt(hourMin[1]);
    } catch (NumberFormatException nfe) {
      error(delegate, "inputTime", 
          "You should specify the time of the reminder in format hh:mm of 24-hour day!",
          ValidationConstraint.CONSISTENCY);
      return false;
    }

    if (hour < 0 || hour > 24 || min < 0 || min > 60) {
      error(delegate, "inputTime", 
          "You should specify the time of the reminder in format hh:mm of 24-hour day!",
          ValidationConstraint.CONSISTENCY);
      return false;
    } else
      return true;
  }
  
  private void error(IValidationDelegate delegate, String componentId,
      String message, ValidationConstraint constraint) {
    IFormComponent component = (IFormComponent) getComponent(componentId);
    delegate.setFormComponent(component);
    delegate.record(message, constraint);
  }

  // As a result of the reminder's updating, the calendar days have to be
  // updated too
  private void updateSetCalendarDay(IRequestCycle cycle, CalendarDay day,
      Date remDate) {
    getVisit().updateWebCalendarDay(day);
    CalendarDay dayNew = null;

    // If the reminder is shifted to another day, update that day's reminders
    if (day != null){
      if (!DateComparator.areDatesEqual(remDate, day.getDate()))
        dayNew = (CalendarDay) getVisit().updateWebCalendarDay(remDate);
    } else
        dayNew = (CalendarDay) getVisit().updateWebCalendarDay(remDate);
    CalendarDayListPage calendarDayListPageU = (CalendarDayListPage) cycle
        .getPage("CalendarDayListPage");

    if (getMode() == MODE_CREATE) {
      // If the reminder is CREATED for the initial date(today), update this
      // day's reminders
      if (dayNew == null)
        dayNew = (CalendarDay) getVisit().updateWebCalendarDay(remDate);
      calendarDayListPageU.setDay(dayNew);

      // If the reminder is created for a day, not listed the the current days list,
      // i.e. for another month or year, dayNew is still null, so check before
      // getting its reminders
      if (dayNew != null)
        calendarDayListPageU.setCalendarDayList(dayNew.getReminders());
    } else {
      calendarDayListPageU.setDay(day);
      // If the reminder for a day, not listed the the current days list,
      // i.e. for another month or year, is updated, dayNew is still null,
      // so check before getting its reminders
      if (day != null)
        calendarDayListPageU.setCalendarDayList(day.getReminders());
    }
  }

  // Parameters and properties order: mode, reminder, time, result, resultString
  private void setParamProp(int mode, Reminder reminder, String time,
      boolean result, String resultString) {
    setResultString(resultString);
    setReminder(reminder);
    setResult(result);
    setTime(time);
    setMode(mode);
  }

  // Api Methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public User getUser() {
    return (User) getVisit().getUser();
  }

  public Project getProject() {
    return (Project) getVisit().getProject();
  }

  public Community getCommunity() {
    return (Community) getVisit().getCommunity();
  }

  public boolean checkPermission(String permProject, String permCommunity){
    Community c = getCommunity();
    Project p = getProject();
    User u = getUser();
    if(p == null & c == null & u != null)
      return true;
    else if(c == null & p != null)
      return getVisit().checkUserPermission(permProject);
    else if(c != null)
      return getVisit().checkUserPermission(permCommunity);
    else 
      return false;
  }
  // Helper Methods for visualizing
  public boolean isResult() {
    return getMode() == MODE_VIEW_ALL;
  }

  public String getButtonName() {
    switch (getMode()) {
    case MODE_UPDATE:
      return "update";
    case MODE_CREATE:
      return "create";
    case MODE_DELETE:
      return "delete";
    case MODE_VIEW:
      return "ok";
    }
    return null;
  }

  public String getStatusDisp(){
    System.out.println("reminder status : " + getReminder().getStatus());
    return getReminder().getStatus()==Reminder.STATUS_ON?"On":"Off";
  }
  
  public void setStatusDisp(String statusVal){
    int stat = statusVal.equals("On")?1:2;
    getReminder().setStatus(stat);
  }

  public String getTitle() {
    int mode = getMode();
    String title = null;
    title = (mode >= 0 && mode < 3) ? TITLE[mode] : "Reminder";
    return title;
  }

  public boolean getInputDisabled() {
    return (getMode() == MODE_VIEW || getMode() == MODE_DELETE);
  }

  public boolean getInCreateMode() {
    int mode = getMode();
    return (mode == MODE_UPDATE || mode == MODE_CREATE);
  }

  public boolean getReminderToolbarShow() {
    int mode = getMode();
    return (mode == MODE_UPDATE || mode == MODE_VIEW || mode == MODE_DELETE);
  }

  public boolean getReminderToolbarButtonUpdateDsbl() {
    boolean hasPermission = checkPermission(Permission.UPDATE_PROJECT_REMINDER, Permission.UPDATE_COMMUNITY_REMINDER);
    return getMode() != MODE_VIEW || !hasPermission;
  }

  public boolean getReminderToolbarButtonDeleteDsbl() {
    boolean hasPermission = checkPermission(Permission.DELETE_PROJECT_REMINDER, Permission.DELETE_COMMUNITY_REMINDER);
    return getMode() != MODE_VIEW || !hasPermission;
  }

}
