package workspace.web.components;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;

import workspace.Visit;
import workspace.web.pages.LoginPage;
import workspace.web.pages.ProjectsPage;
import workspace.web.pages.MyWorkspacePage;
import workspace.web.pages.CommunitiesPage;

public abstract class TabMenu extends BaseComponent implements
    PageRenderListener {
  private String _pageType;

  private String _bHomeClass;
  private String _bProjectsClass;
  private String _bCommunitiesClass;
  private String _bMyWorkspaceClass;
  private String _bLoginClass;
  private String _bRegisterClass;

  public void pageBeginRender(PageEvent event) {
    IPage page = getPage();
    _pageType = page.getSpecification().getProperty("page-type");
    if (_pageType == null)
      _pageType = (String) page.getProperty("pageType");
    if (_pageType == null)
      _pageType = "";
    _bHomeClass = selectTabclass("Home", "here");
    _bProjectsClass = selectTabclass("Projects", "here");
    _bCommunitiesClass = selectTabclass("Communities", "here");
    _bMyWorkspaceClass = selectTabclass("MyWorkspace", "here");
    _bLoginClass = selectTabclass("Login", "here");
    _bRegisterClass = selectTabclass("Register", "here");
  }

  private String selectTabclass(String type, String activeTab) {
    return _pageType.equals(type) ? activeTab : "";
  }

  public String getBHomeClass() {
    return _bHomeClass;
  }

  public String getBProjectsClass() {
    return _bProjectsClass;
  }

  public String getBCommunitiesClass() {
    return _bCommunitiesClass;
  }

  public String getBMyWorkspaceClass() {
    return _bMyWorkspaceClass;
  }

  public String getBLoginClass() {
    return _bLoginClass;
  }

  public String getBRegisterClass() {
    return _bRegisterClass;
  }

  public boolean getIsLogIn() {
    Visit visit = (Visit) getPage().getVisit();
    return visit.isUserLoggedIn();
  }

  public void logout(IRequestCycle cycle) {
    Visit visit = (Visit) getPage().getVisit();
    visit.logout();
    cycle.activate("Home");
    return;
  }

  public void goToProjects(IRequestCycle cycle) {
    Visit visit = (Visit) getPage().getVisit();
    visit.clearCommunityCache();
    visit.clearProjectCache();
    ProjectsPage projectsPage = (ProjectsPage) cycle.getPage("ProjectsPage");
    cycle.activate(projectsPage);
  }

  public void goToCommunities(IRequestCycle cycle) {
    Visit visit = (Visit) getPage().getVisit();
    visit.clearCommunityCache();
    visit.clearProjectCache();
    CommunitiesPage communitiesPage = (CommunitiesPage) cycle.getPage("CommunitiesPage");
    cycle.activate(communitiesPage);
  }

  public void goToMyWorkspace(IRequestCycle cycle) {
    Visit visit = (Visit) getPage().getVisit();
    visit.clearProjectCache();
    visit.clearCommunityCache();
    MyWorkspacePage myWorkspacePage = (MyWorkspacePage) cycle
        .getPage("MyWorkspacePage");
    cycle.activate(myWorkspacePage);
  }
  
  public void goToLogin(IRequestCycle cycle) {
    Visit visit = (Visit) getPage().getVisit();
    visit.clearProjectCache();
    visit.clearCommunityCache();
    if(getIsLogIn()){
      MyWorkspacePage myWorkspacePage = (MyWorkspacePage) cycle
          .getPage("MyWorkspacePage");
      cycle.activate(myWorkspacePage);
    } else{
      LoginPage loginPage = (LoginPage) cycle.getPage("LoginPage");
      cycle.activate(loginPage);     
    }
  }
  
}
