package workspace.web.components;

import java.util.List;
import java.util.ArrayList;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.form.IFormComponent;

import workspace.Visit;
import workspace.web.PrioritySelectionModel;
import workspace.web.StatusSelectionModel;
import workspace.web.UserSelectionModel;
import workspace.web.pages.TaskListPage;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Role;
import com.fmi.weprom.api.Task;
import com.fmi.weprom.api.User;

public abstract class TaskEditor extends BaseComponent {

  // Fields
  private StatusSelectionModel statusModel;
  private PrioritySelectionModel priorityModel;

  // TaskEditor mode constans
  public static final int MODE_VIEW_ALL = 0;
  public static final int MODE_UPDATE = 1;
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  public static final int MODE_DELETE = 4;

  // TaskEditor title constants
  public static String TITLE[] = { "Result", "Update Task", "Create Task",
      "View Task", "Delete Task" };

  // Result strings
  public static String SUCCESS[] = { "", "Your task has been updated!",
      "Your task has been created!", "Your task has been viewed!",
      "Your task has been deleted!" };

  public static String FAILURE1[] = { "",
    "Your task has not been updated! Please try again.",
    "Your task has not been created! Probably it was empty.",
    "Your task has not been viewed! Please try again.",
    "Your task has not been deleted! Please try again." };

  // Fields
  //private UserSelectionModel userModel;
  
  // Pairs of getters-setters for:
  // Component's parameters
  public abstract Task getTask();

  public abstract void setTask(Task task);

  public abstract int getMode();

  public abstract void setMode(int mode);

  // Component's persistent properties
  public abstract String getResultString();

  public abstract void setResultString(String result);

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  // Listeners
  public void updateTask(IRequestCycle cycle) {
    Task task = getTask();
    System.out.println("updateTask: --->" + task.getID());
    setParamProp(MODE_UPDATE, task, false, "");
  }

  public void deleteTask(IRequestCycle cycle) {
    setMode(MODE_DELETE);
  }

  public void manageTask(IRequestCycle cycle) {
    System.out.println("Task List: updateTask: --->");
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if (delegate.getHasErrors()) {
      System.out.println("formUpdateTask - has errors");
    } else {
      Task task = getTask();
      System.out.println("formUpdateTask - no errors");
      System.out.println("Task processor: " + getTask().getProcessor());
      switch (getMode()) {
      case MODE_CREATE:
        if (!validDates(task, delegate))
          return;
        if(getVisit().taskSave(task))
          setParamProp(MODE_VIEW_ALL, null, true,SUCCESS[getMode()]);
        else
          setResultString(FAILURE1[getMode()]);
        break;
      case MODE_UPDATE:
        if (!validDates(task, delegate))
          return;
        if(getVisit().taskUpdate(task))
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        else
          setResultString(FAILURE1[getMode()]);
        break;
      case MODE_DELETE:
        if(getVisit().taskDelete(task))
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        else
          setResultString(FAILURE1[getMode()]);
        break;
      case MODE_VIEW:
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        TaskListPage taskListPage = (TaskListPage) cycle
            .getPage("TaskListPage");
        cycle.activate(taskListPage);
        break;
      }
    }
    System.out.println("Task List: updateTask: --->");
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Validate start and end dates
  private boolean validDates(Task task, IValidationDelegate delegate) {
    if (task.getStartDate() == null) {
      error(delegate, "inputStartDate", "Start Date is required.",
          ValidationConstraint.REQUIRED);
      return false;
    }
    if (task.getEndDate() == null) {
      error(delegate, "inputEndDate", "End Date is required.",
          ValidationConstraint.REQUIRED);
      return false;
    }
    if (task.getStartDate().after(task.getEndDate())) {
      error(delegate, "inputEndDate", "End Date must be after Start Date.",
          ValidationConstraint.CONSISTENCY);
      return false;
    }
    return true;
  }

  private void error(IValidationDelegate delegate, String componentId,
      String message, ValidationConstraint constraint) {
    IFormComponent component = (IFormComponent) getComponent(componentId);
    delegate.setFormComponent(component);
    delegate.record(message, constraint);
  }

  // Parameters and properties order: mode, task, result, resultString
  private void setParamProp(int mode, Task task, boolean result,
      String resultString) {
    setResultString(resultString);
    setTask(task);
    setResult(result);
    setMode(mode);
  }

  // Api Methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public User getUser() {
    return (User) getVisit().getUser();
  }

  public Project getProject() {
    return (Project) getVisit().getProject();
  }

  // Helper Methods for visualizing
  public StatusSelectionModel getStatusModel() {
    if(statusModel == null)
      statusModel = new StatusSelectionModel(StatusSelectionModel.TASK);    
    return statusModel;
  }

  public PrioritySelectionModel getPriorityModel() {
    if(priorityModel == null)
      priorityModel = new PrioritySelectionModel();    
    return priorityModel;
  }

  public UserSelectionModel getUserModel() {
    List users = new ArrayList();    
    Project project;
    if (getProject() == null) 
      project = getTask().getProject();
    else
      project = getProject(); 
    users = project.getUsers();
    UserSelectionModel userModel = new UserSelectionModel(users);    
    return userModel;
  }

  public boolean isResult() {
    return getMode() == MODE_VIEW_ALL;
  }

  public String getButtonName() {
    switch (getMode()) {
    case MODE_UPDATE:
      return "update";
    case MODE_CREATE:
      return "create";
    case MODE_VIEW:
      return "ok";
    case MODE_DELETE:
      return "delete";
    }
    return null;
  }

  public String getTitle() {
    int mode = getMode();
    String title = null;
    if (mode >= 0 && mode < 5) {
      title = TITLE[mode];
    } else
      title = "";
    return title;
  }

  public boolean getInputDisabled() {
    return (getMode() == MODE_VIEW || getMode() == MODE_DELETE);
  }
  public boolean getNoPermissionAssignTask() {
    Permission permission = getVisit().getAccessRights().getPermission(Permission.ASSIGN_TASKS);
    boolean hasPermission = getVisit().getAccessRights()
            .checkPermission(getUser(), getTask().getProject(), permission);
    return (getInputDisabled() || !hasPermission);
  }
  
  public boolean getTaskToolbarShow() {
    int mode = getMode();
    return (mode == MODE_UPDATE || mode == MODE_VIEW || mode == MODE_DELETE);
  }

  public boolean getTaskToolbarButtonUpdateDsbl() {    
    Permission permission = getVisit().getAccessRights().getPermission(Permission.UPDATE_PROJECT_TASK);
    boolean hasPermission = getVisit().getAccessRights()
            .checkPermission(getUser(), getTask().getProject(), permission);
    boolean isProcessor = getUser().equals(getTask().getProcessor())? true : false;
    boolean isOwner = getUser().equals(getTask().getOwner())? true : false;
    System.out.println("Task editor: is processor " + isProcessor);
    System.out.println("Task editor: is owner " + isOwner);
    System.out.println("Task editor: has permission " + hasPermission);
    return getMode() != MODE_VIEW || !(hasPermission & isOwner || isProcessor);
  }

  public boolean getTaskToolbarButtonDeleteDsbl() {
    Permission permission = getVisit().getAccessRights().getPermission(Permission.DELETE_PROJECT_TASK);
    boolean hasPermission = getVisit().getAccessRights()
            .checkPermission(getUser(), getTask().getProject(), permission);
    boolean isOwner = getUser().equals(getTask().getOwner())? true : false;
    //hasPermission = getVisit().checkUserPermission(Permission.DELETE_PROJECT_TASK);
    return getMode() != MODE_VIEW || !hasPermission || !isOwner;
  }

  public boolean getInCreateMode() {
    int mode = getMode();
    return (mode == MODE_CREATE);
  }
  
  public String getAssignedStatus() {
    boolean isAssigned = getTask().isAssigned();
    if(isAssigned)
      return "The task is assigned to:";
    else
      if(getTask().getProcessor()!= null)
        return "The task assignment is pending. Answer to task assignment message is expected from:";
      else
        return "The task is not assigned yet.";
  }

}
