package workspace.web.components;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.util.ComponentAddress;

import workspace.Visit;
import workspace.web.DateComparatorEEEddMMyyyy;
import workspace.web.PercentageComparator;
import workspace.web.PrioritySelectionModel;
import workspace.web.StatusSelectionModel;
import workspace.web.pages.ProjectWorkspacePage;
import workspace.web.pages.TaskEditorPage;
import com.fmi.weprom.api.Task;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.Project;

public abstract class TaskList extends BaseComponent implements
    PageRenderListener {

  // Mode constants
  public static final int MODE_VIEW = 3;

  // Fields
  private IPrimaryKeyConvertor m_taskConvertor;
  private StatusSelectionModel statusModel;
  private PrioritySelectionModel priorityModel;
  public SimpleDateFormat dateFormat = null;

  // TaskList component's properties
  public abstract List getTaskList();

  public abstract void setTaskList(List taskList);

  // Managing the Task Convertor
  public TaskList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_taskConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
        Task task = (Task) objValue;
        return task.getID() + "";
      }

      public Object getValue(Object objPrimaryKey) {
        String prKey = (String) objPrimaryKey;
        List list = getTaskList();
        // Find the item
        int count = list.size();
        for (int i = 0; i < count; i++) {
          Task task = (Task) list.get(i);
          if (prKey.equals(task.getID() + ""))
            return task;
        }
        return getVisit().getProject().createTask(getVisit().getUser()); //Handle data item not found
      }
    };
  }

  public IPrimaryKeyConvertor getTaskConvertor() {
    return m_taskConvertor;
  }

  // Manage the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }

  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnStatusDisp = new SimpleTableColumn("statusDisp",
          "", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getStatDisp(((Task) objRow));
            }
          }, false);
      columnStatusDisp.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("statusDispColumnValue"))));

      SimpleTableColumn columnSubject = new SimpleTableColumn("taskSubject",
          "Subject", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((Task) objRow).getSubject();
            }
          }, true);

      // The following overrides ...
      columnSubject.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("subjectColumnValue"))));

      SimpleTableColumn columnProprity = new SimpleTableColumn("priority",
          "Priority", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getPriorityDisplay((Task) objRow);
            }
          }, true);

      SimpleTableColumn columnProject = new SimpleTableColumn("taskProject",
          "Project", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getTaskProject((Task) objRow);
            }
          }, true);

      SimpleTableColumn columnProcessor = new SimpleTableColumn(
          "taskProcessor", "Processor", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getTaskProcessor((Task) objRow);
            }
          }, true);

      SimpleTableColumn columnStatus = new SimpleTableColumn("taskStatus",
          "Status", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getStatusDisplay((Task) objRow);
            }
          }, true);

      SimpleTableColumn columnProgress = new SimpleTableColumn("progress", "%",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((Task) objRow).getCompleted() + " %";
            }
          }, true);
      columnProgress.setColumnComparator(Visit.prcntgComparator);

      columns.put(columnStatusDisp.getColumnName(), columnStatusDisp);
      columns.put(columnSubject.getColumnName(), columnSubject);
      columns.put(getStartDateColumn().getColumnName(), getStartDateColumn());
      columns.put(getEndDateColumn().getColumnName(), getEndDateColumn());
      columns.put(columnProprity.getColumnName(), columnProprity);
      if (getProject() != null)
        columns.put(columnProcessor.getColumnName(), columnProcessor);
      else
        columns.put(columnProject.getColumnName(), columnProject);
      columns.put(columnStatus.getColumnName(), columnStatus);
      columns.put(columnProgress.getColumnName(), columnProgress);
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }
  }

  public ITableColumn getStartDateColumn() {
    SimpleTableColumn columnStartDate = new SimpleTableColumn("startDate",
        "Start date", new ITableColumnEvaluator() {
          public Object getColumnValue(ITableColumn objColumn, Object objRow) {
            return getTaskDate(((Task) objRow).getStartDate());
          }
        }, true);
    columnStartDate.setColumnComparator(Visit.dtComparatorEEEddMMyyyy);
    return columnStartDate;
  }

  public ITableColumn getEndDateColumn() {
    SimpleTableColumn columnEndDate = new SimpleTableColumn("endDate",
        "End date", new ITableColumnEvaluator() {
          public Object getColumnValue(ITableColumn objColumn, Object objRow) {
            return getTaskDate(((Task) objRow).getEndDate());
          }
        }, true);
    columnEndDate.setColumnComparator(Visit.dtComparatorEEEddMMyyyy);
    return columnEndDate;
  }

  //Formatting the task list display
  public StatusSelectionModel getStatusModel() {
    if(statusModel == null)
      statusModel = new StatusSelectionModel(StatusSelectionModel.TASK);    
    return statusModel;
  }

  public PrioritySelectionModel getPriorityModel() {
    if(priorityModel == null)
      priorityModel = new PrioritySelectionModel();    
    return priorityModel;
  }

  public String getStatDisp(Task task) {
    int status = task.getStatus();
    System.out.println("status: " + status);
    String res= "3";
    switch(status){
    case 0: 
      res = "0";
      break;
    case 1:
      res = "1";
      break;
    case 2:
      res = "2";
      break;
    case 3:
      res = "3";
      break;
    }
    System.out.println("resDisp: " + res);
    return res;
  }
  
  public String getStatDisp(int taskID, int projectID) {
    Project project = getVisit().getWeprom().getProject(projectID);
    Task task = project.getTask(taskID);
    return  getStatDisp(task);
  }
  
  public String getStatusDisplay(Task task){
    return getStatusModel().getStatDisplay(task.getStatus());
  }
  
  public String getPriorityDisplay(Task task){
    return getPriorityModel().getPriorityDisplay(task.getPriority());
  }
  
  public String getTaskDate(Date taskDate) {
    if (taskDate != null)
      return getDateFormat().format(taskDate);
    return "";
  }

  public String getTaskProject(Task task) {
    Project project = task.getProject();
    System.out.println("Task projectId: " + project.getID());
    String projName = project.getName();
    if (projName.length() > 18)
      return projName.substring(0, 16) + "...";
    else
      return projName;
  }

  public String getTaskProcessor(Task task) {
    if (task != null) {
      User processor = task.getProcessor();
      if (processor != null) {
        String procssrId = task.getProcessor().getNick();
        if (procssrId != null)
          if (procssrId.length() > 15)
            return procssrId.substring(0, 13) + "...";
          else
            return procssrId;
        else
          return null;
      }
    }
    return null;
  }

  public DateFormat getDateFormat() {
    if (dateFormat == null) 
      dateFormat = new SimpleDateFormat("EEE dd.MM.yy", Locale.US);
    return dateFormat;
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    List taskList = getVisit().getTaskList();
    setTaskList(taskList);
  }

  // Listeners
  public void viewTask(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters != null && parameters.length > 1) {
      int taskID = ((Integer) parameters[0]).intValue();
      int projectID = ((Integer) parameters[1]).intValue();
      Project project = getVisit().getWeprom().getProject(projectID);
      Task task = project.getTask(taskID);
      if (task != null) {
        TaskEditorPage taskEditorPage = (TaskEditorPage) cycle
            .getPage("TaskEditorPage");
        taskEditorPage.setTask(task);
        taskEditorPage.setMode(MODE_VIEW);
        cycle.activate(taskEditorPage);
      } else
        System.out
            .println("TaskEditorPage()cannot be activated: task= " + task);
    } else
      System.out.println("TaskEditorPage()cannot be activated: parameters= "
          + parameters);
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  //Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public Project getProject() {
    return (Project) getVisit().getProject();
  }
}
