package workspace.web.components;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;

import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.Iterator;

import workspace.Visit;
import workspace.web.pages.UserListPage;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Role;
import com.fmi.weprom.api.User;

public abstract class UserEditor extends BaseComponent implements
    PageRenderListener {
  // UserEditor mode constans
  public static final int MODE_VIEW_ALL = 0;
  public static final int MODE_UPDATE = 1;
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  public static final int MODE_DELETE = 4;

  // UserEditor title constants
  public static String TITLE[] = { "Result", "Update User Role", "Invite User",
      "View User", "Remove User form Workspace" };

  // Result strings
  public static String SUCCESS[] = { "", "The user's role has been updated!",
      "The user's role has been assigned!", "You have viewed the user!",
      "The user has been removed from the project!" };

  // Component's parameters
  public abstract User getUser();

  public abstract void setUser(User user);

  public abstract int getMode();

  public abstract void setMode(int mode);

  // Component's persistent properties
  public abstract String getResultString();

  public abstract void setResultString(String result);

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  // Component's non-persistent properties
  public abstract Role getRole();

  public abstract Set getSelectedRoles();

  public abstract void setSelectedRoles(Set selectedRoles);

  public void pageBeginRender(PageEvent event) {
    if (!event.getRequestCycle().isRewinding()&getUser()!=null)
      setUserRolesSelected();
  }

  // Listeners
  public void updateUser(IRequestCycle cycle) {
    User user = getUser();
    System.out.println("updateUser: --->" + user.getNick());
    setParamProp(MODE_UPDATE, user, false, "");
  }

  public void removeUser(IRequestCycle cycle) {
    setMode(MODE_DELETE);
  }

  public void manageUser(IRequestCycle cycle) {
    System.out.println("User Editor: updateUser: --->");
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if (delegate.getHasErrors()) {
      System.out.println("formUpdateUser - has errors");
    } else {
      User user = getUser();
      System.out.println("formUpdateUser - no errors");
      switch (getMode()) {
      case MODE_CREATE:
        if(!validRole(delegate))
          return;
        manageRoles(user);
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        break;
      case MODE_UPDATE:
        if(!validRole(delegate))
          return;
        manageRoles(user);
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        break;
      case MODE_DELETE:
        List userRoles = new ArrayList();
        if(getProject()!= null){
          userRoles = getVisit().getUserRoleInProject(getUser());
          if(userRoles.size()>0)
            getVisit().deleteUserRolesInProject(user, getProject(), userRoles);
          getVisit().removeUserFromProject(user, getProject());
        }
        else if(getCommunity() != null){
          userRoles = getVisit().getUserRoleInCommunity(getUser());
          if(userRoles.size()>0)
            getVisit().deleteUserRolesInCommunity(user, getCommunity(), userRoles);
          getVisit().removeUserFromCommunity(user, getCommunity());
        }
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        break;
      case MODE_VIEW:
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        UserListPage userListPage = (UserListPage) cycle
            .getPage("UserListPage");
        cycle.activate(userListPage);
        break;
      }
    }
    System.out.println("User Editor: updateUser: <---");
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Validator methods
  private boolean validRole(IValidationDelegate delegate) {
    List roles = getUserRolesSelected();
    if (roles == null || roles.isEmpty()) {
      IFormComponent inputRole = (IFormComponent) getComponent("inputRole");
      delegate.setFormComponent(inputRole);
      delegate.record("You should specify at least one role for " +
          getUser().getNick() + " or you may delete " +
          getUser().getNick() + " from the workspace!",
          ValidationConstraint.CONSISTENCY);
      return false;
    } else
      return true;
  }

  // Parameters and properties order: mode, user, result, resultString
  private void setParamProp(int mode, User user, boolean result,
      String resultString) {
    setResultString(resultString);
    setUser(user);
    setResult(result);
    setMode(mode);
  }
  
  public void manageRoles(User user){
    List allRoles = getVisit().getRoles();    
    Set selectedRoles = getSelectedRoles();
    List userRoles = new ArrayList();
    if(getProject()!= null)
      userRoles = getVisit().getUserRoleInProject(getUser());
    else if(getCommunity() != null)
      userRoles = getVisit().getUserRoleInCommunity(getUser());
    
    List rolesToAdd = new ArrayList();
    List rolesToRemove = new ArrayList();
    
    for (int i = 0; i < allRoles.size(); i++){
      if(userRoles.contains((Role)allRoles.get(i)) & 
          !selectedRoles.contains((Role)allRoles.get(i)) &
          !((Role)allRoles.get(i)).getName().equals(Role.PROJECT_PARTICIPANT)&
          !((Role)allRoles.get(i)).getName().equals(Role.COMMUNITY_PARTICIPANT))        
        rolesToRemove.add((Role)allRoles.get(i));    
      if(!userRoles.contains((Role)allRoles.get(i)) &
          selectedRoles.contains((Role)allRoles.get(i)))
        rolesToAdd.add((Role)allRoles.get(i));
    }
        
    for (int i = 0; i < rolesToAdd.size(); i++){     
      System.out.println("Added role: "+((Role)rolesToAdd.get(i)).getName());
    }
    for (int i = 0; i < rolesToRemove.size(); i++){     
      System.out.println("Removed role: "+((Role)rolesToRemove.get(i)).getName());
    }
    if(getProject()!=null){
      if(rolesToAdd.size()>0)
        getVisit().assignUserRolesInProject(user, getProject(),
          rolesToAdd);
      if(rolesToRemove.size()>0)
        getVisit().deleteUserRolesInProject(user, getProject(),
          rolesToRemove);
    } else if(getCommunity()!=null){
      if(rolesToAdd.size()>0)
        getVisit().assignUserRolesInCommunity(user, getCommunity(),
          rolesToAdd);
      if(rolesToRemove.size()>0)
        getVisit().deleteUserRolesInCommunity(user, getCommunity(),
          rolesToRemove);
    }
  }
  
  // Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public Project getProject() {
    return getVisit().getProject();
  }

  public List getUserRoles() {
    return getVisit().getRoles();
  }

  public Community getCommunity() {
    return (Community) getVisit().getCommunity();
  }

  public boolean checkPermission(String permProject, String permCommunity){
    Community c = getCommunity();
    Project p = getProject();
    if(c == null & p != null)
      return getVisit().checkUserPermission(permProject);
    else if(c != null)
      return getVisit().checkUserPermission(permCommunity);
    else 
      return false;
  }
  
  // Methods for synthetic properties
  public void setRoleSelected(boolean roleSelected) {
    System.out.println("Message Editor: setRoleSelected: >--->");
    Set selectedRoles = getSelectedRoles();
    if (selectedRoles == null)
      setSelectedRoles(new HashSet());
    if (roleSelected) 
      getSelectedRoles().add(getRole());
    else 
      getSelectedRoles().remove(getRole());    
    System.out.println("Message Editor: setRoleSelected: <---<");
  }

  public boolean isRoleSelected() {
    if (getSelectedRoles() == null) {
      setSelectedRoles(new HashSet());
    }
    return getSelectedRoles().contains(getRole());
  }

  public void setUserRolesSelected() {
    if (getSelectedRoles() == null) {
      List userRoles = new ArrayList();
      if(getProject()!= null)
        userRoles = getVisit().getUserRoleInProject(getUser());
      else if(getCommunity() != null)
        userRoles = getVisit().getUserRoleInCommunity(getUser());
      Set selectedRoles = new HashSet();
      for (int i = 0; i < userRoles.size(); i++){
        selectedRoles.add(userRoles.get(i));
        System.out.println(((Role)userRoles.get(i)).getName());
      }
      setSelectedRoles(selectedRoles);
    }
  }

  public List getUserRolesSelected() {
    Set selectedRoles = getSelectedRoles();
    List userRoles = new ArrayList();
    Iterator iterator = selectedRoles.iterator();
    while (iterator.hasNext()) {
      Role role = (Role) iterator.next();
      userRoles.add(role);
    }
    for(int i=0;i<userRoles.size(); i++)
      System.out.println("role "+ i + ": " + userRoles.get(i));
    return userRoles;
  }

  // Helper Methods for visualizing
  public boolean isResult() {
    return getMode() == MODE_VIEW_ALL;
  }

  public String getButtonName() {
    switch (getMode()) {
    case MODE_UPDATE:
      return "update";
    case MODE_CREATE:
      return "assign";
    case MODE_VIEW:
      return "ok";
    case MODE_DELETE:
      return "delete";
    }
    return null;
  }

  public String getTitle() {
    int mode = getMode();
    String title = null;
    if (mode >= 0 && mode < 5) {
      title = TITLE[mode];
    } else
      title = "";
    return title;
  }

  public boolean getInputDisabled() {
    return (getMode() == MODE_VIEW || getMode() == MODE_DELETE);
  }

  public boolean getUserToolbarShow() {
    int mode = getMode();
    return (mode == MODE_UPDATE || mode == MODE_VIEW || mode == MODE_DELETE);
  }

  public boolean getUserToolbarButtonUpdateDsbl() {
    boolean hasPermission = checkPermission(Permission.PROJECT_USER_EDIT_ROLES,
        Permission.COMMUNITY_USER_EDIT_ROLES);
    return getMode() != MODE_VIEW || !hasPermission;
  }

  public boolean getUserToolbarButtonDeleteDsbl() {
    boolean hasPermission = checkPermission(Permission.PROJECT_USER_EDIT_ROLES,
        Permission.COMMUNITY_USER_EDIT_ROLES);
    return getMode() != MODE_VIEW || !hasPermission;
  }

  public boolean getInCreateMode() {
    int mode = getMode();
    return (mode == MODE_CREATE);
  }

}
