package workspace.web.components;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.LinkedHashMap;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.util.ComponentAddress;

//import database.MySQLConnector;
import workspace.MessageUtil;
import workspace.Visit;
import workspace.web.pages.MessageEditorPage;
import workspace.web.pages.UserEditorPage;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.Role;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.Workspace;
import com.fmi.weprom.api.Message;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.WEPROMException;


public abstract class UserInviteList extends BaseComponent implements
    PageRenderListener {

  // Mode constants
  public static final int MODE_CREATE = 2;

  // Fields
  private IPrimaryKeyConvertor m_userConvertor;

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  public abstract void setResultString(String result);

  // Pairs of getter-setter methods for the UserList component's properties
  public abstract List getUserList();

  public abstract void setUserList(List userList);

  // Managing the User Convertor
  public UserInviteList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_userConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
        User user = (User) objValue;
        System.out.println("IPrimaryKeyConvertor: " + user.getLastName() + "->"
            + user.getNick());
        return user.getNick();
      }

      public Object getValue(Object objPrimaryKey) {
        String prKey = (String) objPrimaryKey;
        List list = getUserList();
        // Find the item
        int count = list.size();
        for (int i = 0; i < count; i++) {
          User user = (User) list.get(i);
          if (user.getNick().equals(prKey)) {
            System.out.println("IPrimaryKeyConvertor: " + prKey + "->"
                + user.getLastName());
            return user;
          }
        }
        return getVisit().getWeprom().createUser("user"); //Handle data item not found
      }
    };
  }

  public IPrimaryKeyConvertor getUserConvertor() {
    return m_userConvertor;
  }

  // Managing the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }

  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnCard = new SimpleTableColumn("card", "",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return "User";
            }
          }, false);
      // The following overrides the default SimpleTableColumn.getColumnValue to
      // render the Block component, defined on the HTML template
      columnCard.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("cardColumnValue"))));

      SimpleTableColumn columnUsername = new SimpleTableColumn("username",
          "Nick", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getNick();
            }
          }, true);

      // The following overrides ...
      columnUsername.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("usernameColumnValue"))));

      ITableColumn columnProjects = new SimpleTableColumn("projects",
          "Projects", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              if(getProject()!=null)
                return getUserWorkspaceNames(((User) objRow).getProjects());
              else if(getCommunity()!=null)
                return getUserWorkspaceNames(((User) objRow).getCommunities());
              else return "";
            }
          }, true);

      ITableColumn columnName = new SimpleTableColumn("name", "Name",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getLastName();
            }
          }, true);

      ITableColumn columnFirstName = new SimpleTableColumn("firstName",
          "First Name", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getName();
            }
          }, true);

      ITableColumn columnEmail = new SimpleTableColumn("email", "Email",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getEmail();
            }
          }, true);

      columns.put(columnCard.getColumnName(), columnCard);
      columns.put(columnUsername.getColumnName(), columnUsername);
      columns.put(columnProjects.getColumnName(), columnProjects);
      columns.put(columnName.getColumnName(), columnName);
      columns.put(columnFirstName.getColumnName(), columnFirstName);
      columns.put(columnEmail.getColumnName(), columnEmail);
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }
  }

  public String getUserWorkspaceNames(List projects) {
    int count = projects.size();
    if (count > 0) {
      StringBuffer buffer = new StringBuffer();
      int x = 0;
      Iterator i = projects.iterator();
      while (i.hasNext()) {
        if (++x > 1)
          if (x > count)
            buffer.append("");
          else
            buffer.append(", ");
        if(getProject()!=null){
          Project workspace = (Project) i.next();
          buffer.append(workspace.getName());
        }else if(getCommunity()!=null){
          Community workspace = (Community) i.next();
          buffer.append(workspace.getName());
        } else
          buffer.append("");
      }
      return buffer.toString();
    } else
      return "";
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    List userList = new ArrayList();
    System.out.println("User List: pageBeginRender: --->");
    if(getProject()!=null)
      userList = getVisit().getUserNotInProjectList();
    else if(getCommunity()!=null)
      userList = getVisit().getUserNotInCommunityList();
    setUserList(userList);
    System.out.println("User List: pageBeginRender: --->");
  }

  // Listeners    
  public void inviteUser(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters != null & parameters.length > 0 & checkPermission()) {        
      int userID = ((Integer) parameters[0]).intValue();
      User user = getVisit().getWeprom().getUser(userID);
      if (user != null) {
        try{
          if(getProject()!=null)
            getProject().inviteUser(user);
          else if(getCommunity()!=null)
            getCommunity().inviteUser(user);
          setResult(true);
          setResultString("Invitation to " + getResString() +
              " was successfuly sent to " + user.getNick() + "!");
        }catch(Exception e){
          setResult(true);
          setResultString("Invitation to " + getResString() +
              " could not be send to " + user.getNick() + "!");
        }
      } else
        System.out.println("MessageEditorPage()cannot be activated: user= "
            + user);
    } else
      System.out.println("MessageEditorPage()cannot be activated: parameters= "
          + parameters);
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  //Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public Project getProject() {
    return (Project) getVisit().getProject();
  }

  public Community getCommunity() {
    return (Community) getVisit().getCommunity();
  }

  public String imgRole(String rolesStr) {
    System.out.println(rolesStr);
    String[] roles = rolesStr.split(", ");
    for (int i = 0; i < roles.length; i++)
      if (roles[i].equals(Role.PROJECT_COORDINATOR) ||
          roles[i].equals(Role.COMMUNITY_COORDINATOR))
        return "coordinator";
    return "user";
  }
  
  private boolean checkPermission(){ 
    if(getProject()!=null)    
      return getVisit().checkUserPermission(Permission.PROJECT_USER_EDIT_ROLES);
    else if(getCommunity()!=null)
      return getVisit().checkUserPermission(Permission.COMMUNITY_USER_EDIT_ROLES);
    return false;
  }
  
  private String getResString(){
    String workspaceName = "";
    if(getProject()!=null)    
      workspaceName = getProject().getName() + " project";
    else if(getCommunity()!=null)
      workspaceName = getCommunity().getName() + " community";
    return workspaceName;    
  }
}
