package workspace.web.components.docsbrowser;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.annotations.Persist;
import org.apache.tapestry.contrib.table.model.ITableModel;
import org.apache.tapestry.contrib.tree.components.INodeRenderFactory;
import org.apache.tapestry.contrib.tree.components.TreeView;
import org.apache.tapestry.contrib.tree.model.ITreeDataModel;
import org.apache.tapestry.contrib.tree.model.ITreeModel;
import org.apache.tapestry.contrib.tree.model.ITreeStateListener;
import org.apache.tapestry.contrib.tree.model.ITreeStateModel;
import org.apache.tapestry.contrib.tree.model.TreeStateEvent;
import org.apache.tapestry.contrib.tree.simple.SimpleTreeModel;
import org.apache.tapestry.event.PageBeginRenderListener;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageValidateListener;
import org.apache.tapestry.request.IUploadFile;

import workspace.Visit;

import com.fmi.weprom.api.File;
import com.fmi.weprom.api.WEPROMException;

public abstract class DocsBrowser extends BaseComponent 
                                  implements ITreeStateListener, PageBeginRenderListener, PageValidateListener {
  
  private static final Log LOG = LogFactory.getLog(DocsBrowser.class);
  
  @Persist
  public abstract ITreeModel getTreeModel();
  public abstract void setTreeModel(ITreeModel m);

  @Persist
  public abstract ITableModel getTableModel();
  public abstract void setTableModel(ITableModel m);
  
  // cache for the FileNode objects, scope:requestCycle
  public abstract Map getFileNodeCache();
  public abstract void setFileNodeCache(Map m);
  
  public abstract File getRoot();
  
  public abstract IUploadFile getFile();
  
  public boolean getFileUploadDisabled() {
    return getSelectedFolder()==null;
  }
  
  public void uploadFile(IRequestCycle cycle) {
    String _method = "uploadFile "; 
    LOG.debug(_method);
    IUploadFile uploadFile = getFile(); 
    if (uploadFile == null) {
      LOG.debug(_method+"no file");
      return;
    }    
    FileNode destinationFolder = getSelectedFolder();
    if (destinationFolder == null) {
      LOG.debug(_method+"no destination folder");
      return;      
    }
    LOG.debug(_method+"destinationFolder:"+destinationFolder);
    
    createFile(uploadFile, getTreeDataModel().getFile((Integer)destinationFolder.getID()));
  }
  
  public String getSelectedFolderName() {
    FileNode fn = getSelectedFolder();
    if (fn!=null) {
      return fn.getName();
    } else {
      return "";
    }
  }
  
  public FileNode getSelectedFolder() {
    Integer id = (Integer) getDocsTreeModel().getTreeStateModel().getSelectedNode();
    if (id!=null) {
      return getTreeDataModel().getFileNode(id);
    }
    return null;
  }
  
  public boolean getToolbarShow() {
    return true;
  }
  
  public INodeRenderFactory getRenderFactory() {
    return new NodeRenderFactory();
  }
  
  public ITreeModel getDocsTreeModel() {
    ITreeModel m = getTreeModel();
    if (m==null) {
      LOG.debug("createTreeModel");
      m = createTreeModel();
      setTreeModel(m);
    } else {
      //LOG.debug("tree model already exists");
    }
    return m;
  }
  
  public DocsTableModel getDocsTableModel() {
    DocsTableModel m = (DocsTableModel) getTableModel();
    if (m==null) {
      LOG.debug("createTableModel");
      m = new DocsTableModel(null);
      setTableModel(m);
    } else {
      //LOG.debug("tree model already exists");
    }
    return m;
  }  
  
  private DocsTreeDataModel getTreeDataModel() {
    return (DocsTreeDataModel) getDocsTreeModel().getTreeDataModel();
  }
  
  public ITreeStateListener getTreeStateListener(){
    return this;
  }
  
  public boolean getToolbarButtonNewFolderDisabled() {
    return true;
  }
  
  public boolean getToolbarButtonDeleteFolderDisabled() {
    return true;
  }
  
  void cacheFileNode(FileNode node) {
    LOG.debug("cacheFileNode("+node+")");
    Map cache = getFileNodeCache();
    if (cache==null) {
      LOG.debug("create TreeNodeCache");
      cache = new HashMap();
      setFileNodeCache(cache);
    }
    cache.put(node.getID(), node);
  }
  
  FileNode getCachedFileNode(int id) {
    FileNode result = null;
    Map cache = getFileNodeCache();
    if (cache!=null) {
      result = (FileNode) cache.get(id);
    }
    LOG.debug("getCachedFileNode("+id+") : "+result);
    return result;
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////// ITreeStateListener ///////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public void treeStateChanged(TreeStateEvent objEvent) {
    int eventType = objEvent.getEventType();
    switch (eventType) {
      case TreeStateEvent.SELECTED_NODE_CHANGED:
        LOG.debug("treeStateChanged: SELECTED_NODE_CHANGED");
        break;
        
      case TreeStateEvent.NODE_COLLAPSED:
        LOG.debug("treeStateChanged: NODE_COLLAPSED");
        break;
        
      case TreeStateEvent.NODE_EXPANDED:
        LOG.debug("treeStateChanged: NODE_EXPANDED");
        break;
        
      default:
        LOG.debug("treeStateChanged: UNKNOWN");
        break;
    }
    //LOG.debug("treeStateChanged: " + objEvent);
    
    TreeView treeView = (TreeView)getComponent("treeView");
    if(treeView != null){
      ITreeDataModel thisTreeDataModel = treeView.getTreeModel().getTreeDataModel();
      //LOG.debug("treeStateChanged: "+"selectedNodeID:" + objEvent.getNodeUID());
      
      Object selectedNode = thisTreeDataModel.getObject(objEvent.getNodeUID());
      LOG.debug("treeStateChanged: "+"selectedNode:" + selectedNode);

    } else {
      LOG.debug(">>>>> This shouldn't happen");
    }
    
    ITreeStateModel stateModel = getDocsTreeModel().getTreeStateModel();
    LOG.debug("treeStateChanged: "+"ExpandSelection: "+stateModel.getExpandSelection());
  }

  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////// PageBeginRenderListener /////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public void pageBeginRender(PageEvent event) {
    //getTreeDataModel().setDocsBrowser(this);
    FileNode fn = getSelectedFolder();
    DocsTableModel dtm = getDocsTableModel();
    dtm.setSelectedFolder(fn);
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////////// PageValidateListener //////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public void pageValidate(PageEvent event) {
    // this is to fix the case when the session is serialized/deserialized (server restart)
    getTreeDataModel().setDocsBrowser(this);
    getDocsTableModel().setDocsBrowser(this);
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public void selectFile(IRequestCycle cycle, int id) {
    LOG.debug("selectFile() id="+id+' ');
    downloadAction(cycle, id);
  }
  
  private void downloadAction(IRequestCycle cycle, int id) {
    String _method = "downloadAction(id="+id+")";
    LOG.debug(_method);
    
    String path = "D:\\fmi\\diploma\\demo\\apache-tomcat-5.5.16\\webapps\\weprom\\images\\TreeClosed.gif";
    String mimetype = "application/octet-stream";
    
    try {
      HttpServletResponse response = cycle.getRequestContext().getResponse();
      byte[] data = new byte[1024];
      
      LOG.debug(_method+" file: " + path);
      java.io.File file = new java.io.File(path);
      long size = file.length();
      if (file.exists()) {
        FileInputStream in = new FileInputStream(file); //document.getFileInputstream();
  
        response.setHeader("Content-disposition", "inline; filename=" + file.getName());
        response.setContentType(mimetype);
        response.setContentLength((int)size);
        ServletOutputStream out = response.getOutputStream();
  
        while (in.read(data) > -1) {
          out.write(data);
        }
        in.close();
        response.flushBuffer();
      } else {
        LOG.error(_method+" file \""+path+"\" does not exists.");
      }
    } catch (IOException e) {
      LOG.error(_method+"FAILED", e);
    }
  }
  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public void newFolder(IRequestCycle cycle) {
    LOG.debug("newFolder");
  }
  
  public void deleteFolder(IRequestCycle cycle) {
    LOG.debug("deleteFolder");
  }
  
  private Visit getVisit() {
    return (Visit) getPage().getVisit();
  }
  
  public ITreeModel createTreeModel() {
    Visit visit = getVisit();
    visit.getWeprom();
    DocsTreeDataModel dataModel = new DocsTreeDataModel(visit.getWeprom());
    
    ITreeModel treeModel = new SimpleTreeModel(dataModel);
    //dataModel.setStateModel(treeModel.getTreeStateModel());
    dataModel.setDocsBrowser(this);
    return treeModel;
  }
  
  private void createFile(IUploadFile uploadFile, File folder) {
    String _method = "createFile "; 
    LOG.debug(_method);
    File file = folder.createChild(File.TYPE_FILE);
    String fileName = uploadFile.getFileName(); 
    file.setName(fileName);
    file.setSize(uploadFile.getSize());
    try {
      file.save();
      int fileID = file.getID();
      copyFile(uploadFile.getStream(), fileID);
      LOG.debug(_method+"file "+file+" in " + folder + " ... OK");
    } catch (WEPROMException e) {
      LOG.debug(_method+"FAILED",e);
    } catch (IOException e) {
      LOG.debug(_method+"FAILED",e);
    }
  }
  
  //private static String DOCS_DESTINATION_FOLDER = "D:/fmi/apache-tomcat-5.5.16/webapps/weprom/docs";
  private static String DOCS_DESTINATION_FOLDER = "../webapps/weprom/docs";
  private static void copyFile(InputStream in, int id) throws IOException {
    String _method = "copyFile "; 
    LOG.debug(_method);
    FileOutputStream out = new FileOutputStream(DOCS_DESTINATION_FOLDER+java.io.File.separator+id);
    byte buf[] = new byte[4096];
    while (true) {
      int read = in.read(buf);
      if (read!=-1) {
        out.write(buf, 0, read);
      } else {
        break;
      }
    }
    out.flush();
    out.close();
    in.close();
  }  
}
