package workspace.web.components.docsbrowser;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.engine.state.ApplicationStateManager;

import workspace.Visit;

import com.fmi.weprom.api.AccessRights;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.File;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.UserWorkspace;
import com.fmi.weprom.api.WEPROMSystem;
import com.fmi.weprom.api.Workspace;
import com.nordija.tapestry.bayeux.downloads.DownloadResource;
import com.nordija.tapestry.bayeux.downloads.DownloadResourceSource;

public class DocsDownloadResourceSource implements DownloadResourceSource {

  private static final Log LOG = LogFactory.getLog(DocsDownloadResourceSource.class);
  static final String VISIT_KEY = "visit";

  
  public DownloadResource getDownloadResource(IRequestCycle cycle, String s) throws IOException {
    String _method = "getDownloadResource() : "+s+' ';
    LOG.debug(_method);
    ApplicationStateManager appStateManager = (ApplicationStateManager) cycle.getInfrastructure().getApplicationStateManager();
    Visit visit = (Visit) appStateManager.get(VISIT_KEY);
    if (visit!=null) {
      User user = visit.getUser();
      LOG.debug(_method+" user: "+user);
      
      int fileID = -1;
      try {
        fileID = Integer.parseInt(s);
      } catch (NumberFormatException e) {
        LOG.error(_method+"wrong file id: " + s, e);
        return null;
      }
      
      WEPROMSystem weprom = visit.getWeprom(); 
      File file = weprom.getFile(fileID);
      LOG.debug(_method+"file: " + file);
      if (file!=null) {
        String fileName = file.getName();
        Workspace root = file.getRootWorkspace();
        LOG.debug(_method+"root: " + root);
        String permissionName = null;
        if (root!=null) {
          if (root instanceof Project) {
            permissionName = Permission.VIEW_PROJECT_DOC;
          } else if (root instanceof Community) {
            permissionName = Permission.VIEW_COMMUNITY_DOC;
          } else if (root instanceof UserWorkspace) {
            permissionName = Permission.VIEW_DOC;
          }
        }
        LOG.debug(_method+"permissionName: " + permissionName);
        if (permissionName!=null) {
          AccessRights ar = visit.getWeprom().getAccessRights();
          Permission p = ar.getPermission(permissionName);
          boolean hasPermission = ar.checkPermission(user, root.getID(), p);
          LOG.debug(_method+"hasPermission: " + hasPermission);
          
          if (hasPermission) {
            return new DocsDownloadResource(fileID, fileName);
          }        
        }
      } else {
        LOG.error("no file with id="+fileID);
      }
    } else {
      LOG.debug("visit is null");
    }
    return null;
  }



}
