package workspace.web.components.docsbrowser;

import java.io.Serializable;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.contrib.tree.model.ITreeDataModel;
import org.apache.tapestry.contrib.tree.model.ITreeStateModel;

import com.fmi.weprom.api.File;
import com.fmi.weprom.api.WEPROMSystem;

public class DocsTreeDataModel implements ITreeDataModel, Serializable {
  
  private static final Log LOG = LogFactory.getLog(DocsTreeDataModel.class);
  private WEPROMSystem weprom; // ! should be avoided
  private transient DocsBrowser component;
  
  
  public DocsTreeDataModel(WEPROMSystem weprom) {
    //this.rootID = root.getID();
    this.weprom = weprom;
  }
  
  public Object getRoot() {
    LOG.debug("getRoot()");
    int rootID = component.getRoot().getID();
    return getFileNode(rootID);
  }

  public int getChildCount(Object objParent) {
    String _method = "getChildCount("+objParent+") "; 
    LOG.debug(_method);
    FileNode node = (FileNode) objParent;
    int count = 0;
    if (getStateModel().isUniqueKeyExpanded(node.getID())) {
      count = node.getChildCount();  
    } else {
      count = node._getChildCount();
    }
    LOG.debug(_method+": " + count);
    return count;
  }

  public Iterator getChildren(Object objParent) {
    LOG.debug("getChildren("+objParent+")");
    FileNode node = (FileNode) objParent;
    return node.getChildren().iterator();
  }

  public Object getObject(Object objUniqueKey) {
    String _method = "getObject("+objUniqueKey+") "; 
    LOG.debug(_method);
    Integer id = (Integer) objUniqueKey;
    //File file = weprom.getFile(id);
    FileNode fileNode = getFileNode(id);
    LOG.debug(_method+": " + fileNode);
    return fileNode;
  }

  public Object getUniqueKey(Object objTarget, Object objParentUniqueKey) {
    //String _method = "getUniqueKey("+objTarget+", "+objParentUniqueKey+") ";
    //LOG.debug(_method);
    FileNode node = (FileNode) objTarget;
    Object key = node.getID();
    //LOG.debug(_method+": " + key);
    return key;
  }

  public Object getParentUniqueKey(Object objChildUniqueKey) {
    LOG.debug("getParentUniqueKey("+objChildUniqueKey+")");
    Integer childID = (Integer) objChildUniqueKey;
    return weprom.getFile(childID).getParentID();
  }

  public boolean isAncestorOf(Object objChildUniqueKey, Object objParentUniqueKey) {
    LOG.debug("isAncestorOf("+objChildUniqueKey+", "+objParentUniqueKey+")");
    Integer childID = (Integer) objChildUniqueKey;
    Integer parentID = (Integer) objParentUniqueKey;
    return weprom.getFile(childID).getParentID()==parentID.intValue();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////////////////////////
  
  File getFile(int id) {
    LOG.debug("getFile("+id+")");
    return weprom.getFile(id);
  }
  
  ITreeStateModel getStateModel() {
    return component.getTreeModel().getTreeStateModel();
  }
  
  FileNode getFileNode(int id) {
    FileNode result = component.getCachedFileNode(id);
    if (result==null) {
      result = new FileNode(id, this);
      component.cacheFileNode(result);
    }
    return result;
    //return new FileNode(id, this);
  }

  public void setDocsBrowser(DocsBrowser browser) {
    this.component = browser;
  }
}
