package workspace.web.pages;

import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.IRequestCycle;

import workspace.Visit;

public abstract class RegisterPage extends BasePage {
  public abstract String getUsername();

  public abstract String getPassword();

  public abstract void setPassword(String pass);

  public abstract String getRetypePassword();

  public abstract void setRetypePassword(String retypePass);

  public abstract String getLastName();

  public abstract String getFirstName();

  public abstract String getEmail();

  public abstract String getPhone();

  public abstract String getComment();

  public abstract void setMessage(String message);

  //Component's synthetic properties
  public abstract void setResult(boolean result);

  //Component's persistent properties
  public abstract String getResultString();

  public abstract void setResultString(String result);

  public void finishLoad() {
    Visit visit = (Visit) getPage().getVisit();
    visit.clearSessionCache();
  }

  public void register(IRequestCycle cycle) {
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if(delegate.getHasErrors())
      return;
    if (!validPass(delegate) || !validEmail(delegate))
      return;
    else {
      if (isValidRegister(getUsername(), getPassword(), getRetypePassword(),
          getLastName(), getFirstName(), getEmail(), getPhone(), getComment())) {
        String resultString = "Congratulations! You have been succesfully registered. You could log in.";
        setResultString(resultString);
        setResult(true);
        return;
      } else {
        error(delegate, "inputUsername", "Username " + getUsername()
            + " is taken. Please, try another username.",
            ValidationConstraint.CONSISTENCY);
        return;
      }
    }
  }

  private boolean isValidRegister(String username, String password,
      String retypePassword, String lastName, String firstName, String email,
      String phone, String comment) {
    Visit visit = (Visit) getPage().getVisit();
    return visit.register(username, password, lastName, firstName, email,
        phone, comment);
  }

  // Validation
  private boolean validPass(IValidationDelegate delegate) {
    if (!getPassword().equals(getRetypePassword())) {
      setPassword("");
      setRetypePassword("");
      error(delegate, "retypePassword",
          "Password mismatch! Type and retype password again, please!",
          ValidationConstraint.CONSISTENCY);
      setPassword("");
      setRetypePassword("");
      return false;
    }
    return true;
  }

  private boolean validEmail(IValidationDelegate delegate) {
    IFormComponent inputEmail = (IFormComponent) getComponent("inputEmail");
    delegate.setFormComponent(inputEmail);
    if (delegate.isInError())
      return false;
    if (getEmail() == null) {
      error(delegate, "inputEmail", "Please, enter your email!",
          ValidationConstraint.CONSISTENCY);
      return false;
    }
    return true;
  }

  private void error(IValidationDelegate delegate, String componentId,
      String message, ValidationConstraint constraint) {
    IFormComponent component = (IFormComponent) getComponent(componentId);
    delegate.setFormComponent(component);
    delegate.record(message, constraint);
  }

  public boolean isResult() {
    return getResultString() != null;
  }
}
