package com.fmi.weprom.impl.mysql;

import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.*;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.Contact;
import com.fmi.weprom.api.WEPROMException;
import com.fmi.weprom.util.WepromRequestCycleUtil;

public class ContactImpl implements Contact, Serializable {
  
  private static final Log LOG = LogFactory.getLog(ContactImpl.class);
  private static final String CACHE_NAME = "ContactsCache";
  
  private int id;
  private Integer _ID = null;
  private String firstName;
  private String name;
  private String email;
  private String phone; 
  private String comment;
  int workspace_id;
  
  
  public ContactImpl(int workspace_id) {
    this.workspace_id  = workspace_id;
    this.id = UNSAVED_ID;
  }
  
  private ContactImpl(int id, int workspace_id, String name, String lastname, String email, String phone, String info) {
    this.id = id;
    this.workspace_id  = workspace_id;
    this.name = lastname;
    this.firstName = name;
    this.email = email;
    this.phone = phone;
    this.comment = info;
  }

  public void save() throws WEPROMException {
    LOG.debug("save()");    
    if (id!=UNSAVED_ID) {
      throw new IllegalStateException("Contact already exists");
    }
    save(true);
  }
  
  public void update() throws WEPROMException {
    if (id==UNSAVED_ID) {
      throw new IllegalStateException("Contact is unsaved and could not be updated");
    }
    LOG.debug("update()");
    save(false);
  }

  private void save(boolean unsaved) throws WEPROMException {
    final String _method = "save("+unsaved+") "; 
    LOG.debug(_method);

    Connection conn = null;
    Statement stmt = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    int i = 0;
    try {
      conn = Util.currentSession().getConnection();
      if (unsaved) {
        pstmt = conn.prepareStatement("INSERT INTO "+CONTACTS_TABLE+" (workspace_id,name,lastname,email,phone,info) VALUES(?,?,?,?,?,?)", 
                                  PreparedStatement.RETURN_GENERATED_KEYS);
        pstmt.setInt(++i, workspace_id);
      } else {
        pstmt = conn.prepareStatement("UPDATE "+CONTACTS_TABLE+" SET name=?,lastname=?,email=?,phone=?,info=? WHERE id=?");        
      }
      pstmt.setString(++i, firstName);      
      pstmt.setString(++i, name);
      pstmt.setString(++i, email);      
      pstmt.setString(++i, phone);
      pstmt.setString(++i, comment);            
      if (!unsaved) pstmt.setInt(++i, id);
      
      pstmt.executeUpdate();

      if (unsaved) {
        // get autogenerated project id
        rs = pstmt.getGeneratedKeys();
        
        if (rs.next()) {
          this.id = rs.getInt(1);
          LOG.debug(_method + "generated id="+id);
          cache();
        } else {
            LOG.debug(_method + "FAILED for "+toString());
            throw new WEPROMException("Faild to create contact");
        }
      }
    } catch (SQLException e) {
      LOG.error(_method + "FAILED", e);      
      throw new WEPROMException("Faild to create reminder!");
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (stmt!=null) {
        try {
          stmt.close();
        } catch (SQLException ignore) {}
      }      
    }        
  }  
  
  public void delete() throws WEPROMException {
    
  }

  public String getComment() {
    return comment;
  }

  public void setComment(String comment) {
    this.comment = comment;
  }

  public String getEmail() {
    return email;
  }

  public void setEmail(String email) {
    this.email = email;
  }

  public String getFirstName() {
    return firstName;
  }

  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getPhone() {
    return phone;
  }

  public void setPhone(String phone) {
    this.phone = phone;
  }
  
  public int getID() {
    return id;
  }

  static List loadContacts(int workspace_id) {
    LOG.debug("loadContacts() "+ "workspace_id="+workspace_id);
    List result = null;
    
    Connection conn = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;    
    
    try {
      conn = Util.currentSession().getConnection();
      pstmt = conn.prepareStatement("SELECT id,name,lastname,email,phone,info FROM "+CONTACTS_TABLE+
                                     " WHERE workspace_id=?");
      pstmt.setInt(1, workspace_id);
      rs = pstmt.executeQuery();
      result = new ArrayList();
      while (rs.next()) {
        int i = 0;
        int id = rs.getInt(++i);
        String name = rs.getString(++i);
        String lastname = rs.getString(++i);
        String email = rs.getString(++i);
        String phone = rs.getString(++i);
        String info = rs.getString(++i);
        
        ContactImpl contact = getInstance(id, workspace_id, name, lastname, email, phone, info);
        result.add(contact);
      }
      
    } catch (SQLException e) {
      LOG.error("loadContacts() " + "FAILED", e);      
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }      
    return result;
  }
  
  private static ContactImpl load(int workspace_id, int id) {
    String _method = "load(workspace_id:"+workspace_id+", id:"+id+") ";
    LOG.debug(_method);
    
    Connection conn = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;    
    
    try {
      conn = Util.currentSession().getConnection();
      pstmt = conn.prepareStatement("SELECT name,lastname,email,phone,info FROM "+CONTACTS_TABLE+
                                     " WHERE id=?");
      pstmt.setInt(1, id);
      rs = pstmt.executeQuery();
      if (rs.next()) {
        int i = 0;        
        String name = rs.getString(++i);
        String lastname = rs.getString(++i);
        String email = rs.getString(++i);
        String phone = rs.getString(++i);
        String info = rs.getString(++i);
        
        ContactImpl contact = getInstance(id, workspace_id, name, lastname, email, phone, info);
        return contact;
      }
      
    } catch (SQLException e) {
      LOG.error(_method + "FAILED", e);      
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }      
    return null;
  }
  
  private static ContactImpl getInstance(int id, int workspace_id, String name, String lastname, String email, String phone, String info) {
    ContactImpl contact = getCached(id);
    
    if (contact==null) {
      contact = new ContactImpl(id, workspace_id, name, lastname, email, phone, info);
      LOG.debug("getInstance(...) create new");
      contact.cache();
    }  else {
      contact.setName(lastname);
      contact.setFirstName(name);
      contact.setEmail(email);
      contact.setPhone(phone);
      contact.setComment(info);
      LOG.debug("getInstance(...) update cached : " + contact);
    }
    
    return contact;
  }
  
  static ContactImpl getInstance(int workspace_id, int id) {
    ContactImpl contact = getCached(id);
    if (contact==null) {
      contact = load(workspace_id, id);
    }
    return contact;
  }
  
  private Integer get_ID() {
    if (id==UNSAVED_ID) return null;
    if (_ID==null) {
      _ID = new Integer(id);
    }
    return _ID;
  }
  
  private void cache() {
    LOG.debug("cache : " + toString());
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    cache.put(get_ID(), this);
  }
  
  private static ContactImpl getCached(int id) {
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    return (ContactImpl) cache.get(new Integer(id));
  }
  
  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append("Contact[").append(id).append(',').append(name).append(']');    
    return sb.toString();
  }
  
  private void writeObject(ObjectOutputStream oos) throws IOException {
    oos.defaultWriteObject();    
    LOG.debug("SERIALIZE: " + toString());    
  }
  
  private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
    ois.defaultReadObject();    
    LOG.debug("DESERIALIZE: " + toString());    
  }  
}
