package com.fmi.weprom.impl.mysql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Role;

public class DBInitializer {
  
  private static final Log LOG = LogFactory.getLog(DBInitializer.class);
  
  /*
  static final String ALTER[] = {
     "ALTER TABLE master ADD CONSTRAINT master_owner FOREIGN KEY(owner_id) REFERENCES users (id)"
    ,"ALTER TABLE workspaces ADD CONSTRAINT workspace_master FOREIGN KEY(id) REFERENCES master (id)"
    
  };
  */
  
  // master table
  //
  // access_type could have a default to "inherited" type integer constant 
  static final String CREATE_TABLE_MASTER = 
    "CREATE TABLE master (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "parent_id INTEGER, " +
      "type TINYINT NOT NULL, " +
      "name VARCHAR(255), " +
      "owner_id INTEGER, " +
      "access_type TINYINT, " +
      "creation_date DATETIME," +
      "size BIGINT, " +
      "PRIMARY KEY(id))";
  
  static final String DROP_TABLE_MASTER = "DROP TABLE master";

  // users table
  static final String CREATE_TABLE_USERS = 
    "CREATE TABLE users (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "nick VARCHAR(10) NOT NULL UNIQUE, " +
      "password VARCHAR(10) NOT NULL, " +
      "name VARCHAR(30), " +
      "lastname VARCHAR(30), " +
      "email VARCHAR(20), " +
      "phone VARCHAR(25), " +
      "info VARCHAR(256), " +
      "PRIMARY KEY(id))"; 

  static final String DROP_TABLE_USERS = "DROP TABLE users";
    
  // messages table  
  static final String CREATE_TABLE_MESSAGES = 
    "CREATE TABLE messages (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "receiver_id INTEGER NOT NULL, " +
      "sender_id INTEGER NOT NULL, " +
      "type TINYINT NOT NULL, " +
      "subject VARCHAR(128), " +
      "text VARCHAR(768), " +
      "date DATETIME, " +
      "new BOOL, " +     
      "replied BOOL, " +      
      "forwarded BOOL, " +
      "inbox BOOL, " +
      "PRIMARY KEY(id))";
    
  static final String DROP_TABLE_MESSAGES = "DROP TABLE messages";
  
  // tasks table
  //
  // could have a foreign key to master.id  
  static final String CREATE_TABLE_TASKS = 
    "CREATE TABLE tasks (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "project_id INTEGER NOT NULL, " +
      "owner_id INTEGER NOT NULL, " +
      "processor_id INTEGER, " +
      "assigned BOOL, " +
      "subject VARCHAR(128) NOT NULL, " +
      "description VARCHAR(768), " +
      "priority TINYINT, " +
      "status TINYINT, " +
      "completed TINYINT, " +
      "start_date DATE, " +
      "end_date DATE," +
      "PRIMARY KEY(id))";

  static final String DROP_TABLE_TASKS = "DROP TABLE tasks";
  
  // access_rights table
  //
  // could have a foreign keys:
  // id to master.id
  // user_id to users.id
  // role_id to ?
  static final String CREATE_TABLE_ACCESS_RIGHTS = 
    "CREATE TABLE access_rights (" +
      "id INTEGER NOT NULL, " +
      "user_id INTEGER NOT NULL, " +
      "role_id INTEGER NOT NULL)";
  
  static final String DROP_TABLE_ACCESS_RIGHTS = "DROP TABLE access_rights";
  
  // role_permissions table
  static final String CREATE_TABLE_ROLE_PERMISSIONS = 
    "CREATE TABLE role_permissions (" +
      "role_id INTEGER NOT NULL, " +
      "permission_id INTEGER NOT NULL)";
  
  static final String DROP_TABLE_ROLE_PERMISSIONS = "DROP TABLE role_permissions";

  // roles table
  static final String CREATE_TABLE_ROLES = 
    "CREATE TABLE roles (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "name VARCHAR(30) NOT NULL UNIQUE, " +
      "PRIMARY KEY(id))";
  
  static final String DROP_TABLE_ROLES = "DROP TABLE roles";  

  // permissions table
  static final String CREATE_TABLE_PERMISSIONS = 
    "CREATE TABLE permissions (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "name VARCHAR(30) NOT NULL UNIQUE, " +
      "PRIMARY KEY(id))";
  
  static final String DROP_TABLE_PERMISSIONS = "DROP TABLE permissions";    
  
  static final String CREATE_TABLE_REMINDERS = 
    "CREATE TABLE reminders (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "workspace_id INTEGER NOT NULL, " + 
      "date DATETIME," +
      "subject VARCHAR(40) NOT NULL, " +
      "description VARCHAR(254), " +
      "status TINYINT, " +      
      "PRIMARY KEY(id))";  
  
  static final String DROP_TABLE_REMINDERS = "DROP TABLE reminders";
  
  static final String CREATE_TABLE_WORKSPACES = 
    "CREATE TABLE workspaces (" +
      "id INTEGER NOT NULL, " +
      "description VARCHAR(254), " +
      "status TINYINT, " +
      "completed TINYINT, " +
      "start_date DATE, " +
      "end_date DATE)";
  
  static final String DROP_TABLE_WORKSPACES = "DROP TABLE workspaces";
  
  static final String CREATE_TABLE_CONTACTS = 
    "CREATE TABLE contacts (" +
      "id INTEGER NOT NULL AUTO_INCREMENT, " +
      "workspace_id INTEGER NOT NULL, " + 
      "name VARCHAR(30), " +
      "lastname VARCHAR(30), " +
      "email VARCHAR(64), " +
      "phone VARCHAR(25), " +
      "info VARCHAR(256), " +
      "PRIMARY KEY(id))";    
  
  static final String DROP_TABLE_CONTACTS = "DROP TABLE contacts";
  
  
  private static void createTables(Connection connection) {
    createTable(connection, CREATE_TABLE_MASTER, DROP_TABLE_MASTER);
    createTable(connection, CREATE_TABLE_USERS, DROP_TABLE_USERS);
    createTable(connection, CREATE_TABLE_MESSAGES, DROP_TABLE_MESSAGES);
    createTable(connection, CREATE_TABLE_TASKS, DROP_TABLE_TASKS);
    createTable(connection, CREATE_TABLE_ACCESS_RIGHTS, DROP_TABLE_ACCESS_RIGHTS);
    createTable(connection, CREATE_TABLE_ROLE_PERMISSIONS, DROP_TABLE_ROLE_PERMISSIONS);
    createTable(connection, CREATE_TABLE_ROLES, DROP_TABLE_ROLES);
    createTable(connection, CREATE_TABLE_PERMISSIONS, DROP_TABLE_PERMISSIONS);
    createTable(connection, CREATE_TABLE_REMINDERS, DROP_TABLE_REMINDERS);
    createTable(connection, CREATE_TABLE_WORKSPACES, DROP_TABLE_WORKSPACES);
    createTable(connection, CREATE_TABLE_CONTACTS, DROP_TABLE_CONTACTS);
/*
    System.out.println("\nALTERs:");

    int size = ALTER.length;
    for (int i = 0; i < size; i++) {
      String alter = ALTER[i];
      String _alter = "["+i+"] " + alter;
      System.out.println(_alter);
      try {
        Statement stmt = connection.createStatement();    
        try {
          stmt.executeUpdate(alter);
          //System.out.println(_alter+" ... OK");
        } catch (SQLException e) {
          System.out.println(_alter+" ... FAILED:");
          e.printStackTrace();
        }
        System.out.println();
      } catch (SQLException e) {
        e.printStackTrace();
      }            
      
    }
    System.out.println();
*/
    //loadUsers(connection);
  }
  
  static final String[] PERMISSIONS = {
      Permission.CREATE_REMINDER
    , Permission.DELETE_REMINDER
    , Permission.CREATE_FOLDER
    , Permission.CREATE_DOC
    , Permission.DELETE_FOLDER
    , Permission.DELETE_DOC
    , Permission.CREATE_MESSAGE
    , Permission.CREATE_CONTACT
    , Permission.UPDATE_CONTACT
    , Permission.UPDATE_REMINDER
    , Permission.CREATE_PROJECT_FOLDER
    , Permission.CREATE_PROJECT_DOC
    , Permission.DELETE_PROJECT_FOLDER
    , Permission.DELETE_PROJECT_DOC
    , Permission.UPDATE_PROJECT_TASK
    , Permission.VIEW_PROJECT
    , Permission.VIEW_MY_WORKSPACE
    , Permission.VIEW_CALENDAR
    //, Permission.VIEW_TASKS
    , Permission.VIEW_REMINDER
    //, Permission.VIEW_TASK
    , Permission.VIEW_DOC
    , Permission.VIEW_FOLDER
    , Permission.VIEW_PROJECT_CALENDAR
    , Permission.VIEW_PROJECT_TASKS
    , Permission.VIEW_PROJECT_REMINDER
    , Permission.VIEW_PROJECT_TASK
    , Permission.VIEW_PROJECT_DOC
    , Permission.VIEW_PROJECT_FOLDER
    , Permission.VIEW_CONTACT
    , Permission.VIEW_CONTACTS
    , Permission.CREATE_PROJECT
    , Permission.DELETE_PROJECT
    , Permission.PROJECT_USER_EDIT_ROLES
    , Permission.CREATE_PROJECT_REMINDER
    , Permission.CREATE_PROJECT_TASK
    , Permission.DELETE_PROJECT_REMINDER
    , Permission.DELETE_PROJECT_TASK
    , Permission.UPDATE_PROJECT_REMINDER
    , Permission.UPDATE_PROJECT
    , Permission.ASSIGN_TASKS
    ,Permission.VIEW_COMMUNITY
    ,Permission.CREATE_COMMUNITY
    ,Permission.CREATE_COMMUNITY_FOLDER
    ,Permission.DELETE_COMMUNITY_FOLDER
    ,Permission.VIEW_COMMUNITY_FOLDER
    ,Permission.CREATE_COMMUNITY_DOC
    ,Permission.DELETE_COMMUNITY_DOC
    ,Permission.VIEW_COMMUNITY_DOC
    ,Permission.VIEW_COMMUNITY_CALENDAR
    ,Permission.VIEW_COMMUNITY_REMINDER    
    ,Permission.UPDATE_COMMUNITY
    ,Permission.DELETE_COMMUNITY  
    ,Permission.COMMUNITY_USER_EDIT_ROLES
    ,Permission.CREATE_COMMUNITY_REMINDER
    ,Permission.UPDATE_COMMUNITY_REMINDER
    ,Permission.DELETE_COMMUNITY_REMINDER    
  };
  
  static final String[] ROLES = {
      Role.ADMINISTRATOR
    , Role.PROJECT_COORDINATOR
    , Role.PROJECT_PARTICIPANT
    , Role.COMMUNITY_COORDINATOR
    , Role.COMMUNITY_PARTICIPANT
    , Role.USER
    , Role.GUEST    
  };
  
  static final String [][] ROLE_PERMISSIONS = {
    {Role.USER}, {
       Permission.VIEW_CALENDAR
      ,Permission.VIEW_REMINDER
      ,Permission.CREATE_REMINDER
      ,Permission.DELETE_REMINDER
      ,Permission.UPDATE_REMINDER
      ,Permission.CREATE_FOLDER
      ,Permission.DELETE_FOLDER
      ,Permission.VIEW_FOLDER
      ,Permission.CREATE_DOC
      ,Permission.DELETE_DOC
      ,Permission.VIEW_DOC
      ,Permission.CREATE_CONTACT
      ,Permission.UPDATE_CONTACT
      ,Permission.VIEW_CONTACT
      ,Permission.VIEW_CONTACTS
      ,Permission.CREATE_MESSAGE
      ,Permission.VIEW_MY_WORKSPACE
      ,Permission.CREATE_PROJECT
      ,Permission.CREATE_COMMUNITY
    }
    ,
    {Role.PROJECT_PARTICIPANT}, {      
       Permission.VIEW_PROJECT      
      ,Permission.VIEW_PROJECT_FOLDER
      ,Permission.CREATE_PROJECT_FOLDER
      ,Permission.DELETE_PROJECT_FOLDER
      ,Permission.CREATE_PROJECT_DOC
      ,Permission.DELETE_PROJECT_DOC
      ,Permission.VIEW_PROJECT_DOC
      ,Permission.UPDATE_PROJECT_TASK
      ,Permission.VIEW_PROJECT_TASK
      ,Permission.VIEW_PROJECT_TASKS
      ,Permission.VIEW_PROJECT_REMINDER
      ,Permission.VIEW_PROJECT_CALENDAR
    }
    ,
    {Role.COMMUNITY_PARTICIPANT}, {      
       Permission.VIEW_COMMUNITY
      ,Permission.CREATE_COMMUNITY_FOLDER
      ,Permission.DELETE_COMMUNITY_FOLDER
      ,Permission.VIEW_COMMUNITY_FOLDER
      ,Permission.CREATE_COMMUNITY_DOC
      ,Permission.DELETE_COMMUNITY_DOC
      ,Permission.VIEW_COMMUNITY_DOC
      ,Permission.VIEW_COMMUNITY_CALENDAR
      ,Permission.VIEW_COMMUNITY_REMINDER
    }
    ,
    {Role.PROJECT_COORDINATOR}, {
       Permission.DELETE_PROJECT
      ,Permission.PROJECT_USER_EDIT_ROLES
      ,Permission.CREATE_PROJECT_REMINDER
      ,Permission.CREATE_PROJECT_TASK
      ,Permission.DELETE_PROJECT_REMINDER
      ,Permission.DELETE_PROJECT_TASK
      ,Permission.UPDATE_PROJECT_REMINDER
      ,Permission.UPDATE_PROJECT
      ,Permission.ASSIGN_TASKS
    }
    ,
    {Role.COMMUNITY_COORDINATOR}, {
       Permission.UPDATE_COMMUNITY
      ,Permission.DELETE_COMMUNITY  
      ,Permission.COMMUNITY_USER_EDIT_ROLES
      ,Permission.CREATE_COMMUNITY_REMINDER
      ,Permission.UPDATE_COMMUNITY_REMINDER
      ,Permission.DELETE_COMMUNITY_REMINDER
    }    
  };
  
  static final String[] ADMINISTRATOR_PERMISSIONS = {
    
  };
  static final String[] PROJECT_COORDINATOR = {
    
  };
  static final String[] USER = {
    
  };
  static final String[] GUEST = {
    
  };
  
  private static void loadUsers(Connection connection) {
    try {
      Statement stmt = connection.createStatement();
      String sql = "insert into users (nick, password, name, lastname, email) " +
          "VALUES('pinko', 'pinko', 'pinko', 'pinko', 'pinko@pinko.com')";
      stmt.executeUpdate(sql);
    } catch (SQLException e) {
      e.printStackTrace();
    } 
  }

  private static void createTable(Connection connection, String createSQL, String dropSQL) {
    try {
      System.out.print(createSQL.substring(0, createSQL.indexOf('(')) + " ... ");
      Statement stmt = connection.createStatement();    
      try {
        stmt.executeUpdate(createSQL);
        System.out.println("OK");
      } catch (SQLException e) {
        stmt.executeUpdate(dropSQL);
        stmt.executeUpdate(createSQL);
        System.out.println("OK (dropped first) ");
      }
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }
  
  private static Map loadPermissions(Connection connection) {
    Map map = new HashMap();
    try {
      System.out.print("load Permissions ... ");
      String sql = "INSERT INTO permissions (name) VALUES(?)";
      PreparedStatement pstmt = connection.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
      
      int size = PERMISSIONS.length;
      for (int i = 0; i < size; i++) {
        pstmt.setString(1, PERMISSIONS[i]);
        pstmt.executeUpdate();
        
        ResultSet rs = pstmt.getGeneratedKeys();
        if (rs.next()) {
          int id = rs.getInt(1);
          map.put(PERMISSIONS[i], new Integer(id));
        }
      }
      System.out.println("OK");
    } catch (SQLException e) {
      System.out.println("Error loading Permissions");
      e.printStackTrace();
    }
    return map;
  }
  
  private static Map loadRoles(Connection connection) {
    Map map = new HashMap();
    try {
      System.out.print("load Roles ... ");
      String sql = "INSERT INTO roles (name) VALUES(?)";
      PreparedStatement pstmt = connection.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
      
      int size = ROLES.length;
      for (int i = 0; i < size; i++) {
        pstmt.setString(1, ROLES[i]);
        pstmt.executeUpdate();
        
        ResultSet rs = pstmt.getGeneratedKeys();
        if (rs.next()) {
          int id = rs.getInt(1);
          map.put(ROLES[i], new Integer(id));
        }
      }
      System.out.println("OK");
    } catch (SQLException e) {
      System.out.println("Error loading Roles");
      e.printStackTrace();
    }
    return map;
  }
  
  private static void loadRolePermissions(Connection connection, Map roles, Map permissions) {
    try {
      System.out.print("load RolePermissions ... ");
      String sql = "INSERT INTO role_permissions (role_id, permission_id) VALUES(?,?)";
      PreparedStatement pstmt = connection.prepareStatement(sql);
      
      int size = ROLE_PERMISSIONS.length;
      for (int i = 0; i < size; i+=2) {
        String roleName = ROLE_PERMISSIONS[i][0];
        Integer roleID = (Integer) roles.get(roleName);
        pstmt.setString(1, roleID.toString());
        String rolePermissions[] = ROLE_PERMISSIONS[i+1];
        for (int j = 0; j < rolePermissions.length; j++) {
          String permissionName = rolePermissions[j];
          Integer permissionID = (Integer) permissions.get(permissionName);
          if (permissionID==null) {
            LOG.error("Error loading permission \""+permissionName+"\" for role \""+roleName+"\" : no permission id found");
          }
          pstmt.setString(2, permissionID.toString());
          pstmt.executeUpdate();
        }
      }
      System.out.println("OK");
    } catch (SQLException e) {
      System.out.println("Error loading RolePermissions");
      e.printStackTrace();
    }
  }
  
  public static void main(String arg[]) {
    Connection connection = null;
    long startTime = 0;
    long endTime = 0;
    try {
      // Load the JDBC driver
      Class.forName("com.mysql.jdbc.Driver");
      System.out.println("MySQL Driver loaded successfuly.");
      
      startTime = System.currentTimeMillis();
      // Establish a connection
      // jdbc:mysql://localhost:3306/javatest?autoReconnect=true
      // jdbc:mysql://host:3306/dbname?user=xxxxx&password=yyyy
      String url = "jdbc:mysql://localhost:3306/weprom";
      String user = "weprom";
      String password = "weprom123";
      System.out.print("Connecting to ["+url+"]... ");
      connection = DriverManager.getConnection(url, user, password);
      System.out.println("OK.\n");
     
      createTables(connection);
      Map permissions = loadPermissions(connection);
      Map roles = loadRoles(connection);
      loadRolePermissions(connection, roles, permissions);
     /* 
      // Create a statement
      Statement statement = connection.createStatement();
           
      // Execute a statement
      ResultSet resultSet = statement.executeQuery("select * from pet"); 
      
      // Iterate through the result and print the student names
      while (resultSet.next())
      System.out.println(resultSet.getString(1) + "\t" +
      resultSet.getString(2) + "\t" + resultSet.getString(3));
      */
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    } catch (SQLException e) {
      e.printStackTrace();
    } finally {
      // Close the connection
      try {
        connection.close();  
        endTime = System.currentTimeMillis();
      } catch (SQLException ignore){}
    }
    System.out.println("["+(endTime-startTime)+" ms]");
  }

}
