package com.fmi.weprom.impl.mysql;

import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.TASKS_TABLE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.UNSAVED_ID;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.Message;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Task;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.WEPROMException;
import com.fmi.weprom.util.WepromRequestCycleUtil;

public class TaskImpl implements Task, Serializable {
  
  private static final int INVALID_ID = -1;
  private static final Log LOG = LogFactory.getLog(TaskImpl.class);
  private static final String CACHE_NAME = "TaskCache";
  
  protected int id;
  protected String subject;
  
  protected String description;  
  protected int priority;
  protected int status;
  protected int completed;
  protected Date start_date;
  protected Date end_date;
  
  protected int owner_id = INVALID_ID;
  protected int project_id = INVALID_ID;
  
  private Integer _ID = null;
  private boolean assigned = false;
  private int processor_id = INVALID_ID;
  private int oldProcessor_id = INVALID_ID;
  
  
  private TaskImpl(int id, String subject, int owner_id, int project_id) {
    this.id = id;
    this.subject = subject;
    this.owner_id = owner_id;
    this.project_id = project_id;
  }
    
  TaskImpl(Project project, User owner) {
    this.project_id = project.getID();
    this.owner_id = owner.getID();
    this.id = UNSAVED_ID;
  }
  
  public int getID() {
    return id;
  }
  
  public void setId(int i) {
    this.id = i;
  }

  public void setProcessor(User user) {
    if (user!=null) {
      int previousProcessorID = processor_id; 
      processor_id = user.getID();
      if (previousProcessorID!=processor_id) {
        setAssigned(false);
      }
    } else {
      processor_id = INVALID_ID;
      setAssigned(false);
    }
  }
  
  private boolean isTaskSubjectFree() throws SQLException {
    Connection conn = Util.currentSession().getConnection();
    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();  
      StringBuffer sb = new StringBuffer();
      sb.append("SELECT * FROM "+TASKS_TABLE+" WHERE subject='"+subject+"' AND project_id='"+project_id+"'");
      rs = stmt.executeQuery(sb.toString());
      boolean result = !rs.next();
      LOG.debug("isTaskSubjectFree("+subject+"): "+result);
      return result;
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (stmt!=null) {
        try {
          stmt.close();
        } catch (SQLException ignore) {}
      }        
    }
  }
  
  public void save() throws WEPROMException {
    if (id!=UNSAVED_ID) {
      throw new IllegalStateException("Task already exists");
    }
    if (subject==null) {
      throw new IllegalStateException("Task does not have a subject");
    }
    LOG.debug("save(...) id="+id + " project.id="+project_id);

    Connection conn = null;
    //Statement stmt = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    boolean processorAssigned = processor_id!=INVALID_ID && processor_id!=oldProcessor_id;
    boolean transactionOK = false;
    try {
      conn = Util.currentSession().getConnection();
      //stmt = conn.createStatement();
      pstmt = conn.prepareStatement(
          "INSERT INTO "+TASKS_TABLE+
          " (project_id,owner_id,assigned,subject,description,priority,status,completed,start_date,end_date,processor_id) " +
          "VALUES(?,?,?,?,?,?,?,?,?,?,?)", 
          PreparedStatement.RETURN_GENERATED_KEYS);
      
      LOG.debug("save() BEGIN TRANSACTION: CREATE_TASK");
      // Disable auto commit
      conn.setAutoCommit(false);
      
      if (isTaskSubjectFree()) {      
        /*
        StringBuffer sb = new StringBuffer();
        sb.append("INSERT INTO").append(' ').append(TASKS_TABLE).append(' ');
        sb.append('(');
        sb.append("project_id");
        sb.append(',').append("owner_id");
        sb.append(',').append("assigned");
        sb.append(',').append("subject");
        if (description!=null) {
          sb.append(',').append("description");
        }
        sb.append(',').append("priority");
        sb.append(',').append("status");
        sb.append(',').append("completed");
        if (start_date!=null) {
          sb.append(',').append("start_date");
        }
        if (end_date!=null) {
          sb.append(',').append("end_date");
        }        
        sb.append(')');
        
        sb.append(' ').append("VALUES").append('(');
        sb.append('\'').append(project_id).append('\'');
        sb.append(',').append('\'').append(owner_id).append('\'');
        sb.append(',').append("FALSE");
        sb.append(',').append('\'').append(subject).append('\'');
        if (description!=null) {
          sb.append(',').append('\'').append(description).append('\'');
        }        
        sb.append(',').append(priority);
        sb.append(',').append(status);
        sb.append(',').append(completed);
        if (start_date!=null) {
          sb.append(',').append('\'').append(new java.sql.Date(start_date.getTime())).append('\'');
        }
        if (end_date!=null) {
          sb.append(',').append('\'').append(new java.sql.Date(end_date.getTime())).append('\'');
        }          
        sb.append(')');
        String sql = sb.toString();
        LOG.debug("save() sql="+sql);
        
        stmt.executeUpdate(sql, Statement.RETURN_GENERATED_KEYS);
        */
        int i = 0;
        pstmt.setInt(++i, project_id);
        pstmt.setInt(++i, owner_id);
        pstmt.setBoolean(++i, assigned);
        pstmt.setString(++i, subject);
        pstmt.setString(++i, description);
        pstmt.setInt(++i, priority);
        pstmt.setInt(++i, status);
        pstmt.setInt(++i, completed);
        
        if (start_date!=null) {
          pstmt.setDate(++i, new java.sql.Date(start_date.getTime()));
        } else {
          pstmt.setNull(++i, Types.DATE);
        }
        
        if (end_date!=null) {
          pstmt.setDate(++i, new java.sql.Date(end_date.getTime()));
        } else {
          pstmt.setNull(++i, Types.DATE);
        }
        
        if (processor_id!=INVALID_ID) {
          pstmt.setInt(++i, processor_id);        
        } else {
          pstmt.setNull(++i, Types.INTEGER);
        }
        
        pstmt.executeUpdate();
        
        
        // get autogenerated project id
        rs = pstmt.getGeneratedKeys();
        if (rs.next()) {
          int taksID = rs.getInt(1);
          this.id = taksID; 
          LOG.debug("save() generated id="+id);
          
          // send task assignement message to the processor
          if (processorAssigned) {
            sendTaskAssignementMessage();
          }
          
          // Commit updates
          conn.commit();
          transactionOK = true;
          oldProcessor_id = processor_id;
          LOG.debug("save() END TRANSACTION: CREATE_TASK");        
          cache();
        } else {
          LOG.debug("save() FAILED for "+toString());
          throw new WEPROMException("Faild to create task");
        }       
      }
    } catch (SQLException e) {
      LOG.error("save() FAILED", e);      
      throw new WEPROMException("Faild to create task! Task subject maybe already in use.");
    } finally {
      if (!transactionOK) {
        try {
          LOG.debug("save() ROLLBACK TRANSACTION: CREATE_TASK");
          conn.rollback();
        } catch (SQLException ignore) {}
      }
      try {
        conn.setAutoCommit(true);
      } catch (SQLException ignore) {}      
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }    
  }

  public void delete() throws WEPROMException {
    // TODO send an info message to the assigned user (if the task is assigned) : task xxx deleted 
    LOG.debug("delete() ");
    if (id==UNSAVED_ID) {
      throw new IllegalStateException("Task is unsaved and could not be deleted");
    }
    
    Connection conn = null;
    Statement stmt = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    try {
      conn = Util.currentSession().getConnection();
      pstmt = conn.prepareStatement("DELETE FROM "+TASKS_TABLE+" WHERE id=?");
      pstmt.setInt(1, id);
      pstmt.executeUpdate();
      id = UNSAVED_ID;
    } catch (SQLException e) {
      LOG.error("delete() " + "FAILED", e);      
      throw new WEPROMException("Faild to delete task");
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (stmt!=null) {
        try {
          stmt.close();
        } catch (SQLException ignore) {}
      }      
    } 
    
    uncache();
  }
  
  public void update() throws WEPROMException {
    LOG.debug("update() ");
    if (id==UNSAVED_ID) {
      throw new IllegalStateException("Could not update unsaved Task");
    }
    if (subject==null) {
      throw new IllegalStateException("Task does not have a subject");
    }
    
    boolean transactionOK = false;
    boolean processorAssigned = processor_id!=INVALID_ID && processor_id!=oldProcessor_id;
    
    Connection conn = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    int i = 0;
    try {
      conn = Util.currentSession().getConnection();
      
      LOG.debug("update() BEGIN TRANSACTION: UPDATE_TASK");
      // Disable auto commit
      conn.setAutoCommit(false);
      
      // update task 
      pstmt = conn.prepareStatement("UPDATE "+TASKS_TABLE+" SET completed=?,description=?,end_date=?,priority=?,start_date=?," +
                                      "status=?,subject=?,processor_id=?,assigned=? WHERE id=?");
      pstmt.setInt(++i, completed);
      pstmt.setString(++i, description);
      if (end_date!=null) {
        pstmt.setDate(++i, new java.sql.Date(end_date.getTime()));
      } else {
        pstmt.setNull(++i, Types.DATE);
      }
      pstmt.setInt(++i, priority);
      if (start_date!=null) {
        pstmt.setDate(++i, new java.sql.Date(start_date.getTime()));
      } else {
        pstmt.setNull(++i, Types.DATE);
      }
      pstmt.setInt(++i, status);
      pstmt.setString(++i, subject);
      if (processor_id!=INVALID_ID) {
        pstmt.setInt(++i, processor_id);        
      } else {
        pstmt.setNull(++i, Types.INTEGER);
      }
      pstmt.setBoolean(++i, assigned);
      
      pstmt.setInt(++i, id);
      pstmt.executeUpdate();
      
      // send task assignement message to the processor
      if (processorAssigned) {
        sendTaskAssignementMessage();
      }
      
      // Commit updates
      conn.commit();
      transactionOK = true;
      oldProcessor_id = processor_id;
      LOG.debug("update() END TRANSACTION: UPDATE_TASK");       
    } catch (SQLException e) {
      LOG.error("update() " + "FAILED", e);      
      throw new WEPROMException("Faild to update task");
    } finally {
      if (!transactionOK) {
        try {
          LOG.debug("update() ROLLBACK TRANSACTION: UPDATE_TASK");
          conn.rollback();
        } catch (SQLException ignore) {}
      }
      try {
        conn.setAutoCommit(true);
      } catch (SQLException ignore) {}       
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }        
  }
  
  private void sendTaskAssignementMessage() throws WEPROMException {
    LOG.debug("send task assignement message");
    MessageImpl msg = MessageImpl.createTaskAsignementMessage(getOwner().getUserWorkspace(), this);
    msg.send();
  }

  static List loadUserTasks(int user_id) {
    return loadTasks(INVALID_ID, user_id);
  }
  
  static List loadProjectTasks(int project_id) {
    return loadTasks(project_id, INVALID_ID);
  }

  public UserImpl getOwner() {
    if (owner_id!=INVALID_ID) {
      return UserImpl.getInstance(owner_id);
    }    
    return null;
  }

  public User getProcessor() {
    if (processor_id!=INVALID_ID) {
      return UserImpl.getInstance(processor_id);
    }    
    return null;
  }
  
  private static List loadTasks(int project_id, int user_id) {
    //int project_id = INVALID_ID;
    //int owner_id = INVALID_ID;
    boolean byProject = project_id!=INVALID_ID;
    if (byProject) {
      LOG.debug("loadTasks() for project_id : "+project_id);
    } else {
      LOG.debug("loadTasks() for user_id : "+user_id);
    }
    
    List result = null;
    Connection conn = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    try {
      conn = Util.currentSession().getConnection();
      String sql = null;
      if (byProject) {
        sql = "SELECT id,owner_id,processor_id,assigned,subject,description,priority,status,completed,start_date,end_date " +
                "FROM "+TASKS_TABLE+" WHERE project_id=?"; 
        pstmt = conn.prepareStatement(sql);
        pstmt.setInt(1, project_id);
      } else {
        sql = "SELECT id,project_id,processor_id,assigned,subject,description,priority,status,completed,start_date,end_date " +
                "FROM "+TASKS_TABLE+" WHERE owner_id=? OR (processor_id =? AND assigned=TRUE)";
        pstmt = conn.prepareStatement(sql);
        pstmt.setInt(1, user_id);
        pstmt.setInt(2, user_id);
      }      
      rs = pstmt.executeQuery();
      
      result = new ArrayList();
      while (rs.next()) {
        int id = rs.getInt(1);
        if (byProject) {
          user_id = rs.getInt(2);
        } else {
          project_id = rs.getInt(2);
        }
        int processor_id = rs.getInt(3);
        boolean assigned = rs.getBoolean(4);
        String subject = rs.getString(5);
        String description = rs.getString(6);
        int priority = rs.getInt(7);
        int status = rs.getInt(8);
        int completed = rs.getInt(9);
        Timestamp start_dateTimestamp = rs.getTimestamp(10);        
        Timestamp end_dateTimestamp = rs.getTimestamp(11);
        
        Date start_date = null;
        if (start_dateTimestamp!=null) {
          start_date = new Date(start_dateTimestamp.getTime());
        }
        Date end_date = null;
        if (end_dateTimestamp!=null) {
          end_date = new Date(end_dateTimestamp.getTime());
        }
        
        TaskImpl task = getInstance(
            id,
            project_id,
            user_id,
            processor_id,
            assigned,
            subject,
            description,
            priority,
            status,
            completed,
            start_date,        
            end_date);
            
        LOG.debug("loadTasks() add "+task);
        result.add(task);
      }
    } catch (SQLException e) {
      LOG.error("loadReminders() " + "FAILED", e);      
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }         
    return result;
  }  
  
  static TaskImpl getInstance(
      int id,
      int project_id,
      int owner_id,
      int processor_id,
      boolean assigned,
      String subject,
      String description,
      int priority,
      int status,
      int completed,
      Date start_date,        
      Date end_date) {
    
    boolean cached = true;
    TaskImpl task = getCached(id);
    if (task==null) {
      task = new TaskImpl(id, subject, owner_id, project_id);
      cached = false;
    }
    task.setProcessorID(processor_id);
    task.oldProcessor_id = processor_id; 
    task.setAssigned(assigned);
    task.setDescription(description);
    task.setPriority(priority);
    task.setStatus(status);
    task.setCompleted(completed);
    task.setStartDate(start_date);
    task.setEndDate(end_date);
    
    if (!cached) {
      task.cache();
    } else {
      LOG.debug("getInstance(...) loaded from cache : " + task);
    }
    return task;
  }
  
  static TaskImpl getInstance(int id) {
    TaskImpl task = getCached(id);
    if (task==null) {
      task = load(id);
    }
    return task;
  }
  
  private static TaskImpl load(int id) {
    String _method = "load(" + id + ") "; 
    LOG.debug(_method);
    
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    try {
      conn = Util.currentSession().getConnection();
      pstmt = conn.prepareStatement("SELECT project_id,owner_id,processor_id,assigned,subject,description,priority,status,completed,start_date,end_date " +
                "FROM "+TASKS_TABLE+" WHERE id=?"); 
      pstmt.setInt(1, id);
      rs = pstmt.executeQuery();
      if (rs.next()) {
        int i = 0;
        int project_id = rs.getInt(++i);
        int owner_id = rs.getInt(++i);
        int processor_id = rs.getInt(++i);
        boolean assigned = rs.getBoolean(++i);
        String subject = rs.getString(++i);
        String description = rs.getString(++i);
        int priority = rs.getInt(++i);
        int status = rs.getInt(++i);
        int completed = rs.getInt(++i);
        Timestamp start_dateTimestamp = rs.getTimestamp(++i);        
        Timestamp end_dateTimestamp = rs.getTimestamp(++i);
        
        Date start_date = null;
        if (start_dateTimestamp!=null) {
          start_date = new Date(start_dateTimestamp.getTime());
        }
        Date end_date = null;
        if (end_dateTimestamp!=null) {
          end_date = new Date(end_dateTimestamp.getTime());
        }
        
        TaskImpl task = getInstance(
            id,
            project_id,
            owner_id,
            processor_id,
            assigned,
            subject,
            description,
            priority,
            status,
            completed,
            start_date,        
            end_date);
        return task;
      }
    } catch (SQLException e) {
      LOG.error(_method+"FAILED", e);
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }
    return null;    
  }
  

  public void setAssigned(boolean assigned) {
    this.assigned = assigned;
  }
  
  public boolean isAssigned() {
    return assigned;
  }

  private void setProcessorID(int processor_id) {
    this.processor_id = processor_id;
  }

  private Integer get_ID() {
    if (id==UNSAVED_ID) return null;
    if (_ID==null) {
      _ID = new Integer(id);
    }
    return _ID;
  }
  
  private void uncache() {
    LOG.debug("uncache: " + toString());
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    cache.remove(get_ID());
  }
  
  private void cache() {
    LOG.debug("cache : " + toString());
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    cache.put(get_ID(), this);
  }
  
  private static TaskImpl getCached(int id) {
    //LOG.debug("getCached id=" + id);
    Map cache = Util.currentSession().getCache(CACHE_NAME);
    return (TaskImpl) cache.get(new Integer(id));
  }
  
  public int getCompleted() {
    return completed;
  }

  public void setCompleted(int completed) {
    this.completed = completed;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public Date getEndDate() {
    return end_date;
  }

  public void setEndDate(Date end_date) {
    this.end_date = end_date;
  }

  public int getPriority() {
    return priority;
  }

  public void setPriority(int priority) {
    this.priority = priority;
  }

  public Date getStartDate() {
    return start_date;
  }

  public void setStartDate(Date start_date) {
    this.start_date = start_date;
  }

  public int getStatus() {
    return status;
  }

  public void setStatus(int status) {
    this.status = status;
  }

  public String getSubject() {
    return subject;
  }

  public void setSubject(String subject) {
    if (id!=UNSAVED_ID) {
      throw new IllegalStateException("Could not update task subject.");
    }
    this.subject = subject;
  }

  public int getId() {
    return id;
  }

  public ProjectImpl getProject() {
    if (project_id!=INVALID_ID) {
      return ProjectImpl.getInstance(project_id);
    }
    return null;
  }

  public String toString() {
   StringBuffer sb = new StringBuffer();
   sb.append("Task[").append(id).append(',').append(subject).append(']');
   return sb.toString();
  }
  
  private void writeObject(ObjectOutputStream oos) throws IOException {
    oos.defaultWriteObject();    
    LOG.debug("SERIALIZE: " + toString());    
  }
  
  private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
    ois.defaultReadObject();    
    LOG.debug("DESERIALIZE: " + toString());    
  }    
}
