package com.fmi.weprom.impl.mysql;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.File;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.WEPROMSystem;
import com.fmi.weprom.util.WepromRequestCycleUtil;

public class WEPROMSystemImpl implements WEPROMSystem, Serializable {
  
  private static final Log LOG = LogFactory.getLog(WEPROMSystemImpl.class);
  
  static final int UNSAVED_ID = -1;
  static final int INVALID_ID = -2;
  
  static final String USERS_TABLE = "users";
  static final String MASTER_TABLE = "master";
  static final String TASKS_TABLE = "tasks";
  static final String ACCESS_RIGHTS_TABLE = "access_rights";
  static final String ROLES_TABLE = "roles";
  static final String PERMISSIONS_TABLE = "permissions";
  static final String ROLE_PERMISSIONS_TABLE = "role_permissions";
  static final String REMINDERS_TABLE = "reminders";
  static final String WORKSPACES_TABLE = "workspaces";
  static final String MESSAGES_TABLE = "messages";
  static final String CONTACTS_TABLE = "contacts";
  
  
  static final int MASTER_TYPE_USERWORKSPACE = 1;
  static final int MASTER_TYPE_PROJECT = 2;
  static final int MASTER_TYPE_COMMUNITY = 3;
  static final int MASTER_TYPE_FOLDER = File.TYPE_FOLDER;
  
  static final int ACCESS_TYPE_INHERITED = 0; 
  static final int ACCESS_TYPE_PUBLIC = 1; 
  static final int ACCESS_TYPE_PRIVATE = 2; 
  static final int ACCESS_TYPE_SHARED = 3; 
  
  private transient AccessRightsImpl accessRights;
  
  
  public User getUser(int id) {
    return UserImpl.getInstance(id);
  }
  
  public User getUser(String nick) {
    return UserImpl.getInstance(nick);
  }

  public List getUsers() {
    return UserImpl.getUsers();
  }

  public User createUser(String nick) {
    LOG.debug("createUser("+nick+")");
    return new UserImpl(this, nick);
  }

  public Community createCommunity(User user) {
    LOG.debug("createCommunity(owner="+user.getNick()+")");
    CommunityImpl c = new CommunityImpl(user);
    return c;
  }

  public Project createProject(User user) {
    LOG.debug("createProject(owner="+user.getNick()+")");
    ProjectImpl p = new ProjectImpl(user);
    return p;
  }
  
  public List getProjects() {
    return ProjectImpl.loadProjects();
  }

  public synchronized AccessRightsImpl getAccessRights() {
    if (accessRights==null) {
      accessRights = new AccessRightsImpl();
    }
    return accessRights;
  }

  private void writeObject(ObjectOutputStream oos) throws IOException {
    oos.defaultWriteObject();    
    LOG.debug("SERIALIZE: " + toString());    
  }
  
  private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
    ois.defaultReadObject();    
    LOG.debug("DESERIALIZE: " + toString());    
  }

  public Project getProject(int id) {
    return ProjectImpl.getInstance(id);
  }

  public Community getCommunity(int id) {
    return CommunityImpl.getInstance(id);
  }

  public List getCommunities() {
    return CommunityImpl.loadCommunities();
  }

  public FileImpl getFile(int id) {
    return FileImpl.getInstance(id);
  }  
}
