package com.fmi.weprom.impl.mysql;

import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.ACCESS_RIGHTS_TABLE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TABLE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TYPE_COMMUNITY;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TYPE_PROJECT;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TYPE_USERWORKSPACE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.UNSAVED_ID;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.api.File;
import com.fmi.weprom.api.Workspace;

public abstract class WorkspaceImpl implements Workspace, Serializable {
  private static final Log LOG = LogFactory.getLog(WorkspaceImpl.class);
  protected int id;
  protected transient File rootFolder;
  

  public int getID() {
    return id;
  }
  
  public File getRootFolder() {
    if (rootFolder==null) {
      rootFolder = FileImpl.getInstance(id);
    }
    return rootFolder;
  }  
  
  static List getUsers(Log LOG, int workspace_id) {
    LOG.debug("getUsers() ");
    if (workspace_id==UNSAVED_ID) {
      throw new IllegalStateException("Workspace does not exists");
    }    
    List result = null;
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    try {
      conn = Util.currentSession().getConnection();
      pstmt = conn.prepareStatement("SELECT DISTINCT user_id FROM " + ACCESS_RIGHTS_TABLE + " WHERE id=?");
      pstmt.setInt(1, workspace_id);
      rs = pstmt.executeQuery();
      
      result = new ArrayList();
      while (rs.next()) {
        int user_id = rs.getInt(1);
        LOG.debug("getUsers() "+"user_id="+user_id);
        UserImpl user = UserImpl.getInstance(user_id);
        result.add(user);
      }
    } catch (SQLException e) {
      LOG.error("getUsers() " + "FAILED", e);
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }   
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }      
    return result;
  }  
  
  // helper method
  abstract int getType();
  
  abstract int getDefaultStatus();
/*
  protected transient WEPROMSystemImpl weprom;
  
  
  public WorkspaceImpl(WEPROMSystemImpl weprom) {
    this.weprom = weprom;  
  }
  
  public WEPROMSystemImpl getWEPROMSystem() {
    return weprom;
  }
  */
  
  static WorkspaceImpl getInstance(int id) {
    return loadWorkspace(id);
  }
  
  protected static WorkspaceImpl loadWorkspace(int id) {
    String _method = "load(" + id + ") "; 
    LOG.debug(_method);
    
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    try {
      conn = Util.currentSession().getConnection();
      pstmt = conn.prepareStatement("SELECT name, owner_id, type FROM "+MASTER_TABLE+ " WHERE id=?"); 
      pstmt.setInt(1, id);
      rs = pstmt.executeQuery();
      if (rs.next()) {
        int i = 0;
        String name = rs.getString(++i);
        int owner_id = rs.getInt(++i);
        int type = rs.getInt(++i);
        
        switch (type) {
          case MASTER_TYPE_USERWORKSPACE:
            UserImpl user = UserImpl.getInstance(owner_id);
            return user.getUserWorkspace();
          
          case MASTER_TYPE_COMMUNITY:
            return CommunityImpl.getInstance(owner_id, name, id);
          
          case MASTER_TYPE_PROJECT:
            return ProjectImpl.getInstance(owner_id, name, id);
        
        default:
          return null;
        }        
      }
    } catch (SQLException e) {
      LOG.error(_method+"FAILED", e);
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }
    return null;    
  }

}
