package com.fmi.weprom.impl.mysql;

import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.MASTER_TABLE;
import static com.fmi.weprom.impl.mysql.WEPROMSystemImpl.WORKSPACES_TABLE;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fmi.weprom.util.WepromRequestCycleUtil;

public class WorkspaceInfo {

  private static final Log LOG = LogFactory.getLog(WorkspaceInfo.class);

  protected WorkspaceImpl workspace;
  protected String description;
  protected int status;
  protected int completed;
  protected Date startDate;
  protected Date endDate;
  
  WorkspaceInfo(WorkspaceImpl workspace) {
    this.workspace = workspace;
    this.status = workspace.getDefaultStatus();
  }

  private WorkspaceInfo(WorkspaceImpl workspace, String description, int status, int completed, Date startDate, Date endDate) {
    this.workspace = workspace;
    this.description = description;
    this.status = status;
    this.completed = completed;
    this.startDate = startDate;
    this.endDate = endDate;
  }

  void save() throws SQLException {
    LOG.debug("save() for " + workspace);
    save(false);
  }
  
  void update() throws SQLException {
    LOG.debug("update() for " + workspace);
    save(true);
  }  
  
  private void save(boolean update) throws SQLException {
    Connection conn = Util.currentSession().getConnection();
    PreparedStatement pstmt = null;
    int i = 0;
    if (update) {
      pstmt = conn.prepareStatement("UPDATE "+WORKSPACES_TABLE+" SET description=?,status=?,completed=?,start_date=?,end_date=? " +
                                    " WHERE id=?");      
    } else {
      pstmt = conn.prepareStatement("INSERT INTO "+WORKSPACES_TABLE+" (id,description,status,completed,start_date,end_date)" +
                                    " VALUES(?,?,?,?,?,?)");
      pstmt.setInt(++i, workspace.getID());      
    }
    
    pstmt.setString(++i, description);
    pstmt.setInt(++i, status);
    pstmt.setInt(++i, completed);

    Timestamp startT = null;
    if (startDate!=null) {
      startT = new Timestamp(startDate.getTime());
    }
    pstmt.setTimestamp(++i, startT);
    
    Timestamp endT = null;
    if (endDate!=null) {
      endT = new Timestamp(endDate.getTime());
    }
    pstmt.setTimestamp(++i, endT);
    
    if (update) {
      pstmt.setInt(++i, workspace.getID());
    }
    
    pstmt.executeUpdate();
  }
  
  static WorkspaceInfo load(WorkspaceImpl workspace) {
    String _method = "load(" + workspace + ") "; 
    LOG.debug(_method);
    
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    try {
      conn = Util.currentSession().getConnection();
      pstmt = conn.prepareStatement("SELECT description,status,completed,start_date,end_date FROM "+WORKSPACES_TABLE+ " WHERE id=?"); 
      pstmt.setInt(1, workspace.getID());
      rs = pstmt.executeQuery();
      if (rs.next()) {
        int i = 0;
        String description = rs.getString(++i);
        int status = rs.getInt(++i);
        int completed = rs.getInt(++i);
        Timestamp startDate_timestamp = rs.getTimestamp(++i);
        Timestamp endDate_timestamp = rs.getTimestamp(++i);
        
        Date startDate = null;
        if (startDate_timestamp!=null) {
          startDate = new Date(startDate_timestamp.getTime()); 
        }
        
        Date endDate = null;
        if (endDate_timestamp!=null) {
          endDate = new Date(endDate_timestamp.getTime()); 
        }        
        
        WorkspaceInfo wi = new WorkspaceInfo(workspace, description, status, completed, startDate, endDate);
        return wi;
      }
    } catch (SQLException e) {
      LOG.error(_method+"FAILED", e);
    } finally {
      if (rs!=null) {
        try {
          rs.close();
        } catch (SQLException ignore) {}
      }
      if (pstmt!=null) {
        try {
          pstmt.close();
        } catch (SQLException ignore) {}
      }      
    }
    return null;    
  }

  public int getCompleted() {
    return completed;
  }


  public void setCompleted(int completed) {
    this.completed = completed;
  }


  public String getDescription() {
    return description;
  }


  public void setDescription(String description) {
    this.description = description;
  }


  public Date getEndDate() {
    return endDate;
  }


  public void setEndDate(Date endDate) {
    this.endDate = endDate;
  }


  public Date getStartDate() {
    return startDate;
  }


  public void setStartDate(Date startDate) {
    this.startDate = startDate;
  }


  public int getStatus() {
    return status;
  }


  public void setStatus(int status) {
    this.status = status;
  }

  private void writeObject(ObjectOutputStream oos) throws IOException {
    oos.defaultWriteObject();    
    LOG.debug("SERIALIZE: " + toString());    
  }
  
  private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
    ois.defaultReadObject();    
    LOG.debug("DESERIALIZE: " + toString());    
  }
  
}
