package workspace.web.components;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;
import org.apache.tapestry.valid.IValidationDelegate;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.form.IFormComponent;

import workspace.Visit;
import workspace.web.pages.CommunityListPage;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.User;

public abstract class CommunityEditor extends BaseComponent {

  
  // CommunityEditor mode constans
  public static final int MODE_VIEW_ALL = 0;
  public static final int MODE_UPDATE = 1;
  public static final int MODE_CREATE = 2;
  public static final int MODE_VIEW = 3;
  public static final int MODE_DELETE = 4;

  // CommunityEditor title constants
  public static String TITLE[] = { "Result", "Update Community",
      "Create Community", "View Community", "Delete Community" };

  // Result strings
  public static String SUCCESS[] = { "", "Your community has been updated!",
      "Your community has been created!", "Your community has been viewed!",
      "Your community has been deleted!" };

  public static String FAILURE1[] = { "",
      "Your community has not been updated! Please try again.",
      "Your community has not been created! Probably it was empty.",
      "Your community has not been viewed! Please try again.",
      "Your community has not been deleted! Please try again." };

  // Pairs of getters-setters for:
  // Component's parameters
  public abstract Community getCommunity();
  public abstract void setCommunity(Community community);
  public abstract int getMode();
  public abstract void setMode(int mode);

  // Component's persistent properties
  public abstract String getResultString();
  public abstract void setResultString(String result);

  // Component's synthetic properties
  public abstract void setResult(boolean result);

  // Listeners
  public void updateCommunity(IRequestCycle cycle) {
    Community community = getCommunity();
    System.out.println("updateCommunity: --->" + community.getID());
    setParamProp(MODE_UPDATE, community, false, "");
  }

  public void deleteCommunity(IRequestCycle cycle) {
    setMode(MODE_DELETE);
  }

  public void manageCommunity(IRequestCycle cycle) {
    System.out.println("Community List: updateCommunity: --->");
    IValidationDelegate delegate = (IValidationDelegate) getBeans().getBean(
        "delegate");
    if (delegate.getHasErrors()) {
      System.out.println("formUpdateCommunity - has errors");
    } else {
      Community community = getCommunity();
      System.out.println("formUpdateCommunity - no errors");
      System.out.println("community description: " + community.getDescription());
      switch (getMode()) {
      case MODE_CREATE:
        if(getVisit().communitySave(community))
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        else
          setResultString(FAILURE1[getMode()]);
        break;
      case MODE_UPDATE:
        System.out.println("manageCommunity UPDATE: --->" + community.getID());
        if(getVisit().communityUpdate(community))
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        else
          setResultString(FAILURE1[getMode()]);
        break;
      case MODE_DELETE:
        if(getVisit().communityDelete(community))
          setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        else
          setResultString(FAILURE1[getMode()]);
        break;
      case MODE_VIEW:
        setParamProp(MODE_VIEW_ALL, null, true, SUCCESS[getMode()]);
        CommunityListPage communityListPage = (CommunityListPage) cycle
            .getPage("CommunityListPage");
        cycle.activate(communityListPage);
        break;
      }
    }
    System.out.println("Community List: updateCommunity: --->");
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Parameters and properties order: mode, community, result, resultString
  private void setParamProp(int mode, Community community, boolean result,
      String resultString) {
    setResultString(resultString);
    setCommunity(community);
    setResult(result);
    setMode(mode);
  }

  // Api Methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public User getUser() {
    return (User) getVisit().getUser();
  }

  public Community getCurrCommunity() {
    return (Community) getVisit().getCommunity();
  }

  // Helper Methods for visualizing
  public boolean isResult() {
    return getMode() == MODE_VIEW_ALL;
  }

  public String getButtonName() {
    switch (getMode()) {
    case MODE_UPDATE:
      return "update";
    case MODE_CREATE:
      return "create";
    case MODE_VIEW:
      return "ok";
    case MODE_DELETE:
      return "delete";
    }
    return null;
  }

  public String getTitle() {
    int mode = getMode();
    String title = null;
    if (mode >= 0 && mode < 5) {
      title = TITLE[mode];
    } else
      title = "";
    return title;
  }

  public boolean getInputDisabled() {
    int mode = getMode();
    return (mode == MODE_VIEW || mode == MODE_DELETE);
  }

  public boolean getCommunityToolbarShow() {
    int mode = getMode();
    return (mode == MODE_UPDATE || mode == MODE_VIEW || mode == MODE_DELETE);
  }

  public boolean getCommunityToolbarButtonUpdateDsbl() {
    boolean hasPermission;
    if (getCommunity() != null)
      hasPermission = getVisit().checkUserPermission(Permission.UPDATE_COMMUNITY);
    else
      hasPermission = false;
    return getMode() != MODE_VIEW || !hasPermission;
  }

  public boolean getCommunityToolbarButtonDeleteDsbl() {
    boolean hasPermission;
    if (getCommunity() != null)
      hasPermission = getVisit().checkUserPermission(Permission.DELETE_COMMUNITY);
    else
      hasPermission = false;
    return getMode() != MODE_VIEW || !hasPermission;
  }

  public boolean getInCreateMode() {
    int mode = getMode();
    return (mode == MODE_CREATE);
  }

}
