package workspace.web.components;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.util.ComponentAddress;

import workspace.Visit;
import workspace.web.DateComparatorEEEddMMyyyy;
import workspace.web.PercentageComparator;
import workspace.web.pages.CommunityWorkspacePage;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.Contact;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.Workspace;
import com.fmi.weprom.api.Permission;

public abstract class CommunityList extends BaseComponent implements
    PageRenderListener {

  // Mode constants
  public static final int MODE_VIEW = 3;

  // Fields
  private IPrimaryKeyConvertor m_communityConvertor;

  public SimpleDateFormat dateFormat = null;

  // Pairs of getter-setter methods for the ContactList component's properties
  public abstract List getCommunityList();
  public abstract void setCommunityList(List communityList);

  // Managing the Community Convertor
  public CommunityList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_communityConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
        Community community = (Community) objValue;
        return community.getID() + "";
      }

      public Object getValue(Object objPrimaryKey) {
        String idStr = (String) objPrimaryKey + "";
        List list = getCommunityList();
        // Find the item
        int count = list.size();
        for (int i = 0; i < count; i++) {
          Community community = (Community) list.get(i);
          if (idStr.equals(community.getID() + "")) {
/*            System.out.println("IPrimaryKeyConvertor: " + idStr + "->"
                + community.getName());*/
            return community;
          }
        }
        return getVisit().getWeprom().createCommunity(getUser()); 
      }
    };
  }

  public IPrimaryKeyConvertor getCommunityConvertor() {
    return m_communityConvertor;
  }

  // Managing the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }

  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnId = new SimpleTableColumn("id",
          "", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((Community) objRow).getID();
            }
          }, false);
      // The following overrides the default SimpleTableColumn.getColumnValue to
      // render the Block component, defined on the HTML template
      columnId.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("idColumnValue"))));

      SimpleTableColumn columnName = new SimpleTableColumn("name", "Name",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getCommunityName(((Community) objRow));
            }
          }, true);

      // The following overrides ...
      columnName.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("nameColumnValue"))));

      SimpleTableColumn columnDescript = new SimpleTableColumn("desciption", "Description",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getCommunityDescription(((Community) objRow));
            }
          }, true);

      columns.put(columnId.getColumnName(), columnId);
      columns.put(columnName.getColumnName(), columnName);
      columns.put(columnDescript.getColumnName(), columnDescript);
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }
  }

  // Formatting the task list display
  public String getCommunityName(Community community) {
    String name = community.getName();
    if(name!=null)
      if (name.length() > 38) 
        return name.substring(0, 36) + "...";
      else
        return name;
    return "";
  }

  public String getCommunityDescription(Community community) {
    String name = community.getDescription();
    if(name!=null)
      if (name.length() > 62) 
        return name.substring(0, 60) + "...";
      else
        return name;
    return "";
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    System.out.println("Community List: pageBeginRender: --->");
    List communityList = getVisit().getUserCommunityList();
    setCommunityList(communityList);
    System.out.println("Community List: pageBeginRender: --->");
  }

  // Listeners
  public void enterCommunity(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters != null && parameters.length > 0) {
      int communityID = ((Integer) parameters[0]).intValue();
      Community community = getVisit().getWeprom().getCommunity(communityID);
      //Community community = (Community) parameters[0];
      //if (getVisit().checkUserPermission(Permission.VIEW_COMMUNITY))
        if (community != null) {

          getVisit().setCommunity(community);
          String userRole = getVisit().getUserRoleInCommunity();

          CommunityWorkspacePage communityWorkspacePage = (CommunityWorkspacePage) cycle
              .getPage("CommunityWorkspacePage");
          communityWorkspacePage.setCommunity(community);
          communityWorkspacePage.setUser(getUser());
          communityWorkspacePage.setRole(userRole);
          cycle.activate(communityWorkspacePage);
        } else
          System.out
              .println("CommunityWorkspacePage()cannot be activated: community= "
                  + community);
    } else
      System.out
          .println("CommunityWorkspacePage()cannot be activated: parameters= "
              + parameters);
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public User getUser() {
    return (User) getVisit().getUser();
  }

  public Workspace getCommunity() {
    return (Workspace) getVisit().getCommunity();
  }

}
