package workspace.web.components;

import java.util.List;
import java.util.Iterator;
import java.util.LinkedHashMap;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.util.ComponentAddress;

//import database.MySQLConnector;
import workspace.Visit;
import workspace.web.pages.ContactEditorPage;
import com.fmi.weprom.api.Contact;
import com.fmi.weprom.api.Permission;
import com.fmi.weprom.api.UserWorkspace;

public abstract class ContactList extends BaseComponent implements
    PageRenderListener {

  // Mode constants
  public static final int MODE_VIEW = 3;

  // Fields
  private IPrimaryKeyConvertor m_contactConvertor;

  // Pairs of getter-setter methods for the ContactList component's properties
  public abstract List getContactList();

  public abstract void setContactList(List contactList);

  // Managing the Contact Convertor
  public ContactList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_contactConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
        Contact contact = (Contact) objValue;
        System.out.println("IPrimaryKeyConvertor: " + contact.getName() + "->"
            + contact.getID());
        return contact.getID() + "";
      }

      public Object getValue(Object objPrimaryKey) {
        String prKey = (String) objPrimaryKey;
        List list = getContactList();
        // Find the item
        int count = list.size();
        for (int i = 0; i < count; i++) {
          Contact contact = (Contact) list.get(i);
          if (prKey.equals(contact.getID() + "")) {
            System.out.println("IPrimaryKeyConvertor: " + prKey + "->"
                + contact.getName());
            return contact;
          }
        }
        return getVisit().getUserWorkspace().createContact(); //Handle data item not found
      }
    };
  }

  public IPrimaryKeyConvertor getContactConvertor() {
    return m_contactConvertor;
  }

  // Managing the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }

  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnCard = new SimpleTableColumn("card", "",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return new Integer(((Contact) objRow).getID());
            }
          }, false);
      // The following overrides the default SimpleTableColumn.getColumnValue to
      // render the Block component, defined on the HTML template
      columnCard.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("cardColumnValue"))));

      SimpleTableColumn columnName = new SimpleTableColumn("name", "Name",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((Contact) objRow).getName();
            }
          }, true);

      // The following overrides ...
      columnName.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("nameColumnValue"))));

      ITableColumn columnFirstName = new SimpleTableColumn("firstName",
          "First Name", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((Contact) objRow).getFirstName();
            }
          }, true);

      ITableColumn columnEmail = new SimpleTableColumn("email", "Email",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((Contact) objRow).getEmail();
            }
          }, true);

      columns.put(columnCard.getColumnName(), columnCard);
      columns.put(columnName.getColumnName(), columnName);
      columns.put(columnFirstName.getColumnName(), columnFirstName);
      columns.put(columnEmail.getColumnName(), columnEmail);
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    System.out.println("Contact List: pageBeginRender: --->");
    List contactList = getVisit().getUserWorkspace().getContacts();
    setContactList(contactList);
    System.out.println("Contact List: pageBeginRender: --->");
  }

  // Listeners    
  public void viewContact(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters != null & parameters.length > 0) {
      int contactID = ((Integer) parameters[0]).intValue();
      Contact contact = getVisit().getUserWorkspace().getContact(contactID);
      //Contact contact = (Contact) parameters[0];
      if (contact != null) {
        ContactEditorPage contactEditorPage = (ContactEditorPage) cycle
            .getPage("ContactEditorPage");
        contactEditorPage.setContact(contact);
        contactEditorPage.setMode(MODE_VIEW);
        cycle.activate(contactEditorPage);
      } else
        System.out.println("ContactEditorPage()cannot be activated: contact= "
            + contact);
    } else
      System.out.println("ContactEditorPage()cannot be activated: parameters= "
          + parameters);
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  //Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }
}
