package workspace.web.components;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Locale;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.util.ComponentAddress;

//import database.MySQLConnector;
import workspace.Visit;
import workspace.MessageUtil;
import workspace.web.DateComparator;
import workspace.web.pages.MessageEditorPage;
import com.fmi.weprom.api.Message;
import com.fmi.weprom.api.Task;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.UserWorkspace;
import com.fmi.weprom.api.WEPROMException;

public abstract class MessageList extends BaseComponent implements
    PageRenderListener {

  // Mode constants
  public static final int MODE_VIEW = 3;
  public static final int OUTBOX = 0;
  public static final int INBOX = 1;

  // MessageList title constants
  public static String TITLE[] = { "Outbox", "Inbox" };

  // Fields
  private IPrimaryKeyConvertor m_messageConvertor;
  public SimpleDateFormat dateFormat = null;

  // Pairs of getter-setter methods for:
  // MessageList component's parameters
  public abstract int getBox();

  // MessageList component's properties
  public abstract List getMessageList();
  public abstract void setMessageList(List messageList);

  // Managing the Message Convertor
  public MessageList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_messageConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
        Message message = (Message) objValue;
        return message.getID() + "";
      }

      public Object getValue(Object objPrimaryKey) {
        String prKey = (String) objPrimaryKey;
        List list = getMessageList();
        // Find the item
        int count = list.size();
        for (int i = 0; i < count; i++) {
          Message message = (Message) list.get(i);
          if (prKey.equals(message.getID() + ""))
            return message;
        }
        Message msg = getVisit().getUserWorkspace().createMessage();
        msg.setSender(getUser());
        return msg; // Handle data item not found
      }
    };
  }

  public IPrimaryKeyConvertor getMessageConvertor() {
    return m_messageConvertor;
  }

  // Manage the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }

  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnStatus = new SimpleTableColumn("status", "",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return MessageUtil.getRplyFrwrdNewStatsDisp((Message) objRow,
                  getUser());
            }
          }, false);
      // The following overrides the default SimpleTableColumn.getColumnValue to
      // render the Block component, defined on the HTML template
      columnStatus.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("statusColumnValue"))));

      SimpleTableColumn columnSubject = new SimpleTableColumn("subject",
          "Subject", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return getMessageSubject((Message) objRow);
            }
          }, true);

      // The following overrides ...
      columnSubject.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("subjectColumnValue"))));

      SimpleTableColumn columnUser;
      if (getBox() == INBOX)
        columnUser = new SimpleTableColumn("sender", "Sender",
            new ITableColumnEvaluator() {
              public Object getColumnValue(ITableColumn objColumn, Object objRow) {
                return getUserDisplayId(((Message) objRow).getSender());
              }
            }, true);
      else
        columnUser = new SimpleTableColumn("sender", "Receiver",
            new ITableColumnEvaluator() {
              public Object getColumnValue(ITableColumn objColumn, Object objRow) {
                return getUserDisplayId(((Message) objRow).getReceiver());
              }
            }, true);

      columns.put(columnStatus.getColumnName(), columnStatus);
      columns.put(columnSubject.getColumnName(), columnSubject);
      columns.put(columnUser.getColumnName(), columnUser);
      columns.put(getMessageDateColumn().getColumnName(), getMessageDateColumn());
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }

  }

  public ITableColumn getMessageDateColumn() {
    SimpleTableColumn columnMessageDate = new SimpleTableColumn("messageDate",
        "Date", new ITableColumnEvaluator() {
          public Object getColumnValue(ITableColumn objColumn, Object objRow) {
            return getMessageDate((Message) objRow);
          }
        }, true);
    columnMessageDate.setColumnComparator(Visit.dtComparator);
    return columnMessageDate;
  }

  // Formatting the message list display
  public String getRplyFrwrdNewStatsDisp(int msgID) {
    Message message = getVisit().getUserWorkspace().getMessage(msgID);
    return MessageUtil.getRplyFrwrdNewStatsDisp(message, getUser());
  }

  public String getStatus(boolean status) {
    if (status)
      return "1";
    else
      return "0";
  }

  public String getMessageDate(Message message) {
    Date messageDate = message.getDate();
    if (messageDate != null) {
      return getDateFormat().format(messageDate);
    }
    return "";
  }

  public DateFormat getDateFormat() {
    if (dateFormat == null)
      dateFormat = new SimpleDateFormat("dd.MM.yyyy kk:mm", Locale.US);     
    return dateFormat;
  }

  public String getUserDisplayId(User user) {
    String displayId = user.getLastName() + " " + user.getName() + " [ "
        + user.getNick() + " ]";
    if (displayId.length() > 30) {
      return displayId.substring(0, 28) + "...";
    } else
      return displayId;
  }

  public String getMessageSubject(Message message) {
    String subject = message.getSubject();
    if (subject.length() > 38) {
      return subject.substring(0, 36) + "...";
    } else
      return subject;
  }

  public String getMessageSubject(int msgID) {
    Message message = getVisit().getUserWorkspace().getMessage(msgID);
    return getMessageSubject(message);
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    List messageList = (List) getMessageList();
    if (getBox() == INBOX)
      try {
        messageList = getUserWorkspace().getInboxMessages();
        setMessageList(messageList);
      } catch (WEPROMException e) {
      }
    else
      try {
        messageList = getUserWorkspace().getOutboxMessages();
        setMessageList(messageList);
      } catch (WEPROMException e) {
      }
  }

  // Listeners
  public void viewMessage(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters != null && parameters.length > 0) {
      int msgID = ((Integer) parameters[0]).intValue();
      Message message = getVisit().getUserWorkspace().getMessage(msgID);      
      //Message message = (Message) parameters[0];
      if (message != null) {
        MessageEditorPage messageEditorPage = (MessageEditorPage) cycle
            .getPage("MessageEditorPage");
        try {
          message.setNew(false);
          message.update();
        } catch (WEPROMException e) {
          System.out.println("MessageEditorPage(): message wasn't set read= "
              + message);
        }
        messageEditorPage.setMessage(message);
        messageEditorPage.setMode(MODE_VIEW);
        cycle.activate(messageEditorPage);
      } else
        System.out.println("MessageEditorPage()cannot be activated: message= "
            + message);
    } else
      System.out.println("MessageEditorPage()cannot be activated: parameters= "
          + parameters);
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  // Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public User getUser() {
    return (User) getVisit().getUser();
  }

  public UserWorkspace getUserWorkspace() {
    return (UserWorkspace) getVisit().getUserWorkspace();
  }

  // Helper methods for visualizing
  public String getTitle() {
    int box = getBox();
    String title = null;
    if (box >= 0 && box < 2)
      title = TITLE[box];
    else
      title = "Messages";
    return title;
  }

}
