package workspace.web.components;

import java.util.List;
import java.util.Iterator;
import java.util.LinkedHashMap;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.contrib.table.model.IPrimaryKeyConvertor;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.util.ComponentAddress;

//import database.MySQLConnector;
import workspace.Visit;
import workspace.web.pages.UserEditorPage;
import com.fmi.weprom.api.User;
import com.fmi.weprom.api.Project;
import com.fmi.weprom.api.Community;
import com.fmi.weprom.api.Role;

public abstract class UserList extends BaseComponent implements
    PageRenderListener {

  // Mode constants
  public static final int MODE_VIEW = 3;

  // Fields
  private IPrimaryKeyConvertor m_userConvertor;

  // Pairs of getter-setter methods for the UserList component's properties
  public abstract List getUserList();

  public abstract void setUserList(List userList);

  // Managing the User Convertor
  public UserList() {
    super();
    // Define a IPrimaryKeyConvertor that gets DataItems from the original list
    m_userConvertor = new IPrimaryKeyConvertor() {
      public Object getPrimaryKey(Object objValue) {
        User user = (User) objValue;
        System.out.println("IPrimaryKeyConvertor: " + user.getName() + "->"
            + user.getNick());
        return user.getNick();
      }

      public Object getValue(Object objPrimaryKey) {
        String prKey = (String) objPrimaryKey;
        List list = getUserList();
        // Find the item
        int count = list.size();
        for (int i = 0; i < count; i++) {
          User user = (User) list.get(i);
          if (user.getNick().equals(prKey)) {
            System.out.println("IPrimaryKeyConvertor: " + prKey + "->"
                + user.getName());
            return user;
          }
        }
        return getVisit().getWeprom().createUser("user"); //Handle data item not found
      }
    };
  }

  public IPrimaryKeyConvertor getUserConvertor() {
    return m_userConvertor;
  }

  // Managing the Column Model
  public ITableColumnModel getColumnModel() {
    return new ColumnModel();
  }

  class ColumnModel implements ITableColumnModel {
    private LinkedHashMap columns;

    public ColumnModel() {
      this.columns = new LinkedHashMap();

      SimpleTableColumn columnCard = new SimpleTableColumn("card", "",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              if(getProject()!=null)
                return getVisit().getUserRoleInProjectStr(((User) objRow));
              else if(getCommunity()!=null)
                return getVisit().getUserRoleInCommunityStr(((User) objRow));
              else
                return "";
            }
          }, false);
      // The following overrides the default SimpleTableColumn.getColumnValue to
      // render the Block component, defined on the HTML template
      columnCard.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("cardColumnValue"))));

      SimpleTableColumn columnUsername = new SimpleTableColumn("username",
          "Nick", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getNick();
            }
          }, true);

      // The following overrides ...
      columnUsername.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(getComponent("usernameColumnValue"))));

      ITableColumn columnRole = new SimpleTableColumn("role", "Role",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              if(getProject()!=null)
                return getVisit().getUserRoleInProjectStr(((User) objRow));
              else if(getCommunity()!=null)
                return getVisit().getUserRoleInCommunityStr(((User) objRow));
              else
                return "";
            }
          }, true);

      ITableColumn columnName = new SimpleTableColumn("name", "Name",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getLastName();
            }
          }, true);

      ITableColumn columnFirstName = new SimpleTableColumn("firstName",
          "First Name", new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getName();
            }
          }, true);

      ITableColumn columnEmail = new SimpleTableColumn("email", "Email",
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              return ((User) objRow).getEmail();
            }
          }, true);

      columns.put(columnCard.getColumnName(), columnCard);
      columns.put(columnUsername.getColumnName(), columnUsername);
      columns.put(columnRole.getColumnName(), columnRole);
      columns.put(columnName.getColumnName(), columnName);
      columns.put(columnFirstName.getColumnName(), columnFirstName);
      columns.put(columnEmail.getColumnName(), columnEmail);
    }

    public int getColumnCount() {
      return this.columns.size();
    }

    public ITableColumn getColumn(String strName) {
      return (ITableColumn) columns.get(strName);
    }

    public Iterator getColumns() {
      return columns.values().iterator();
    }
  }

  // Manage rendering
  public void pageBeginRender(PageEvent event) {
    List userList = (List) getUserList();
    System.out.println("User List: pageBeginRender: --->");
    if(getProject()!=null)
      userList = getVisit().getProjectUserList();
    else if(getCommunity()!=null)
      userList = getVisit().getCommunityUserList();
    setUserList(userList);
    System.out.println("User List: pageBeginRender: --->");
  }

  // Listeners    
  public void viewUser(IRequestCycle cycle) {
    Object[] parameters = cycle.getServiceParameters();
    if (parameters != null && parameters.length > 0) {
      int userID = ((Integer) parameters[0]).intValue();
      User user = getVisit().getWeprom().getUser(userID);
      //User user = (User) parameters[0];
      if (user != null) {
        UserEditorPage userEditorPage = (UserEditorPage) cycle
            .getPage("UserEditorPage");
        userEditorPage.setUser(user);
        userEditorPage.setMode(MODE_VIEW);
        cycle.activate(userEditorPage);
      } else
        System.out
            .println("UserEditorPage()cannot be activated: user= " + user);
    } else
      System.out.println("UserEditorPage()cannot be activated: parameters= "
          + parameters);
  }

  public void formSubmit(IRequestCycle cycle) {
  }

  //Api methods
  public Visit getVisit() {
    return (Visit) getPage().getVisit();
  }

  public Project getProject() {
    return (Project) getVisit().getProject();
  }

  public Community getCommunity() {
    return (Community) getVisit().getCommunity();
  }

  public String imgRole(String rolesStr) {
    System.out.println(rolesStr);
    String[] roles = rolesStr.split(", ");
    for (int i = 0; i < roles.length; i++)
      if (roles[i].equals(Role.PROJECT_COORDINATOR) ||
          roles[i].equals(Role.COMMUNITY_COORDINATOR))
        return "coordinator";
    return "user";
  }
}
