package workspace.web.components.docsbrowser;

import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.contrib.table.model.ITableColumn;
import org.apache.tapestry.contrib.table.model.ITableColumnModel;
import org.apache.tapestry.contrib.table.model.ITableModel;
import org.apache.tapestry.contrib.table.model.ITablePagingState;
import org.apache.tapestry.contrib.table.model.ITableSortingState;
import org.apache.tapestry.contrib.table.model.common.BlockTableRendererSource;
import org.apache.tapestry.contrib.table.model.simple.ITableColumnEvaluator;
import org.apache.tapestry.contrib.table.model.simple.SimpleTableColumn;
import org.apache.tapestry.util.ComponentAddress;

import com.fmi.weprom.api.File;

public class DocsTableModel implements 
  ITableModel, ITableColumnModel, ITableSortingState, ITablePagingState, Serializable {
  
  private static final Log LOG = LogFactory.getLog(DocsTableModel.class);
  
  private FileNode source;
  private int pageSize = 10;
  private int currentPage;
  
  private String sortColumn;
  private boolean sortOrder;
  
  private LinkedHashMap columns;
  private transient FileNode selectedFolder;
  private transient DocsBrowser component;
  private transient DateFormat dateFormat;
  
  
  DocsTableModel(FileNode source) {
    this.source = source;
    
    //initColumns();
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////// ITableModel //////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public ITableColumnModel getColumnModel() {
    //LOG.debug("ITableModel:getColumnModel()");
    return this;
  }

  public ITableSortingState getSortingState() {
    //LOG.debug("ITableModel:getSortingState()");
    return this;
  }

  public ITablePagingState getPagingState() {
    //LOG.debug("ITableModel:getPagingState()");
    return this;
  }

  public int getPageCount() {
    String _method = "ITableModel:getPageCount() "; 
    //LOG.debug(_method);
    int pageCount = 0;
    if (selectedFolder!=null) {
      int psize = getPageSize();
      int rowsCount = selectedFolder.getFilesCount();
      pageCount = rowsCount/psize;
      if (pageCount*psize<rowsCount) {
        pageCount++;
      }
      //LOG.debug(_method + " PageSize: "+ psize + " FilesCount: " +rowsCount + " PageCount: "+ pageCount);
    }
    return pageCount;
  }

  public Iterator getCurrentPageRows() {
    LOG.debug("ITableModel:getCurrentPageRows() selectedFolder="+selectedFolder);
    // select * from master where parent_id=17 and type=5 order by name desc limit 0,10;
    int pageSize = getPageSize();
    String sortColumn = getSortColumn();
    boolean sortOrder = getSortOrder(); 
    int currentPage = getCurrentPage();
    int offset = currentPage * pageSize;
    if (selectedFolder!=null) {
      return selectedFolder.getFiles(sortColumn, sortOrder, offset, pageSize).iterator();
    } else {
      return null;
    }
  }

  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////// ITableColumnModel ///////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////

  public int getColumnCount() {
    int size = _getColumns().size();
    LOG.debug("ITableColumnModel:getColumnCount() : " + size);
    return size;
  }

  public ITableColumn getColumn(String strName) {
    LOG.debug("ITableColumnModel:getColumn("+strName+")");
    return (ITableColumn) _getColumns().get(strName);
  }

  public Iterator getColumns() {
    LOG.debug("ITableColumnModel:getColumns()");
    return _getColumns().values().iterator();
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////// ITableSortingState ///////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public String getSortColumn() {
    //LOG.debug("ITableSortingState:getSortColumn() : " + sortColumn);
    return sortColumn;
  }

  public boolean getSortOrder() {
    //LOG.debug("ITableSortingState:getSortOrder() : " + sortOrder);
    return sortOrder;
  }

  public void setSortColumn(String strName, boolean bOrder) {
    LOG.debug("ITableSortingState:setSortColumn("+strName+","+bOrder+")");
    this.sortColumn = strName;
    this.sortOrder = bOrder;
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////// ITablePagingState ///////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  
  public int getPageSize() {
    //LOG.debug("ITablePagingState:getPageSize() : " + pageSize);
    return pageSize;
  }
  
  public void setPageSize(int nPageSize) {
    LOG.debug("ITablePagingState:setPageSize("+nPageSize+")");
    this.pageSize = nPageSize;
  }

  public int getCurrentPage() {
    //LOG.debug("ITablePagingState:getCurrentPage() : " + currentPage);
    return currentPage;
  }

  public void setCurrentPage(int nPage) {
    LOG.debug("ITablePagingState:setCurrentPage("+nPage+")");
    this.currentPage = nPage;
  }
  
  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////

  private HashMap _getColumns() {
    initColumns();
    return columns;
  }
  
  private void initColumns() {
    if (columns==null) {
      columns = new LinkedHashMap();
      SimpleTableColumn column = null;
      
      column = new SimpleTableColumn("name", "Name", 
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              //System.out.println("name:getColumnValue(objColumn="+objColumn+", objRow="+objRow+")");
              File fn = (File) objRow;
              return fn.getName();
            }    
          }
          , true);
      column.setValueRendererSource(new BlockTableRendererSource(
          new ComponentAddress(component.getComponent("nameColumnValue"))));    
      columns.put(column.getColumnName(), column);
      
      column = new SimpleTableColumn("date", "Date", 
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              //System.out.println("name:getColumnValue(objColumn="+objColumn+", objRow="+objRow+")");
              File fn = (File) objRow;
              Date date = fn.getDate();
              String dateStr = getDateFormat().format(date);
              return dateStr;
            }    
          }
          , true);
      columns.put(column.getColumnName(), column);
      
      column = new SimpleTableColumn("size", "Size", 
          new ITableColumnEvaluator() {
            public Object getColumnValue(ITableColumn objColumn, Object objRow) {
              //System.out.println("name:getColumnValue(objColumn="+objColumn+", objRow="+objRow+")");
              File fn = (File) objRow;
              long size = fn.getSize();
              return Long.toString(size);
            }    
          }
          , true);
      columns.put(column.getColumnName(), column);
      
    }
  }
  
  public void setSelectedFolder(FileNode fn) {
    LOG.debug("setSelectedFolder("+fn+")");
    this.selectedFolder = fn;
  }

  public void setDocsBrowser(DocsBrowser browser) {
    this.component = browser;
  }
  
  private DateFormat getDateFormat() {
    if (dateFormat==null) {
      dateFormat = new SimpleDateFormat("dd.MM.yyyy kk:mm", Locale.US);   
    }
    return dateFormat;
  }
}
