package workspace.web.components.docsbrowser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.contrib.tree.model.ITreeNode;

import com.fmi.weprom.api.File;
import com.fmi.weprom.api.WEPROMException;

public class FileNode implements ITreeNode {
  
  private static final Log LOG = LogFactory.getLog(FileNode.class);
  private DocsTreeDataModel dataModel;
  private int id;
  private transient String toString = null;
  FileNode parent = null;
  
  
  FileNode(int id, DocsTreeDataModel dataModel) {
    this.id = id;
    this.dataModel = dataModel;
    LOG.debug(" >>>>>>>>>>>>>>> NEW : " + id);
  }

  public List getFiles() {
    return dataModel.getFile(id).getFiles();
  }
  
  public Collection getChildren() {
    LOG.debug("getChildren() ");
    List result = null;
    List _folders = dataModel.getFile(id).getFolders();
    if (_folders!=null) {
      int size = _folders.size();
      result = new ArrayList(size);
      for (int i = size; --i>=0 ; ) {
        File file = (File) _folders.get(i);
        FileNode fileNode = dataModel.getFileNode(file.getID());
        fileNode.setParent(this);
        result.add(fileNode);
      }
    }
    LOG.debug("getChildren() "+": "+result);
    return result;
  }

  private void setParent(FileNode parent) {
    this.parent = parent;
  }

  public int getChildCount() {
    LOG.debug("getChildCount()");
    List folders = dataModel.getFile(id).getFolders();
    if (folders!=null) {
      return folders.size();
    }
    return 0;
  }
  
  public int getFilesCount() {
    try {
      return dataModel.getFile(id).getFilesCount();
    } catch (WEPROMException e) {
      LOG.error("getFilesCount()", e);
    }
    return 0;
  }
  
  int _getChildCount() {
    LOG.debug("_getChildCount()");
    File file = dataModel.getFile(id);
    try {
      return file.getFoldersCount();
    } catch (WEPROMException e) {
      LOG.error("_getChildCount FAILED", e);
    }
    return 0;
  }

  public ITreeNode getParent() {
    LOG.debug("getParent()");
    return parent;
  }

  public boolean containsChild(ITreeNode node) {
    LOG.debug("containsChild("+node+")");
    // TODO Auto-generated method stub
    return false;
  }

  public boolean getAllowsChildren() {
    LOG.debug("getAllowsChildren()");
    // TODO Auto-generated method stub
    return false;
  }

  public boolean isLeaf() {
    LOG.debug("isLeaf()");
    // TODO Auto-generated method stub
    return false;
  }

  public Object getID() {
    //LOG.debug("getID()");
    return id;
  }
  
  public String getName() {
    return dataModel.getFile(id).getName();
  }

  public String toString() {
    if (toString==null) {
      StringBuffer sb = new StringBuffer();
      sb.append("[").append(dataModel.getFile(id)).append(']');
      toString = sb.toString();
    }
    return toString;
  }
  
  public int hashCode() {
    return id;
  }

  public boolean equals(Object objTarget) {
    if(objTarget == this)
        return true;
    if(! (objTarget instanceof FileNode))
        return false;

    FileNode objTargetNode = (FileNode)objTarget;
    return id == objTargetNode.id;
  }

  public List getFiles(String sortColumn, boolean sortOrder, int offset, int pageSize) {
    return dataModel.getFile(id).getFiles(sortColumn, sortOrder, offset, pageSize);
  }   
}
